#ifndef CONTAINER_H
#define CONTAINER_H
//created March 2011

#include <map>
#include <vector>
#include "graph.h"
#include "utilities.h"


struct MapStats
{
	bool anchorsMatch;
	int setOverlap; 
	int mapOverlap;
	int minSize; 
	float evalutation; 
};

class Container 
{
public:
	Container(MyGraph * f, MyGraph * G, unsigned int anchorF, unsigned int anchorC, unsigned short i, unsigned int maxMappings, 
		unsigned int findNBest, char comp, float diminish, unsigned int method /*= 0*/, char distMethod,struct arguments args1);
	Container(MyGraph * f, MyGraph * G, unsigned int anchorF, unsigned int anchorC,struct arguments args1);
	Container(bool t,char distMethod,struct arguments args1);

	//determines if at least one node overlaps
	int setOverlap();
	float percentSetOverlap();
	float mapOverlap(std::map<unsigned int, unsigned int>& map);


	std::vector<std::pair<std::map<unsigned int, unsigned int>, double> > getMaps();

	double evaluate(std::map<unsigned int, unsigned int> mapping, char Method);

	~Container();

private:
	/*helper functions*/
	void backtrackMap(char Method);
	void malikMap(char Method);

	//sort the fragment nodes such that each successive has the most possible connections to the nodes before it
	void sortByBackEdges(std::vector<unsigned int>& fragNodes);

	//sorts the fragment nodes such that each node is either closer to x or if the same distance, is 
	void sortByDistDiverse(std::vector<unsigned int>& fragNodes, unsigned int start, unsigned int end);
	bool isCloserOrMoreDiverse(unsigned int a, unsigned int b);

	//fills in qi with IDs from container graph with similar i degrees
	void computeQ(unsigned int fragID, std::vector<unsigned int>& qi,char Method);
	void computeQ(unsigned int fragID, std::vector<unsigned int>& qi, unsigned int contParent,char Method);

	void greedyExpand(unsigned int level, std::vector<unsigned int>& sortedBack, std::vector <std::vector<unsigned int> >& Q, std::map<unsigned int, unsigned int>& partialMap,char Method);
	
	void backTrackExpand(unsigned int level,unsigned int maxBackLevel, std::vector<unsigned int>& sortedBack, 
		std::vector <std::vector<unsigned int> >& Q, std::map<unsigned int, unsigned int>& partialMap, std::vector<unsigned int>& parentIDs,char Method);
	
	void Map( std::vector<unsigned int>& sortedBack, std::vector <std::vector<unsigned int> >& Q, char Method);


	/*representation*/
	MyGraph * fragment;
	MyGraph * container;	//for now this is the induced subgraph from the large graph
	unsigned int fragAnchorID;
	unsigned int contAnchorID;
	unsigned int maxMaps;
	unsigned int nBest;
	unsigned short i_degree;		//how many i-degrees should be compared (0-3)
	char compare;
	std::vector<std::pair<std::map<unsigned int, unsigned int>, double> > fragToContainerIDs;	
	std::map<unsigned int, unsigned int> distances;
	unsigned int mapMethod; //0 - original backtracking; 1 - Malik Method
	bool test;
	float diminish;
	struct arguments args;
};

//is at least 1 ID in the fragment also found in the mapping?
bool test_criteria_overlap(const std::map<unsigned int, unsigned int> & fragToContIDs);
//is the center node (the first in the map) mapped to in the container?
bool test_criteria_containsCenter(const std::map<unsigned int, unsigned int> & fragToContIDs );
//is (frag intersect container) / (min (frag, container) >= percentage?
bool test_criteria_PercentOverlap(const std::map<unsigned int, unsigned int> & fragToContIDs, double percentage);
#endif
