#ifndef _GRAPH_H_
#define _GRAPH_H_

#include <list>
#include <limits.h>
#include <float.h>
#include <map>
#include <string>
#include <stdio.h>
#include <errno.h>
#include <vector>
#include <math.h>
#include <unordered_map>

#include <pthread.h>

#include <boost/config.hpp>
#include <boost/utility.hpp>             // for boost::tie
#include <boost/graph/graph_traits.hpp>  // for boost::graph_traits
#include <boost/graph/adjacency_list.hpp>

#include "stats_keeper.h"
#define ULLint (unsigned long long int)
class MyGraph;
class MyNode;
class iDeg;
//#define bagsType std::map<unsigned int, mappingType >
#define bagsType std::unordered_map<unsigned int, mappingType >
//#define mappingType std::map<unsigned int, unsigned int>
#define mappingType std::unordered_map<unsigned int, unsigned int>
#include "partitioning.h"
#define LABEL_LEN 8
//#define PARALLEL
#define mapIDNodes std::map<unsigned int, MyNode>

/** Anonymous Namespace with Defines **/
namespace {
	const unsigned int MANY_EDGES     (250);//statistics only
	const unsigned int FEW_EDGES      (0);//statistics//Min out neighbors for frag gen(3?)
	const unsigned int POPULAR        (0);//enum
	const unsigned int DIVERSE        (1);//enum
	const unsigned int CENTER_METHOD  (DIVERSE);
}

#define iDegreeList std::map<unsigned short, float>

struct positioningArgument
{
	int method;	// if method > 0 the methos used is the entire <method>-neighborhood (for example 1-Neighborhood) if method =-1 k-random walk is used  if method = 0 the i-degrees are read from the file
	unsigned int size;	//if needed the max number in a locality
	unsigned int k;		// if needed the number of restarts
	std::string file;		// if needed the file to be read from
};
class iDeg
{
public:
	iDeg();
	iDeg* getOut(bool create);
	iDeg* getIn(bool create);
	iDeg* getOut()const;
	iDeg* getIn()const;
	const iDegreeList* getList()const;
	iDegreeList* getListVar();
	void setList(const iDegreeList list);
	void append(unsigned short a ,float b);
private:
	iDeg* out,*in;
	iDegreeList iList;
	
};
class MyEdge{
	MyNode* secondNode;
	std::string edgeLabel;
public:
	MyEdge(){};
	MyEdge(MyNode* m, std::string label);
	MyNode* getNode()const;
	void setNode(MyNode* m);
	std::string getEdgeLabel()const;
};

#define edgelist std::unordered_map<unsigned int, MyEdge>
class MyNode
{
public:
	MyNode();
	MyNode(unsigned int id1, unsigned short type1, char* label1, unsigned char llen1);
	unsigned int getID() const;
	char* getLabel() const;
	unsigned short getType() const;
	unsigned char getLabelLen() const;
	const iDeg* getIDegree(unsigned int i,bool inDeg) const;
	iDeg* getIDegreeVar(unsigned int i, bool inDeg);
	unsigned int getNumOutNeighbors()const;
	unsigned int getNumInNeighbors()const;
	unsigned int getNumAllNeighbors()const;
	unsigned long index;
	boost::adjacency_list<boost::vecS, boost::vecS, boost::bidirectionalS>* adj;
	edgelist getUndirectedEdges() const;
	edgelist* getOutEdges();
	const edgelist* getOutEdgesConst()const;
	//std::list<MyNode*> getOutNeighbors() const;
	edgelist* getInEdges();
	const edgelist* getInEdgesConst()const;
	
	edgelist getOutOnlyEdges() const;
	edgelist getInOnlyEdges() const;
	bool addOutEdge(MyEdge ptr);
	bool addInEdge(MyEdge ptr);
	bool removeOutEdge(MyNode* ptr);
	bool removeInEdge(MyNode* ptr);
	bool removeAllEdges();
	bool resortEdges();
	std::string partitionString;
	//std::vector<unsigned int> partitioning;
	unsigned int lastPartition;
	std::vector<unsigned int> partitioningBackup;
	bool neighborChanged;
	bool indexed;
	//newindexing method
	//void updateIndex(char method,unsigned int i,char method2);
	bool token;//used in reduceMx functions
	
	//int compareIndex(IndexingValue& j);
	//bool compareIndex(const MyNode* y)const;
	//bool isUnindexed(){return !indexedCheck;};
	std::map<edgeDescriptor,float> currentDeg, oldDeg;
	std::vector<IndexingNode*> partitionPtrs;
private:
	//IndexingValue indexVal;//new index type
	//std::vector<IndexingNode*> indexPointers;//new index number
	//bool indexedCheck;
	edgelist outonly_edges;
	edgelist inonly_edges;
private:
	const iDeg* getIDegreeRecurse(const iDeg* R, unsigned int i,bool inDeg) const;
	unsigned int id;
	unsigned short type;
	unsigned char llen;
	char* label;
	iDeg* iList;
};



class MyGraph {
public:
	/*** Constructors ***/
	//initializes an empty graph such that this->directed = directed, num_nodes = 0, and max_id = 0
	//sets uptodatePartitions to true, uptodateIndices to true
	MyGraph(bool directed);
	//reads nodes from a file, if edges = true graph will add edges from the file. these edges will be directed/undirected based on the last argument
	//this sets i-degrees of nodes according to degreeArg
	MyGraph(std::string file, bool edges, positioningArgument& degreeArg, bool directed, char partitionMethod,bool plain,struct arguments& args);
	
	/*** Init ***/
	//initializes partitions: sorts nodes in the vector nodeSorted lexicographically by i-degrees 
	//then iterates through node sorted and fills in the partitionStarts[0-3] where paritionsStarts[i] is the start of each new partition
	//sets uptodatePartitions to true
	void createNodeSorted();
	
	/*** File I/O ***/
	//reads from a file
	int readGraph(const char* file, bool edges,arguments& args);
	int readPlainGraph(const char* file, bool edges,arguments& args);
	//writes a graph to the file <file>.grf
	int writeGraph(const char* file)const;
	int writePlainGraph(const char* file)const;
	//reads in the i-degrees from file
	int readIdegrees(std::string file);
	//writes the i-degrees of the graph to the file
	int writeIdegrees(const char* file)const;

    /*** Accessors ***/
	//returns the total number of nodes in the graph
    unsigned int getNumNodes()const;
	//returns the total number of types in the graph
	unsigned short getNumTypes()const;
	//returns the total number of edges
	unsigned long getNumEdges()const;
	//returns a copy of the types vector
    std::vector<unsigned short> getTypes()const;
	//finds the beginning and end of a type partition
	void getStartEndPartition(unsigned int &start,unsigned int &end, const MyNode* center)const;
	//returns a Node pointer based on the node id a null pointer is returned if the ID doesn't exist
	MyNode* getNodeById(unsigned int id);
	const MyNode* getNodeByIdConst(unsigned int id)const;
	//returns a Node pointer based on the index, if the index is not within [0,numNodes) a null pointer is returned
	MyNode* getNodeByIndex(unsigned int index)const;
	//returns the index of the node <ID>  if <ID> doesn't exit UINT_MAX is returned
	unsigned int getIndexById(unsigned int id)const;
	//returns pointers to the neighbors of the Node at NodeSorted[index]
	//if the graph hasn't been indexed, this will return an empty vector
	std::vector<const MyNode *> getNeighborsByIndex(unsigned int index)const;
	//returns the nodes within <degree> of <id> 
	std::vector<const MyNode *> getNeighborsToDegreeById(unsigned int id, unsigned int degree)const;
	//returns the <i> degree of n
//	std::vector<float> getIDegree(const MyNode* n, unsigned int i) const;	
//	std::vector<float> getInIDegree(const MyNode* n, unsigned int i) const;
	//returns the <i> degree of <id>
//	std::vector<float> getIDegree(unsigned int id, unsigned int i)const;
	//returns the node in the graph with the largest number of neighbors
	const MyNode* centerPopular() const;
	//returns the node in the graph with the highest number of different neighbors (ties are broken by total number of neighbors)
    const MyNode* centerDiverse() const;
	const MyNode* centerDiverseTotal() const;
	bool isDirected() const;
	bool isEdge(unsigned int id1, unsigned int id2) const;
	const MyNode * getOriginator() const;
	std::vector<const MyNode *> findNodesOfType(unsigned short type) const;
	std::vector<unsigned int> getSingletonIDs() const;
	//this function returns true  when:
	//in a directed graph there is a node c such that dist(c,start) + dist(c,target) <=max dist  
	//in an undirected graph dist(start, target) <= maxDist
	bool closeNodes(unsigned int start, unsigned int target, unsigned int maxDist);
	//gets the index of a given type.
	unsigned int getTypeIndex(unsigned int a)const;
	//this node returns the most diverse neigbhor of the given node specified by id
	const MyNode * getMostDiverseNeighbor(unsigned int id, unsigned int limits[] = NULL, unsigned int size = 0)const;
	//returns the number of different types found in the neighbors of the node
	unsigned int diversity(const MyNode * n)const ;
	unsigned int diversityTotal(const MyNode * n)const;
	//returns true if diversity(l) > diversity (r) or if diversity(l) == diversity(r) AND l has more neighbors
	bool isMoreDiverse(const MyNode * l, const MyNode * r)const;
	bool isMoreDiverseTotal(const MyNode * l, const MyNode * r)const;
	
	//frequencies should be a NULL pointer, when returned frequencies[A][B] will represent the frequency that type B is a neighbor of type A
	bool frequencyOfNeighbors(std::vector<std::vector<float> > & frequencies) const;
	
	int setOverlap(const MyGraph* f)const;


	//gets the <degree> neighbor hood around <id> and forms the induced subgraph around it
	//if skipPartitioning is true: i-degrees are not calculated otherwise: i-degrees are calculated based on the entire subgraph
	MyGraph* getSubgraphToDegreeById(unsigned int id, unsigned char degree, bool skipPartitioning,const arguments& args, bool centerIdegree=false);
	
	MyGraph* getSubgraphOfSize(unsigned int id, unsigned int minSize,const arguments& args);

	MyGraph* generateFragment_1N(const MyNode* center,const arguments& args);
	//sets the 1, 2, 3 degrees of all nodes within the graph based on infinite locality
	void updatePositions(int method,const arguments& args);
	/*** Mutators ***/
	
	//adds a node of <type> with <label> to the graph, the ID of the node will be set to max_id+1 
	//max_id and numNodes will be increased; uptodatePartitions will be set to false
	//if successfully added a 1 will be returned else a 0 will be returned
	unsigned int addNode(unsigned char llen, char* label,  unsigned short type);	
	//adds a node of <type> with  <label> (converted to c-string) to the graph, the ID of the node will be set to either id or max_id+1 
	//max_id and numNodes will be increased; uptodatePartitions will be set to false
	//if successfully added a 1 will be returned else a 0 will be returned
	unsigned int addNode(std::string label, unsigned short type, long int id=-1);
	//adds a node of <type> with  <label>  to the graph, with id: 
	//max_id may be increased and numNodes will be increased; uptodatePartitions will be set to false
	//if successfully added a 1 will be returned else a 0 will be returned
	unsigned int addNode(unsigned int id, unsigned char llen, char* label, unsigned short type);
	//adds an exact copy of n to the graph (the new node will have the same ID, type, and label. 
	//if edges = true any incoming and outgoing edges of n with both endpoints in the graph, will be added
	//max_id may be increased; num_nodes will be increased and uptodatePartions will be set to false; uptodateIndices &&= !edges
	//if successfully added a 1 will be returned else a 0 will be returned
	unsigned int addNode(const MyNode *n, bool edges);
	//attemps to add an edge from id1 to id2 calls addDirectedEdge or addUndirectedEdge as needed
	//returns success
	bool addEdge(unsigned int id1, unsigned int id2,std::string label);
	//if force = false: removes a node only if it has no edges  uptodatePartitions is set to false
	//if force = true: removes all Edges connected the node then removes the node, uptodatePartitions is set to false, uptodateIndices is set within removeEdges
	bool removeNode(unsigned int id, bool force);
	//removes any nodes which contain no in or out edges
	unsigned int removeSingletonNodes();
	//removes any and all edges from id1 to id2, sets uptodate Indices to false
	//if the edge doesn't exist, returns 0 else returns 1
	unsigned int removeEdge(unsigned int id1, unsigned int id2);
	//checks all outgoing/incoming edges of all nodes to ensure that for any node i with outgoing edges j that for each j, i is in the incoming of j.  for any which do not have valid pairs, the value is removed
	unsigned int removeRogueEdges();
	
	void setOriginator(unsigned int o);
	//mapping

	/***Statistics ***/
	unsigned int partitionStats(stats_keeper stats[], unsigned int toofew);
	void fuzzyStats(unsigned int i_deg, unsigned int maxDiffers, unsigned int toofew, unsigned int samples,stats_keeper *fuzzydiffers, stats_keeper& avgFuzziness, char distMethod,struct arguments args );

	std::vector< std::vector<unsigned int > > connectedComponents();
	unsigned int Diameter();
	/* Core Functionality */

	//generates a graph randomly using a normal distribution for determining the number of edges per node
	//
	unsigned long generateGraph(unsigned int numnodes, unsigned short num_types, unsigned short edges_per_node,const arguments& args,bool directed=false);

	/*index init*/
	void initIndex(char partitionMethod, positioningArgument& degreeArg, arguments& args);
	/* methods for finding fragments */
	
	//uses i-degree and distance to find candidates for some node
	std::vector<std::pair<double,unsigned int> > find_by_i(const MyNode * center, char similarityMethod, unsigned int i_degree, unsigned int nBest, char distMethod,struct arguments args);
	std::vector<std::pair<double,unsigned int> > multi_find_by_i(const MyNode* center,char similarityMethod, unsigned int i_degree, unsigned int nBest, char distMethod,struct arguments args);
	
	std::vector<std::pair<double,unsigned int> > find_by_tree(const MyNode *center, const MyGraph* fragment, char similarityMethod, unsigned int i_degree, unsigned int nBest,unsigned int N, char distMethod,struct arguments args);
	//added July 26
	//searches for the nBest matchs to the center such that each match has an adjacent node of the same type as neighbor
	//and the combined i-similarity(center, id) + i-similarity(neighbor, match) is minimized
	std::vector<std::pair<double,unsigned int> > find_by_pair(const MyNode *center, const MyNode * neighbor, char similarityMethod, unsigned int i_degree, unsigned int nBest, char distMethod,struct arguments args);
	std::vector<std::pair<double,unsigned int> > find_by_Ntree(const MyNode *center, const MyGraph * fragment, char similarityMethod, unsigned int i_degree, unsigned int nBest,unsigned int N, char distMethod, struct arguments args);
	std::vector<std::pair<double,std::map<unsigned int, unsigned int> > > find_by_NtreeNeighborhood(const MyNode *center, const MyGraph * fragment, char similarityMethod, unsigned int i_degree, unsigned int nBest,unsigned int N, char distMethod, struct arguments args);

	float NtreeSimilarity(unsigned int N, const MyNode *center, const MyGraph* fragment, MyNode* candidate, unsigned int i_degree, char similarityMethod, std::vector<unsigned int> best_neighbors, char distMethod,struct arguments args);
	std::vector<std::pair<double,std::map<unsigned int, unsigned int> > > find_by_itPart(const MyNode *center, const MyGraph* fragment, struct arguments args);
	
	std::vector<std::pair<double,std::map<unsigned int, unsigned int> > > find_by_itPart_iterate(const MyNode *center, const MyGraph* fragment, struct arguments args, bagsType & bags,MyPartitioning iterPart,unsigned int depth);

	//use similarity in utilities.cpp float distanceBetweenNodes(char similarityMethod,const Node* center, const Node* center2, unsigned int i_degree);
	double verifyEdges(const MyGraph* fragment, std::map<unsigned int, unsigned int>& mapping,struct arguments args);
	double verifyEdges(const MyGraph* fragment, std::map<unsigned int, unsigned int>& mapping,struct arguments args,MyNode* question);
	
	//searches for the nBest matchs to the center such that each match has an adjacent node of the same type as neighbor
	//and the sum( i-similarity(chainLink[], candidates[]) is minimized
	//chainLinks is the list of nodes such that chainLink[i] is unique and there is an edge chainLink[i]->chainLink[i+1]
	//std::vector<unsigned int> find_by_chain(std::vector<const Node *> chainLinks, char similarityMethod, unsigned int i_degree, unsigned int nBest);


	/* fragment generation methods */
	//all methods for generating fragments MUST add as possible types the entire type vector of the original graph to allow for proper comparison of i-degrees
	
	//generates a fragment by a probabilistic-random walk. With a probability of p, the walk will restart
	//if the walk restarts and no new nodes were discovered, p is updated to p = p*alpha
	void generateFragment_pRandomWalk(MyGraph* database, unsigned int numnodes, float p, float alpha, unsigned int minEdges, arguments& args,bool useBackEdges=false);
	void generateFragment_pRandomWalk(MyGraph* database, const MyNode* center, unsigned int numnodes, float p, float alpha, arguments& args,bool useBackEdges=false);
	

	//restricted breadth first search to maximum of maxDist
	bool rBFS(unsigned int startID, unsigned int targetID, unsigned int maxDist, int direction) const;
	std::map<unsigned int, unsigned int> rBFS(unsigned int startID, unsigned int maxDist, bool outgoing) const;

	//map iterativePartitioned nodes
	unsigned int mapItPartFrag(const MyGraph* fragment,std::map<unsigned int, mappingType > bags,std::map<unsigned int, unsigned int> mapping,unsigned int penalty,std::vector<unsigned int> order,MyPartitioning& iterPart);
	void compareItPartMappings(std::list<std::map<unsigned int,unsigned int> > maps);
	
	
	std::list<std::map<unsigned int,unsigned int> > mapItPartGreedy(const MyGraph* fragment,bagsType bags,unsigned int penalty,std::vector<unsigned int> order, unsigned int centerOrder,MyPartitioning& iterPart,arguments& args,std::map<unsigned int, std::vector<unsigned int> >& restrictions);
	std::list<std::map<unsigned int,unsigned int> > mapItPartBFS(const MyGraph* fragment, bagsType bags,unsigned int penalty,std::vector<unsigned int> order, unsigned int centerOrder,MyPartitioning& iterPart);
	std::list<std::map<unsigned int,unsigned int> > mapItPartBFS2(const MyGraph* fragment,const bagsType &bags,std::map<unsigned int, unsigned int> mapping,unsigned int penalty,unsigned int next,unsigned int min,MyPartitioning& iterPart);
	float greedyNeighborMap(unsigned int candidateID, const MyGraph* frag, unsigned int anchor, char similarityMethod, unsigned int iDist, char distMethod,struct arguments args) const;

	//options method of iterative partition mapping
	std::vector<std::pair<double,std::map<unsigned int, unsigned int> > > calculateOptions(const MyGraph* fragment, bagsType bags, const MyNode* center,struct arguments args,MyPartitioning& iterPart);
	/*locality*/

	//computes the local positions of all nodes based on the locality
	//locality = 1,2,3 positions are updated based on a distance of (locality) from v
	//locality = -1 positions are updated based on a random walk with returns
	void localPositions(int locality,const arguments& args);
	void positionedByNeighborhood(int n,const arguments& args);

	//statistics
	void calculateBasicStatistics();
	
	/*destructor*/
	~MyGraph();
	

private:
	//returns the index of the type in the types vector
	unsigned int normalizedType(unsigned short type) const;
	//attempts to add a directed edge from ID1 to ID2: 
	//if id1 == id2 returns false and doesn't add edge
	//if an edge doesn't exist, id2 is added to id1.outgoing_edges and id1 is added to id2.incoming_edges
	bool addDirectedEdge  (unsigned int id1, unsigned int id2,std::string label);
	//attempts to add an undirected edge between ID1 and ID2: 
	//if id1 == id2 returns false and doesn't add edge
	//if an edge doesn't exist, id2 is added to id1.outgoing_edges and id1 is added to id2.outgoing_edges
	bool addUndirectedEdge(unsigned int id1, unsigned int id2,std::string label);
	/* helper methods */
	//creates the actual node if possible
	unsigned int createNode(unsigned char llen, char* label, unsigned short type, unsigned int id);
	
	//calculates the i-degree of node <id> using <method> method = 0: sum of types, method = 1, probability of ending at a type
	void calculateIDegree(unsigned int id, int method, unsigned int i);
	
		
	/* Representation */
	unsigned int max_id;
	unsigned int originator;
	unsigned long num_edges;
	bool directed;	
	bool uptodatePartitions; //keeps track of whether or not nodeSorted and partitionStarts[0-3] are currently up-to-date //old
	bool uptodateIndices;	// keeps track of whether updatePositions needs to be called //old
	arguments* args;
	
	public: 
	std::vector< MyNode*> nodeSorted;
	mapIDNodes nodes;
	private:
	std::vector<unsigned int> partitionStarts[7]; //0-3 //oldindexing
	public:
	MyPartitioning itPart;//old indexing
	
	Indexing indexing;//New Indexing. TODO (remove old indexing to clean)
	
	private:
	/* Graph Boost Representation */
	boost::adjacency_list<boost::vecS, boost::vecS, boost::bidirectionalS>* bGraph;//A graph using an adjacencylist, vector of outedges, vector of nodes, has access to in and out edges
	
	
};

/*parallel*/
void* findByItPartThread(void* u);

struct findByItPartArgs
{
	const MyGraph *	G;
	const MyGraph *fragment;
	std::vector<MyNode*>::const_iterator start;
	std::map<unsigned int, bool>* rechecker,*nextChecker;
	unsigned int increment,dLimit,nextID;
	#ifdef PARALLEL
	pthread_mutex_t lock,lockCont,lockID;
	#endif
	bagsType bags;
	std::vector<bagsType > returnVal;
	arguments* args;
	bool cont;
	MyPartitioning* iterPart;
};

#endif
