#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <queue>
#include <time.h>
#include <algorithm>
#include <math.h>
#include <fstream>	

#include <boost/config.hpp>
#include <boost/utility.hpp>             // for boost::tie
#include <boost/graph/graph_traits.hpp>  // for boost::graph_traits
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/max_cardinality_matching.hpp> //For Edmonds matching algorithm
#include <boost/property_map/property_map.hpp> //for matemap?
#include <boost/concept_check.hpp>

#include "graph.h"

/*** Constructors ***/

//creates a ideg structure, and inits the out and in to null.
iDeg::iDeg()
{
	out=0;
	in=0;
}
//returns the outdegree. If it doesn't exist, 'create' will create it
iDeg* iDeg::getOut(bool create)
{
	if(out==0 && create)
	{
		out= new iDeg();
	}
	return out;
}
//returns the indegree. If it doesn't exist, 'create' will create it
iDeg* iDeg::getIn(bool create)
{
	if(in==0 && create)
	{
		in= new iDeg();
	}
	return in;
}
//return outdegree
iDeg* iDeg::getOut()const
{
	return out;
}
//returns the indegree
iDeg* iDeg::getIn()const
{
	return in;
}
//returns a const idegree list
const iDegreeList* iDeg::getList()const
{
	if(this==0)
	{//this shouldn't happen
		return 0;
	}
	return &iList;
}

//returns the idegree ptr
iDegreeList* iDeg::getListVar()
{
	return &iList;
}

//sets an idegree to an input idegree
void iDeg::setList(const iDegreeList list)
{
	iList=list;
}

//add b to the type [a] in this idegree
void iDeg::append(unsigned short a ,float b)
{
	if(b==0)
	{
		return;
	}
	iDegreeList::iterator temp=iList.find(a);
	if(temp==iList.end())
	{
		iList[a]=b;
	}
	else
	{
		(temp)->second+=b;
	}
}