#include <iostream>
#include <fstream>
#include "graph.h"
#include <algorithm>
#include "utilities.h"
#include "indexing.h"

void Indexing::updateIndex(unsigned int i)
{
	for(std::list<MyNode*>::iterator itr=root->nodes.begin();itr!=root->nodes.end();itr++)
	{
		(*itr)->oldDeg=(*itr)->currentDeg;
		(*itr)->currentDeg.clear();
	}
	for(std::list<MyNode*>::iterator itr=root->nodes.begin();itr!=root->nodes.end();itr++)
	{
		if(i==0)
		{
			edgeDescriptor ED('0',(*itr)->getType(),"_");
			(*itr)->currentDeg[ED]=1;
		}
		else if(i==1)
		{
			if(args->similarityMethod=='E' ||args->similarityMethod=='F')
			{
				const edgelist* O=(*itr)->getOutEdgesConst();
				for(edgelist::const_iterator E=O->begin();E!=O->end();E++)
				{
					edgeDescriptor ED('O',E->second.getNode()->getType(),E->second.getEdgeLabel());
					std::map<edgeDescriptor,float>::iterator x=(*itr)->currentDeg.find(ED);
					if(x==(*itr)->currentDeg.end())
					{
						(*itr)->currentDeg[ED]=1;
					}
					else
					{
						(*itr)->currentDeg[ED]+=1;
					}
				}
			}
			if(args->similarityMethod=='F')
			{
				edgelist* O=(*itr)->getInEdges();
				for(edgelist::iterator E=O->begin();E!=O->end();E++)
				{
					edgeDescriptor ED('I',E->second.getNode()->getType(),E->second.getEdgeLabel());
					std::map<edgeDescriptor,float>::iterator x=(*itr)->currentDeg.find(ED);
					if(x==(*itr)->currentDeg.end())
					{
						(*itr)->currentDeg[ED]=1;
					}
					else
					{
						(*itr)->currentDeg[ED]+=1;
					}
				}
			}
		}
		else
		{
			
			if(args->similarityMethod=='E' ||args->similarityMethod=='F')
			{
			const edgelist* O=(*itr)->getOutEdgesConst();
			for(edgelist::const_iterator E=O->begin();E!=O->end();E++)
			{
				for(std::map<edgeDescriptor,float>::iterator jtr=E->second.getNode()->oldDeg.begin();jtr!=E->second.getNode()->oldDeg.end();jtr++)
				{
					std::map<edgeDescriptor,float>::iterator x=(*itr)->currentDeg.find(jtr->first);
					if(x==(*itr)->currentDeg.end())
					{
						(*itr)->currentDeg[jtr->first]=jtr->second;
					}
					else
					{
						(*itr)->currentDeg[jtr->first]+=jtr->second;
					}
				}
			}
			}
			if(args->similarityMethod=='F')
			{
			edgelist* O=(*itr)->getInEdges();
			for(edgelist::iterator E=O->begin();E!=O->end();E++)
			{
				for(std::map<edgeDescriptor,float>::iterator jtr=E->second.getNode()->oldDeg.begin();jtr!=E->second.getNode()->oldDeg.end();jtr++)
				{
					std::map<edgeDescriptor,float>::iterator x=(*itr)->currentDeg.find(jtr->first);
					if(x==(*itr)->currentDeg.end())
					{
						(*itr)->currentDeg[jtr->first]=jtr->second;
					}
					else
					{
						(*itr)->currentDeg[jtr->first]+=jtr->second;
					}
				}
			}
			}
		}
		//fprintf(stderr,"oldsize=%d new=%d\n",(*itr)->oldDeg.size(),(*itr)->currentDeg.size());
	}
}
void Indexing::partition(unsigned int i)
{
	partitionLevels.push_back(std::list<IndexingNode>());//add new level
	//fprintf(stderr,"Partition Level: %d\n",i);
	if(i==0)
	{	
		root->partitionByType(this);
	}
	else
	{
		//fprintf(stderr,"Level=%d / %d\n",i,partitionLevels.size()-1);
		for(std::list<IndexingNode>::iterator itr=partitionLevels[i].begin();itr!=partitionLevels[i].end();itr++)
		{
			itr->partitionByIDegree(i,this);
		}
	}
	fprintf(stderr,"Num. of Partitions on level %d -> %d\n",i,partitionLevels[i+1].size());
}

void Indexing::init(std::vector<MyNode*>& nodeSorted,arguments& args)
{
	args.T.start("Indexing::init");
	this->args=&args;
	partitionLevels.push_back(std::list<IndexingNode>());
	partitionLevels[0].push_back(IndexingNode(0,nodeSorted,0));
	root=&(*(partitionLevels[0].begin()));
	for(unsigned int i=0;i<=args.i_degree;i++)
	{
		//args.T.start("update "+my_itoa(i));
		args.T.start("update index");
		updateIndex(i);
		args.T.stop("update index");
		//args.T.stop("update "+my_itoa(i));
		//args.T.start("indexing-"+my_itoa(i,10));
		args.T.start("partition index");
		partition(i);
		args.T.stop("partition index");
		//args.T.stop("indexing-"+my_itoa(i,10));
	}
	args.T.stop("Indexing::init");
}

IndexingNode* Indexing::newPartition(unsigned int level,std::list<MyNode*>::const_iterator begin, std::list<MyNode*>::const_iterator end,IndexingNode* parent)
{
	while(partitionLevels.size()<level)
	{
		partitionLevels.push_back(std::list<IndexingNode>());
	}
	partitionLevels[level].push_back(IndexingNode(level,begin,end,parent));
	partitionLevels[level].back().updatePtrs();
	return &(partitionLevels[level].back());
}

IndexingNode* Indexing::getRoot()
{
	return root;
}