#include "graph.h"
#include "indexingNode.h"
#include "utilities.h"
#include <algorithm>
#include <list>
#include <vector>

IndexingNode::IndexingNode(unsigned int l,std::list<MyNode*>::const_iterator begin, std::list<MyNode*>::const_iterator end,IndexingNode* parentPtr)
{	
	level=l;
	addNodes(begin,end);
	parent=parentPtr;
	value.setIDegree((*begin)->currentDeg);
	//fprintf(stderr,"size= %d\n",nodes.size());
	mark=false;
}	
IndexingNode::IndexingNode(unsigned int l,std::vector<MyNode*> n,IndexingNode* parentPtr)
{
	level=l;
	addNodes(n);
	parent=parentPtr;
	value.setIDegree((*n.begin())->currentDeg);
	//fprintf(stderr,"size= %d\n",nodes.size());
	mark=false;
}
void IndexingNode::addNodes(std::list<MyNode*>::const_iterator begin, std::list<MyNode*>::const_iterator end)
{
	std::copy( begin, end, std::back_inserter(nodes));
}	
void IndexingNode::addNodes(std::vector<MyNode*> n)
{
	std::copy(n.begin(),n.end(),std::back_inserter(nodes));
}
void IndexingNode::partitionByType(Indexing* index)
{
	nodes.sort(typeCompare);
	std::list<MyNode*>::const_iterator itr=nodes.begin();
	std::list<MyNode*>::const_iterator itrFirst=nodes.begin();
	itr++;
	for(;itr!=nodes.end();itr++)
	{
		if(typeCompare(*itrFirst,*itr))
		{
			//fprintf(stderr,"%d != %d\n",(*itrFirst)->getType(),(*itr)->getType());
			createSubpartition(itrFirst,itr,index);
			itrFirst=itr;
		}
	}
	//fprintf(stderr,"last type = %d\n",(*itrFirst)->getType());
	createSubpartition(itrFirst,itr,index);
}
void IndexingNode::partitionByIDegree(unsigned int i,Indexing* index)
{
	nodes.sort(lexFirstIdegree);
	std::list<MyNode*>::const_iterator itr=nodes.begin();
	std::list<MyNode*>::const_iterator itrFirst=nodes.begin();
	itr++;
	for(;itr!=nodes.end();itr++)
	{
		if(lexFirstIdegree(*itrFirst,*itr))
		{
			createSubpartition(itrFirst,itr,index);
			itrFirst=itr;
		}
	}
	createSubpartition(itrFirst,itr,index);
}
void IndexingNode::createSubpartition(std::list<MyNode*>::const_iterator l,std::list<MyNode*>::const_iterator r,Indexing* index)
{
	IndexingNode* ptr= index->newPartition(level+1,l,r,this);
	subpartitions.push_back(ptr);
}

std::list<IndexingNode*>* IndexingNode::getSubpartitions()
{
	return &subpartitions;
}

IndexingValue* IndexingNode::getIndexValue()
{
	return &value;
}

void IndexingNode::updatePtrs()
{
	for(std::list<MyNode*>::iterator itr=nodes.begin();itr!=nodes.end();itr++)
	{
		(*itr)->partitionPtrs.push_back(this);
	}
}

void IndexingNode::intersect(std::list<MyNode*>& iList)
{
	iList.sort();
	nodes.sort();
	std::vector<MyNode*> result(iList.size()+nodes.size());
	/*std::list<MyNode*> temporary=iList;
	std::copy(nodes.begin(),nodes.end(),std::back_inserter(temporary));
	temporary.sort(idSort);
	std::list<MyNode*>::iterator itr=temporary.begin(),itr2;
	itr2=itr;
	itr2++;
	for(;itr2!=temporary.end();itr2++)
	{
		if((*itr)->getID()==(*itr2)->getID()){
		fprintf(stderr,"==%d\n",(*itr)->getID());}
		itr++;
	}*/
	
	std::vector<MyNode*>::iterator it;
	it=std::set_intersection(iList.begin(),iList.end(),nodes.begin(),nodes.end(),result.begin());
	result.resize(it-result.begin()); 
	std::list<MyNode*> r(result.begin(),result.end());
	//fprintf(stderr,"Intersect %d <- (%d , %d)\n",r.size(),iList.size(),nodes.size());
	if(r.size()==0)
	{
		fprintf(stderr,"There's a problem %d\n",result.size());
		exit(0);
	}
	nodes=r;
}

