#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <queue>
#include <time.h>
#include <algorithm>
#include <math.h>
#include <fstream>

#include <boost/config.hpp>
#include <boost/utility.hpp>             // for boost::tie
#include <boost/graph/graph_traits.hpp>  // for boost::graph_traits
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/max_cardinality_matching.hpp> //For Edmonds matching algorithm
#include <boost/property_map/property_map.hpp> //for matemap?
#include <boost/concept_check.hpp>

#include "graph.h"
#include "utilities.h"

/*** Constructors ***/
MyNode::MyNode()
{
	token=false;
//indexedCheck=false;
}
MyNode::MyNode(unsigned int id1, unsigned short type1, char* label1, unsigned char llen1)
{
	token=false;
	
	id=id1;
	type=type1;
	label=label1;
	llen=llen1;
	iList = new iDeg();
	//indexedCheck=false;
}
unsigned int MyNode::getID() const
{
	return id;
}
char* MyNode::getLabel() const
{
	return label;
}
unsigned short MyNode::getType() const
{
	return type;
}
unsigned char MyNode::getLabelLen() const
{
	return llen;
}
//returns the <i> degree of n
const iDeg* MyNode::getIDegree(unsigned int i,bool inDeg) const
{
	return getIDegreeRecurse(iList, i,inDeg);
}

const iDeg* MyNode::getIDegreeRecurse(const iDeg* R, unsigned int i,bool inDeg) const
{
	if(R==0)
	{
		return 0;
	}
	if(i>0)
	{
		const iDeg* next=(inDeg)?R->getIn():R->getOut();
		return getIDegreeRecurse(next,i-1,inDeg);
	}
	return R;
}

iDeg* MyNode::getIDegreeVar(unsigned int i, bool inDeg)
{
	iDeg* returnVal=iList;
	for(;i>0;i--)
	{
		if(!inDeg)
		{
			if(returnVal->getOut(true)!=0)
			{
				returnVal=returnVal->getOut();
			}
			else
			{
				return 0;
			}
		}
		else
		{
			if(returnVal->getIn(true)!=0)
			{
				returnVal=returnVal->getIn();
			}
			else
			{
				return 0;
			}
		}
	}
	return returnVal;
}
unsigned int MyNode::getNumOutNeighbors() const
{
	return outonly_edges.size();
}

unsigned int MyNode::getNumInNeighbors() const
{
	return inonly_edges.size();
}

unsigned int MyNode::getNumAllNeighbors() const
{
	return inonly_edges.size()+outonly_edges.size();
}

edgelist MyNode::getUndirectedEdges() const
{
	edgelist edges;
	edges.insert(outonly_edges.begin(),outonly_edges.end());
	edges.insert(inonly_edges.begin(),inonly_edges.end());
	return edges;
}
edgelist* MyNode::getOutEdges()
{
	return &outonly_edges;
}
const edgelist* MyNode::getOutEdgesConst() const
{
	return &outonly_edges;
}

edgelist* MyNode::getInEdges()
{
	return &inonly_edges;
}
const edgelist* MyNode::getInEdgesConst()const 
{
	return &inonly_edges;
}
edgelist MyNode::getOutOnlyEdges() const
{
	return outonly_edges;
}
edgelist MyNode::getInOnlyEdges() const
{
	return inonly_edges;
}
bool MyNode::addOutEdge(MyEdge ptr)
{
	if(outonly_edges.find(ptr.getNode()->getID())!=outonly_edges.end())
	{
		return false;
	}
	outonly_edges[ptr.getNode()->getID()]=ptr;
	ptr.getNode()->addInEdge(MyEdge(this,ptr.getEdgeLabel()));
	return true;
}
bool MyNode::addInEdge(MyEdge ptr)
{
	if(inonly_edges.find(ptr.getNode()->getID())!=inonly_edges.end())
	{
		return false;
	}
	inonly_edges[ptr.getNode()->getID()]=ptr;
	ptr.getNode()->addOutEdge(MyEdge(this,ptr.getEdgeLabel()));
	return true;
}
bool MyNode::removeOutEdge(MyNode* ptr)
{
	edgelist::iterator i=outonly_edges.find(ptr->getID());
	if(i!=outonly_edges.end())
	{
		outonly_edges.erase(i);
		ptr->removeInEdge(this);
	}
	return true;
}
bool MyNode::removeInEdge(MyNode* ptr)
{
	edgelist::iterator i=inonly_edges.find(ptr->getID());
	if(i!=inonly_edges.end())
	{
		inonly_edges.erase(i);
		ptr->removeOutEdge(this);
	}
	return true;
}
bool MyNode::removeAllEdges()
{
	//todo implement
	fprintf(stderr,"Error: removeAllEdges is unimplemented.\n");
	exit(0);
}
bool MyNode::resortEdges()
{
	//outonly_edges.sort(iterativePartitionNeighborSort);	
	//inonly_edges.sort(iterativePartitionNeighborSort);
	//bidirectional_edges.sort(iterativePartitionNeighborSort);
	return true;
}

/*std::list<MyNode*> MyNode::getOutNeighbors() const
{
	const edgelist* e=getOutEdgesConst();
	std::list<MyNode*> returnVal;
	for(edgelist::const_iterator itr=e->begin();itr!=e->end();itr++)
	{
		returnVal.push_back(itr->second.getNode());
	}
	return returnVal;
}*/



/*bool MyNode::compareIndex(const MyNode* r)const
{
	if(this->indexVal.getMethod()!=r->indexVal.getMethod())
	{
		fprintf(stderr,"compareIndex:Different methods\n");
		exit(0);
	}
	if(this->indexVal.getMethod()=='N'||this->indexVal.getMethod()=='i')
	{
		if(this->getType()==r->getType())
		{
			//fprintf(stderr,"index Size %d %d\n",this->getIndexSize(),r->getIndexSize());
			if(this->getIndexSize()+r->getIndexSize()==0)
			{
				return 0;
			}
			int t=0;
			t=this->indexVal.compare(r->indexVal);
			//fprintf(stderr,"t=%d\n",t);
			return 0<t;
		}
		else
		{	
			return this->getType() < r->getType();
		}
	}
	else
	{
		fprintf(stderr,"compareIndex: Unknown method %c\n",this->indexVal.getMethod());
		exit(0);
	}
}
void MyNode::updateIndex(char method, unsigned int i,char method2)
{
	indexVal.clear();
	indexVal.setMethod(method,method2);
	indexVal.setType(this->getType());
if(i==0)
{
	return;
}
	if(method=='N')
	{
		bool newIndex=(isUnindexed());
		if(method2=='B'){//use bidirectional edges
			for(edgelist::iterator itr=bidirectional_edges.begin();itr!=bidirectional_edges.end();itr++)
			{
				indexVal.addIPartition('B',itr->getNode()->getIndexNum2(i),itr->getEdgeLabel(),1);
			}
			for(edgelist::iterator itr=inonly_edges.begin();itr!=inonly_edges.end();itr++)
			{
				indexVal.addIPartition('I',itr->getNode()->getIndexNum2(i),itr->getEdgeLabel(),1);
			}
			for(edgelist::iterator itr=outonly_edges.begin();itr!=outonly_edges.end();itr++)
			{
				indexVal.addIPartition('O',itr->getNode()->getIndexNum2(i),itr->getEdgeLabel(),1);
			}
		}
		if(method2=='i'||method2=='b')//use in /out edges
		{
			edgelist iTemp=getInEdges();
			for(edgelist::iterator itr=iTemp.begin();itr!=iTemp.end();itr++)
			{
				indexVal.addIPartition('I',itr->getNode()->getIndexNum2(i),itr->getEdgeLabel(),1);
			}
		}
		if(method2=='o'||method2=='b')//out edges only
		{
			edgelist oTemp=getOutEdges();
			for(edgelist::iterator itr=oTemp.begin();itr!=oTemp.end();itr++)
			{
				indexVal.addIPartition('O',itr->getNode()->getIndexNum2(i),itr->getEdgeLabel(),1);
			}
		}
		if(method2!='B'&&method2!='b'&&method2!='o'&&method2!='i')
		{
			fprintf(stderr,"UpdateIndex: Unknown method2=%c\n",method2);
			exit(0);
		}
		indexVal.sortMe();
	}
	else if(method=='i')
	{
		if(method2=='o')
		{
			edgelist oTemp=getOutEdges();
			for(edgelist::iterator itr=oTemp.begin();itr!=oTemp.end();itr++)
			{
				if(i==1)
				{
				//fprintf(stderr,"%c %d %s %d\n",'O',itr->getNode()->getIndexNum2(i),itr->getEdgeLabel().c_str(),1);
					indexVal.addIPartition('O',itr->getNode()->getIndexNum2(i),itr->getEdgeLabel(),1);
				}
				else{
				for(std::map<iPartitionEntryType,unsigned int>::const_iterator indexItr=itr->getNode()->getIndexNum2(i)->getIndexValue()->getFirstEntry();indexItr!=itr->getNode()->getIndexNum2(i)->getIndexValue()->getEndEntry();indexItr++)
				{
					
					indexVal.addIPartition('O',
					indexItr->first.nodePtr,
					indexItr->first.label,
					indexItr->second);
				}
				}
			}
		}
		else
		{
			fprintf(stderr,"UpdateIndex: Unknown method2\n");
			exit(0);
		}
		indexVal.sortMe();
	}
	else
	{
		fprintf(stderr,"Unknown updateIndex method\n");
		exit(0);
	}
	indexedCheck=true;
}*/
