#include <iostream>
#include <fstream>
#include <algorithm>
#include "utilities.h"
#include "graph.h"
#include "searcher.h"
#include <list>
#include <vector>
#include <queue>

//boost
#include <boost/config.hpp>
#include <boost/utility.hpp>             // for boost::tie
#include <boost/graph/graph_traits.hpp>  // for boost::graph_traits
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/max_cardinality_matching.hpp> //For Edmonds matching algorithm
#include <boost/property_map/property_map.hpp> //for matemap?
#include <boost/concept_check.hpp>
#include "dsrBipartiteMatching.h"
Searcher::Searcher()
{
}
Searcher::Searcher(Indexing& ind, MyGraph* f, arguments& args, MyGraph* d)
{
	Search(ind,f,args,d);
}
bool Searcher::shrinkUntilStable(std::map<unsigned int, std::list<unsigned int> >& Mx,MyGraph* fragment, MyGraph* database,FILE* output)
{
	unsigned int mxtotal=1;
	unsigned int mxNew=mxtotal;
	do{
		mxtotal=mxNew;
		mxNew=0;
		mxReduce(Mx,fragment, database, stderr);
			
		for(std::map<unsigned int, std::list<unsigned int> >::iterator mxitr=Mx.begin();mxitr!=Mx.end();mxitr++)
		{
			mxNew+=mxitr->second.size();
		}
	}while(mxNew!=mxtotal);
	return true;
}
void Searcher::Search(Indexing& ind,MyGraph* f, arguments& args, MyGraph* d)
{
	this->database=d;
	this->args=&(args);
	fprintf(stderr,"Search begin\n");
	index=&ind;
	fragment=f;
	args.T.start("S:init");
	initPossibilities();
	args.T.stop("S:init");
	args.T.start("S:refine");
	std::map<unsigned int, std::list<unsigned int> > Mx;
	//refine(args.i_degree);//refine this many steps
	//convertMX(Mx);
	refineDFS(args.i_degree,Mx);
	args.T.stop("S:refine");
	for(std::map<unsigned int, std::list<unsigned int> >::iterator nodeitr=Mx.begin();nodeitr!=Mx.end();nodeitr++)
	{
		//fprintf(args.mXoutput,"%d ",nodeitr->second.size());
		//fprintf(stdout,"%s ",fragment->getNodeById(nodeitr->first)->getLabel());
		fprintf(args.mXoutput,"%s ",fragment->getNodeById(nodeitr->first)->getLabel());
		for(std::list<unsigned int>::iterator listItr=nodeitr->second.begin();listItr!=nodeitr->second.end();listItr++)
		{
			//fprintf(stdout,"%s ",database->getNodeById(*listItr)->getLabel());
			fprintf(args.mXoutput,"%s ",database->getNodeById(*listItr)->getLabel());
		}
		//fprintf(stdout,"\n");
		fprintf(args.mXoutput,"\n");
	}
	args.T.print(stderr);
	//fprintf(args.mXoutput,"\n");
	//fflush(args.mXoutput);
		
	//printMx(args.mXoutput);//print before shrinking
	
	/*unsigned int v1=100,v2=0;
	args.T.start("S:shrink");
	while(v2!=v1)
	{
	v1=v2;
	v2=initReducedMx(possibilityHeirarchy.back());
	fprintf(stderr,"v2= %d\n",v2);
	}
	args.T.stop("S:shrink");*/
	//
	
	args.T.start("S:Shrink2");
	shrinkUntilStable(Mx,fragment,database,args.mXoutput2);
	args.T.stop("S:Shrink2");
	/*for(std::map<unsigned int, std::list<unsigned int> >::iterator nodeitr=Mx.begin();nodeitr!=Mx.end();nodeitr++)
	{
		//fprintf(args.mXoutput2,"%d ",nodeitr->second.size());
		fprintf(stderr,"Mx2=%d\n",nodeitr->second.size());
	}*/
	for(std::map<unsigned int, std::list<unsigned int> >::iterator nodeitr=Mx.begin();nodeitr!=Mx.end();nodeitr++)
	{
		//fprintf(args.mXoutput,"%d ",nodeitr->second.size());
		//fprintf(stdout,"%s ",fragment->getNodeById(nodeitr->first)->getLabel());
		fprintf(args.mXoutput2,"%s ",fragment->getNodeById(nodeitr->first)->getLabel());
		for(std::list<unsigned int>::iterator listItr=nodeitr->second.begin();listItr!=nodeitr->second.end();listItr++)
		{
			//fprintf(stdout,"%s ",database->getNodeById(*listItr)->getLabel());
			fprintf(args.mXoutput2,"%s ",database->getNodeById(*listItr)->getLabel());
		}
		//fprintf(stdout,"\n");
		fprintf(args.mXoutput2,"\n");
	}
	//fprintf(args.mXoutput2,"\n");
	//fflush(args.mXoutput2);
	//
		
	args.T.print(stderr);
	//printMx(args.mXoutput2);//print after shrinking
	MxFinished=Mx;
}
void Searcher::convertMX(std::map<unsigned int, std::list<unsigned int> >& Mx)
{
	for(MxVector::iterator phItr=possibilityHeirarchy.back().begin();phItr!=possibilityHeirarchy.back().end();phItr++)
	{
		std::list<unsigned int> nodeList;
		for(MxOptions::iterator optionsItr=phItr->second.begin();optionsItr!=phItr->second.end();optionsItr++)
		{
			for(std::list<MyNode*>::iterator oItr=optionsItr->first->nodes.begin();oItr!=optionsItr->first->nodes.end();oItr++)
			{
				nodeList.push_back((*oItr)->getID());
			}
		}
		for(std::list<MyNode*>::iterator nodeItr=phItr->first->nodes.begin();nodeItr!=phItr->first->nodes.end();nodeItr++)
		{
			Mx[(*nodeItr)->getID()]=nodeList;
		}
	}
}
void Searcher::printMx(FILE* output)
{
	unsigned int count=0;
	for(MxVector::const_iterator itr=possibilityHeirarchy.back().begin();
		itr!=possibilityHeirarchy.back().end();itr++)
	{
		for(std::list<MyNode*>::const_iterator nodeItr=itr->first->getNodesBegin();
			nodeItr!=itr->first->getNodesEnd();nodeItr++)
		{
			unsigned int mxCount=0;
			for(MxOptions::const_iterator jtr=itr->second.begin();
				jtr!=itr->second.end();jtr++)
			{
				mxCount+=jtr->first->getNumNodes();
			}
			fprintf(stderr,"M(%d)->\t%d\n",(*nodeItr)->getID(), mxCount);
			fprintf(output,"%d ",mxCount);//the mx count for each node
			count++;
		}
	}
	for(;count<args->fragSize;count++)
	{
		fprintf(output,"9999999 ");
	}
	fprintf(output,"\n");
	fflush(output);
}
void Searcher::initPossibilities()
{
	possibilityHeirarchy.clear();
	possibilityHeirarchy.push_back(MxVector());//add an empty list
	(*possibilityHeirarchy.begin())[fragment->indexing.getRoot()][index->getRoot()]=true;//init so the root of F can map to the root of G
	return;
}
void Searcher::refineInner(unsigned int currentStep)
{
	MxVector::iterator PHBegin=(possibilityHeirarchy.rbegin())->begin();
	MxVector::iterator PHEnd=(possibilityHeirarchy.rbegin())->end();
	possibilityHeirarchy.push_back(MxVector());
	for(MxVector::iterator i=PHBegin;i!=PHEnd;i++)
	{
		std::list<IndexingNode* >::const_iterator childBEGIN,childEND;
		childBEGIN=i->first->getSubpartitions()->begin();
		childEND=i->first->getSubpartitions()->end();
		for(std::list<IndexingNode* >::const_iterator child=childBEGIN;child!=childEND;child++)
		{
			for(MxOptions::iterator j=i->second.begin();j!=i->second.end();j++)
			{
				std::list<IndexingNode* >::const_iterator jchildBEGIN,jchildEND;
				jchildBEGIN=(*j).first->getSubpartitions()->begin();
				jchildEND=(*j).first->getSubpartitions()->end();
			
				for(std::list<IndexingNode* >::const_iterator jchild=jchildBEGIN;jchild!=jchildEND;jchild++)
				{	
					bool mappable=testMappable(*child,*jchild,currentStep);//replace with heirarchy level? TODO
					if(mappable)
					{
						(*(possibilityHeirarchy.rbegin()))[*child][*jchild]=true;
					}
				}
			}
		}
	}
}
void Searcher::refine(unsigned int steps)
{
	for(unsigned int currentStep=0; currentStep<=steps; currentStep++)
	{
		refineInner(currentStep);
	}
	return;
}
void Searcher::refineDFS(unsigned int steps, std::map<unsigned int, std::list<unsigned int> > &Mx)
{
	//create an ordering: BFS from a node
	MyNode* center=fragment->getNodeById(fragment->centerDiverseTotal()->getID());
	std::list<MyNode*> mappingOrder;
	mappingOrder.push_back(center);//start at some arbitrary node
	std::list<MyNode*>::iterator myItr=mappingOrder.begin();
	std::map<MyNode*,bool> alreadyInList;
	alreadyInList[*(mappingOrder.begin())]=true;
	std::vector<unsigned int> levelstarts;
	levelstarts.push_back(0);
	unsigned int mappingCount=0;
	unsigned int currentCount=1;
	//BFS
	args->T.start("BFS");
	while(myItr!=mappingOrder.end())
	{
		if(mappingCount>levelstarts.back() && currentCount<fragment->getNumNodes())
		{
			levelstarts.push_back(currentCount);
			fprintf(stderr,"Level Start = %d\n",levelstarts.back());
		}
		for(edgelist::iterator edgeItr=(*myItr)->getOutEdges()->begin();edgeItr!=(*myItr)->getOutEdges()->end();edgeItr++)
		{//add out edges
			if(alreadyInList.find(edgeItr->second.getNode())==alreadyInList.end())
			{//not in list yet
				alreadyInList[edgeItr->second.getNode()]=true;
				mappingOrder.push_back(edgeItr->second.getNode());
				currentCount++;
			}
		}
		for(edgelist::iterator edgeItr=(*myItr)->getInEdges()->begin();edgeItr!=(*myItr)->getInEdges()->end();edgeItr++)
		{//add in edges
			if(alreadyInList.find(edgeItr->second.getNode())==alreadyInList.end())
			{//not in list yet
				alreadyInList[edgeItr->second.getNode()]=true;
				mappingOrder.push_back(edgeItr->second.getNode());
				currentCount++;
			}
		}
		myItr++;
		mappingCount++;
	}
	args->T.stop("BFS");
	std::list<std::pair<unsigned int, std::list<IndexingNode*> > > tempMx;
	for(std::list<MyNode*>::iterator nodeItr=mappingOrder.begin();
		nodeItr!=mappingOrder.end();nodeItr++)
	{//create an ordering of nodes to examine
		std::list<IndexingNode*> tempList;
		tempList.push_back(index->getRoot());
		tempMx.push_back(std::pair<unsigned int, std::list<IndexingNode*> >((*nodeItr)->getID(),tempList));
	}
	//calc mx
unsigned int nodeNumber=0,levelNumber=0;
	for(std::list<std::pair<unsigned int, std::list<IndexingNode*> > >::iterator nodeItr=tempMx.begin();
		nodeItr!=tempMx.end();nodeItr++)
	{
		IndexingNode* fRoot=fragment->indexing.getRoot();
		//find if there is a neighbor to restrict this node
		unsigned int neighbor=0;
		unsigned int leastMX=0;
		bool outMark=false;//if an out neighbor is found, mark the in neighborhood of them
		for(edgelist::iterator a=fragment->getNodeById(nodeItr->first)->getOutEdges()->begin();
			a!=fragment->getNodeById(nodeItr->first)->getOutEdges()->end();a++)
		{
			if(Mx.find(a->first)!=Mx.end())
			{
				if(neighbor!=0 && Mx[a->first].size()>=leastMX)
				{
					continue;
				}
				neighbor=a->first;
				leastMX=Mx[a->first].size();
			}
		}
		for(edgelist::iterator a=fragment->getNodeById(nodeItr->first)->getInEdges()->begin();
			a!=fragment->getNodeById(nodeItr->first)->getInEdges()->end();a++)
		{
			if(Mx.find(a->first)!=Mx.end())
			{
				if(neighbor!=0 && Mx[a->first].size()>=leastMX)
				{
					continue;
				}
				neighbor=a->first;
				leastMX=Mx[a->first].size();
				outMark=true;//if an in-neighbor was found, mark the out-neighbors of them
			}
		}
		std::list<IndexingNode*> neighborList;
		if(neighbor!=0)
		{
			args->T.start("S:ToggleMark");
			neighborList=toggleMark(Mx[neighbor],neighbor,outMark);//mark
			args->T.stop("S:ToggleMark");
		}
		unsigned int threshold=100;
		if(neighborList.size()<threshold)
		{
			//do something here....
		}
		args->T.start("S:MxRestrict");
		for(unsigned int i=0;i<=steps;i++)
		{
			//find the next partition level of this node
			bool b=false;
			for(std::list<IndexingNode*>::iterator subPartitionItr=fRoot->subpartitions.begin();
				!b&&subPartitionItr!=fRoot->subpartitions.end();subPartitionItr++)
			{
				for(std::list<MyNode*>::iterator childItr=(*subPartitionItr)->nodes.begin();
					!b&&childItr!=(*subPartitionItr)->nodes.end();childItr++)
				{
					if((*childItr)->getID()==nodeItr->first)
					{
						b=true;
						fRoot=*subPartitionItr;
					}
				}
			}
			//we found the next partition level for this node
			std::list<IndexingNode*> theNewList;
			for(std::list<IndexingNode*>::iterator j=nodeItr->second.begin();j!=nodeItr->second.end();j++)
			{
				std::list<IndexingNode* >::const_iterator jchildBEGIN,jchildEND;
				jchildBEGIN=(*j)->getSubpartitions()->begin();
				jchildEND=(*j)->getSubpartitions()->end();
			
				for(std::list<IndexingNode* >::const_iterator jchild=jchildBEGIN;jchild!=jchildEND;jchild++)
				{	
					if(neighbor!=0 && (*jchild)->mark!=neighbor)
					{//if we are using restriction, but this option has no mark: skip it
						continue;
					}
					bool mappable=testMappable(fRoot,*jchild,i);//replace with heirarchy level? TODO
					if(mappable)
					{
						theNewList.push_back(*jchild);
					}
				}
			}
			nodeItr->second=theNewList;
		}
		args->T.stop("S:MxRestrict");
		/*if(neighbor!=0)
		{
			args->T.start("S:ToggleMark");
			toggleMark(Mx[neighbor],false,outMark);//unmark
			args->T.stop("S:ToggleMark");
		}*/
		//completedCalcs[nodeItr->first]=&nodeItr->second;
		{
			std::list<unsigned int> newList;
			for(std::list<IndexingNode*>::iterator optionItr=nodeItr->second.begin();optionItr!=nodeItr->second.end();optionItr++)
			{
				for(std::list<MyNode*>::iterator option=(*optionItr)->nodes.begin();
					option!=(*optionItr)->nodes.end();option++)
				{
					newList.push_back((*option)->getID());
				}
			}
			Mx[nodeItr->first]=newList;
		}
		//this is shrinking
		
		if(nodeNumber==levelstarts[levelNumber])
		{
		if(levelNumber>0)
		{
		args->T.start("S:shrink");
		shrinkUntilStable(Mx, fragment, database, args->mXoutput2);
		args->T.stop("S:shrink");
		}
		levelNumber++;
		}
		nodeNumber++;
	}
}
bool Searcher::toggleMark(std::list<IndexingNode*>* leafs, bool value, bool outNeighbors)
{
	for(std::list<IndexingNode*>::iterator itr=leafs->begin();itr!=leafs->end();
		itr++)
	{
		for(std::list<MyNode*>::iterator nodeItr=(*itr)->nodes.begin();nodeItr!=(*itr)->nodes.end();nodeItr++)
		{
			//for each node in MX
			//for each neighbor of those nodes
			edgelist::iterator B,E;
			if(outNeighbors){
				B=(*nodeItr)->getOutEdges()->begin();
				E=(*nodeItr)->getOutEdges()->end();
			}
			else
			{
				B=(*nodeItr)->getInEdges()->begin();
				E=(*nodeItr)->getInEdges()->end();
			}
			for(edgelist::iterator edgeItr=B;edgeItr!=E;edgeItr++)
			{
						
				IndexingNode* current=edgeItr->second.getNode()->partitionPtrs.back();
				current->mark=value;
				while(current->parent!=0)
				{//mark this node and it's parents
					current->parent->mark=value;
					current=current->parent;
				}
			}
		}
		//
	}
}

std::list<IndexingNode*> Searcher::toggleMark(std::list<unsigned int> leafs, unsigned int value, bool outNeighbors)
{
	std::list<IndexingNode*> returnVal;
	unsigned int numMarks=0;
	for(std::list<unsigned int>::iterator itr=leafs.begin();itr!=leafs.end();
		itr++)
	{
MyNode* thisnode=database->getNodeById(*itr);
		//for each node in MX
		//for each neighbor of those nodes
		edgelist::iterator B,E;
		if(outNeighbors){
			B=(thisnode)->getOutEdges()->begin();
			E=(thisnode)->getOutEdges()->end();
		}
		else
		{
			B=(thisnode)->getInEdges()->begin();
			E=(thisnode)->getInEdges()->end();
		}
		for(edgelist::iterator edgeItr=B;edgeItr!=E;edgeItr++)
		{
			IndexingNode* current=edgeItr->second.getNode()->partitionPtrs.back();
			current->mark=value;
			numMarks++;
			returnVal.push_back(current);
			while(current->parent!=0 && current->parent->mark!=value)
			{//mark this node and it's parents
				current->parent->mark=value;
				current=current->parent;
				numMarks++;
			}
		}
	}
	fprintf(stderr,"\tNeighbors=%d -> marks =%d \n",leafs.size(),numMarks);
	return returnVal;
}
bool Searcher::testMappable(IndexingNode* child, IndexingNode* jchild,unsigned int currentLevel)
{
	
	if(dominates(jchild, child,currentLevel))
	{
		return true;
	}
	return false;
}

bool Searcher::dominates(IndexingNode* vDom, IndexingNode* v2,unsigned int level)const
{
	std::map<edgeDescriptor, float>::const_iterator itr2=v2->getIndexValue()->getIDegree()->begin();
	for(;itr2!=v2->getIndexValue()->getIDegree()->end();itr2++)
	{
		std::map<edgeDescriptor, float>::const_iterator itr=vDom->getIndexValue()->getIDegree()->find(itr2->first);
		if(itr!=vDom->getIndexValue()->getIDegree()->end())
		{
			if(itr2->second <= itr->second)
			{//vdom has at least as many edges of this type as v2
				continue;
			}
		}
		return false;
	}
	return true;
}
unsigned int Searcher::initReducedMx(MxVector mx)
{
	unsigned int returnVal=0;
	fprintf(stderr,"ReduceMX\n");
	MxVector tempMX,newMX;
	std::unordered_map<MyNode*,IndexingNode*> nodeIndex;
	//init indexnodes
	//create a bunch of empty indexing nodes to be used below.
	if(reducedMx.size()==0)
	{
		for(MxVector::const_iterator itr=mx.begin();
			itr!=mx.end();itr++)
		{
			for(std::list<MyNode*>::const_iterator nodeItr=itr->first->getNodesBegin();
				nodeItr!=itr->first->getNodesEnd();nodeItr++)
			{//for each node in the fragment
				std::list<MyNode*>::const_iterator nodeItr2=nodeItr;
				nodeItr2++;
				//create a indexNode for each fragment node
				reducedMx.push_back(IndexingNode(0,nodeItr,nodeItr2,itr->first));
				tempMX[&reducedMx.back()]=itr->second;
				nodeIndex[*nodeItr]=&reducedMx.back();
				newMX[&reducedMx.back()]=MxOptions();
				reducedMxDB.push_back(IndexingNode());
				newMX[&reducedMx.back()][&reducedMxDB.back()]=true;
			}
		}
	}
	else
	{
		for(std::list<IndexingNode>::iterator itr=reducedMx.begin();itr!=reducedMx.end();
		itr++)
		{
			tempMX[&*itr]=mx[&*itr];//.second;
			nodeIndex[*(itr->getNodesBegin())]=&*itr;
			newMX[&*itr]=MxOptions();
			reducedMxDB.push_back(IndexingNode());
			newMX[&*itr][&reducedMxDB.back()]=true;
		}
	}
	std::unordered_map<const MyEdge*, std::unordered_map<MyNode*, bool> > masterTokens;
	
	std::unordered_map<const MyEdge*, std::unordered_map<MyNode*, bool> > masterINTokens;
	//run shrinking
	unsigned int e=0;
	for(MxVector::const_iterator itr=tempMX.begin();
		itr!=tempMX.end();itr++)
	{std::list<MyNode*>::const_iterator nodeItr=itr->first->getNodesBegin();
		//for each node in the fragment
			for(MxVector::const_iterator itr2=tempMX.begin();
				itr2!=tempMX.end();itr2++)
			{//find the neighbor, X'
				std::list<MyNode*>::const_iterator nodeItr2=itr2->first->getNodesBegin();
				
		for(edgelist::const_iterator edgeItr=(*nodeItr)->getOutEdgesConst()->begin();
			edgeItr!=(*nodeItr)->getOutEdgesConst()->end();edgeItr++)
		{//for each edge of X, X'
				//for each node in the fragment
				if((*nodeItr2)->getID()==edgeItr->first)
				{//this is the edge X->X'
					unsigned tokenCount=0;
					std::unordered_map<MyNode*,bool> tokens;//default empty
					//init tokens
					for(MxOptions::const_iterator optionItr=itr2->second.begin();
						optionItr!=itr2->second.end();optionItr++)
					{//for each option of X'
						for(std::list<MyNode*>::const_iterator tokenItr=optionItr->first->getNodesBegin();tokenItr!=optionItr->first->getNodesEnd();tokenItr++)
						{//for each option node of X', Y'
							tokens[*tokenItr]=true;
							tokenCount++;
						}
					}
					masterTokens[&(edgeItr->second)]=tokens;
				}
			}
			for(edgelist::const_iterator edgeItr=(*nodeItr)->getInEdgesConst()->begin();
			edgeItr!=(*nodeItr)->getInEdgesConst()->end();edgeItr++)
		{//for each edge of X, X'
				//for each node in the fragment
				if((*nodeItr2)->getID()==edgeItr->first)
				{//this is the edge X<-X'
					unsigned tokenCount=0;
					std::unordered_map<MyNode*,bool> tokens;//default empty
					//init tokens
					for(MxOptions::const_iterator optionItr=itr2->second.begin();
						optionItr!=itr2->second.end();optionItr++)
					{//for each option of X'
						for(std::list<MyNode*>::const_iterator tokenItr=optionItr->first->getNodesBegin();tokenItr!=optionItr->first->getNodesEnd();tokenItr++)
						{//for each option node of X', Y'
							tokens[*tokenItr]=true;
							tokenCount++;
						}
					}
					masterINTokens[&(edgeItr->second)]=tokens;
				}
			}
		}
	
	}
	
	for(MxVector::const_iterator itr=tempMX.begin();
		itr!=tempMX.end();itr++)
	{std::list<MyNode*>::const_iterator nodeItr=itr->first->getNodesBegin();
		//for each node in the fragment
		for(MxOptions::const_iterator optionItr=itr->second.begin();
						optionItr!=itr->second.end();optionItr++)
					{//for every option of X
						for(std::list<MyNode*>::const_iterator tokenItr=optionItr->first->getNodesBegin();
							tokenItr!=optionItr->first->getNodesEnd();tokenItr++)
						{//for every option node of X, Y
						
					bool found=true;
		for(edgelist::const_iterator edgeItr=(*nodeItr)->getOutEdgesConst()->begin();
			found && edgeItr!=(*nodeItr)->getOutEdgesConst()->end();edgeItr++)
		{//for each edge of X, X'
			for(MxVector::const_iterator itr2=tempMX.begin();
				found&& itr2!=tempMX.end();itr2++)
			{//find the neighbor, X'
				std::list<MyNode*>::const_iterator nodeItr2=itr2->first->getNodesBegin();
				//for each node in the fragment
				if((*nodeItr2)->getID()==edgeItr->first)
				{//this is the edge X->X'
					//check this edge
					unsigned tokenCount=0;
					bool tempFound=false;
							for(edgelist::const_iterator edgeItr2=(*tokenItr)->getOutEdgesConst()->begin();
								edgeItr2!=(*tokenItr)->getOutEdgesConst()->end();edgeItr2++)
							{//for each edge of Y->Y'
								if(masterTokens[&(edgeItr->second)].find(edgeItr2->second.getNode())!=masterTokens[&(edgeItr->second)].end())
								{
									tokenCount++;
									tempFound=true;
									break;
								}
							}
							if(!tempFound)
							{
								found=false;
							}
						}
					}
				}
		for(edgelist::const_iterator edgeItr=(*nodeItr)->getInEdgesConst()->begin();
			found && edgeItr!=(*nodeItr)->getInEdgesConst()->end();edgeItr++)
		{//for each edge of X, X'
			for(MxVector::const_iterator itr2=tempMX.begin();
				found&& itr2!=tempMX.end();itr2++)
			{//find the neighbor, X'
				std::list<MyNode*>::const_iterator nodeItr2=itr2->first->getNodesBegin();
				//for each node in the fragment
				if((*nodeItr2)->getID()==edgeItr->first)
				{//this is the edge X->X'
					//check this edge
					unsigned tokenCount=0;
					bool tempFound=false;
							for(edgelist::const_iterator edgeItr2=(*tokenItr)->getInEdgesConst()->begin();
								edgeItr2!=(*tokenItr)->getInEdgesConst()->end();edgeItr2++)
							{//for each edge of Y->Y'
								if(masterINTokens[&(edgeItr->second)].find(edgeItr2->second.getNode())!=masterINTokens[&(edgeItr->second)].end())
								{
									tokenCount++;
									tempFound=true;
									break;
								}
							}
							if(!tempFound)
							{
								found=false;
							}
						}
					}
				}
				if(found)
				{
					std::list<MyNode*>::const_iterator tokenItr2=tokenItr;
					tokenItr2++;
					newMX[nodeIndex[(*nodeItr)]].begin()->first->addNodes(tokenItr,tokenItr2);
					returnVal++;
				}
			}
		}
	}
	possibilityHeirarchy.push_back(newMX);
	return returnVal;
}

CandidateList Searcher::mapFragment(MyGraph* database,arguments& args)
{
	args.T.start("M: init");
	CandidateList candidates;
	//get a count of the M(x) sizes 
	std::vector<std::pair<unsigned int, unsigned int> > mxsizes;
	for(std::map<unsigned int, std::list<unsigned int> >::iterator itr= MxFinished.begin();
		itr!=MxFinished.end();itr++)
	{
		mxsizes.push_back((std::pair<unsigned int, unsigned int>(itr->second.size(),itr->first)));
	}//size first, then node ID
	//  std::map<unsigned int, std::list<unsigned int> > Mx;
	//sort the partitions by M(x) sizes.
	std::sort(mxsizes.begin(),mxsizes.end(),mxSort);
	std::queue<std::map<unsigned int, unsigned int> > partialMappings;
	
	std::vector<std::pair<unsigned int, unsigned int> >::const_iterator firstItr=mxsizes.begin();
	std::map<unsigned int, std::list<unsigned int> >::const_iterator mVecItr=MxFinished.find(firstItr->second);
	//std::list<MyNode*>::const_iterator FnodeItr=firstItr->second->getNodesBegin();
	
	if(mVecItr==MxFinished.end())
	{
		fprintf(stderr,"A node has no possibilities\n");
		return candidates;
	}
	//init some maps
	const unsigned int MaxMaps=args.maxMaps;
	//unsigned int iCount=0;
	for(std::list<unsigned int>::const_iterator DnodeItr=mVecItr->second.begin();
		partialMappings.size()< MaxMaps
		&&DnodeItr!=mVecItr->second.end();DnodeItr++)
	{//for each node in the mapping partition
		std::map<unsigned int,unsigned int> tempMap;
		tempMap[mVecItr->first]=*DnodeItr;
		partialMappings.push(tempMap);
	}
	
	//fprintf(stderr,"mappings=%d\n",partialMappings.size());
	//elaborate some maps
	std::map<unsigned int, unsigned int> tempMap;
	while(partialMappings.size()<MaxMaps && partialMappings.size()>0)
	{//fprintf(stderr,"mappings=%d\n",partialMappings.size());
		tempMap=partialMappings.front();
		partialMappings.pop();
		//try to map next unmapped
		bool breakBool=false;
		for(std::vector<std::pair<unsigned int, unsigned int> >::const_iterator itr=mxsizes.begin();!breakBool&& itr!=mxsizes.end();itr++)
		{//for each partition, in order of Mxsize
			std::map<unsigned int, std::list<unsigned int> >::const_iterator mVecItr=MxFinished.find(itr->second);
			MyNode* FnodeItr=fragment->getNodeById(mVecItr->first);
			//for each node in this partition
			if(tempMap.find(FnodeItr->getID())==tempMap.end())
			{
				for(std::list<unsigned int>::const_iterator jtr=mVecItr->second.begin();
					!breakBool && jtr!=mVecItr->second.end();jtr++)
				{//for each partition the fragment partition can map to
					unsigned int DnodeItr=*jtr;
					//for each node in the mapping partition
					tempMap[FnodeItr->getID()]=DnodeItr;
					float score= evaluatePartialMap(tempMap,fragment,database,true);
					if(score>=0 && score <1)
					{
						partialMappings.push(tempMap);
					}
					if(partialMappings.size()>=MaxMaps)
					{
						breakBool=true;
					}
				}
			breakBool=true;
			}
		}
		
		if(breakBool==true)
		{continue;}
		else
		{break;}
	}
	args.T.stop("M: init");
	args.T.start("M: greedy");
	//perform greedy mapping, estimate score, and return
	while(partialMappings.size()>0)
	{args.T.start("++While");
		bool invalid=false;
		fprintf(stderr,"numpartials =%d\n",partialMappings.size());
		fflush(stderr);
		tempMap=partialMappings.front();
		partialMappings.pop();
		//greedy map the remaining
		bool unMapped=false;
		std::vector<unsigned int> occupiedNodes;
		for(std::vector<std::pair<unsigned int, unsigned int> >::const_iterator itr=mxsizes.begin();
			!invalid && itr!=mxsizes.end();itr++)
		{//for each partition, in order of Mxsize
			std::map<unsigned int, std::list<unsigned int> >::const_iterator mVecItr=MxFinished.find(itr->second);
			MyNode* FnodeItr=fragment->getNodeById(mVecItr->first);
			//for each node in this partition
			std::map<unsigned int, unsigned int>::iterator isMappedItr;
			if(!unMapped)
			{
				isMappedItr=tempMap.find(FnodeItr->getID());
				unMapped=(isMappedItr==tempMap.end());
			}
			if(unMapped)
			{
				unMapped=true;
				unsigned int bestID=0;
				float bestScore=-100;
				for(std::list<unsigned int>::const_iterator jtr=mVecItr->second.begin();
					jtr!=mVecItr->second.end();jtr++)
				{//for each partition the fragment partition can map to
					unsigned int DnodeItr=*jtr;
					//for each node in the mapping partition
					bool alreadyMapped=false;
					for(std::vector<unsigned int>::iterator tempX=occupiedNodes.begin();tempX!=occupiedNodes.end();tempX++)
					{
						if((*tempX)==DnodeItr)
						{
							alreadyMapped=true;
							break;
						}
					}
					if(alreadyMapped)
					{continue;}
					tempMap[FnodeItr->getID()]=DnodeItr;
					args.T.start("onenode");
					float tempScore=evaluateOneNode(tempMap,fragment,database,FnodeItr,bestScore);
					args.T.stop("onenode");
					if(bestScore==-100 ||(tempScore!=-1 && tempScore<bestScore))
					{
						bestID=DnodeItr;
						bestScore=tempScore;
					}
				}
				if(bestScore<0 || bestScore>=1)
				{
					//invalid=true;
					//continue;
				}
				tempMap[FnodeItr->getID()]=bestID;
				occupiedNodes.push_back(bestID);
			}
			else
			{
				occupiedNodes.push_back(isMappedItr->second); 
			}
		}
		
		//calculate a score of somekind
		args.T.start("M:partial");
		float score=evaluatePartialMap(tempMap,fragment,database);
		args.T.stop("M:partial");
		if(score< 1 && score >= 0)
		{
			candidates.push_back(std::pair<double, std::map<unsigned int, unsigned int> >(score, tempMap));
		}
	}
	args.T.stop("M: greedy");
	sort(candidates.begin(),candidates.end());
	return candidates;
}

float Searcher::evaluatePartialMap(std::map<unsigned int, unsigned int>& M, MyGraph* frag, MyGraph* database, bool noError)
{
	float returnVal=0;
	for(std::vector<MyNode*>::iterator itr=frag->nodeSorted.begin();itr!=frag->nodeSorted.end();itr++)
	{
		float temp=evaluateOneNode(M,frag,database,*itr, -100, noError);
		returnVal+=temp;
		if(noError && temp>=1)
		{
			return returnVal;
		}
	}
	return returnVal;
}
float Searcher::evaluateOneNode(std::map<unsigned int, unsigned int>& M, MyGraph* frag, MyGraph* database, MyNode* node,float bestScore,bool noError)
{
	float returnVal=0;
	std::map<unsigned int, unsigned int>::iterator node1Itr=M.find(node->getID());
	if(node1Itr==M.end())
	{
		return returnVal;//this node isn't in the mapping?
	}
	const edgelist* E=node->getOutEdgesConst();
	for(edgelist::const_iterator edgeItr=E->begin();edgeItr!=E->end();edgeItr++)
	{//for each out neighbor of this node
		std::map<unsigned int, unsigned int>::iterator node2Itr=M.find((edgeItr)->first);
		if( node2Itr==M.end())
		{
			continue;
		}
		if(!database->isEdge(node1Itr->second,node2Itr->second))
		{
			returnVal+=1;
			if(bestScore!=-100 && returnVal>bestScore)
			{
				return returnVal;
			}
			if(noError && returnVal>=1)
			{
				return returnVal;
			}
		}
	}
	if(args->similarityMethod=='F')
	{
		E=node->getInEdgesConst();
		for(edgelist::const_iterator edgeItr=E->begin();edgeItr!=E->end();edgeItr++)
		{//for each out neighbor of this node
			std::map<unsigned int, unsigned int>::iterator node2Itr=M.find((edgeItr)->first);
			if( node2Itr==M.end())
			{
				continue;
			}
			if(!database->isEdge(node2Itr->second,node1Itr->second))
			{
				returnVal+=1;
				if(bestScore!=-100 && returnVal>bestScore)
				{
					return returnVal;
				}
				if(noError && returnVal>=1)
				{
					return returnVal;
				}
			}
		}
	}
	return returnVal;
}



bool Searcher::mxReduce(std::map<unsigned int, std::list<unsigned int> >& Mx,MyGraph* fragment, MyGraph* database,FILE* output)
{
	for(std::map<unsigned int, std::list<unsigned int> >::iterator nodeitr=Mx.begin();nodeitr!=Mx.end();nodeitr++)
	{//for each node that can be mapped
		//let M(x)==y1...yk
		MyNode* node=fragment->getNodeById(nodeitr->first);
		edgelist* edges=node->getOutEdges();
		for(edgelist::iterator edgeItr=edges->begin();
		edgeItr!=edges->end();edgeItr++)
		{//for every x' member E(x)
			std::map<unsigned int, bool> tokens;
			std::map<unsigned int, std::list<unsigned int> >::iterator MxFindEdge=Mx.find(edgeItr->first);
			if(MxFindEdge==Mx.end() || MxFindEdge->second.size()==0)
			{//this edge is not being considered in Mx
				continue;
			}
			std::list<unsigned int>::iterator MxPrimeItr=MxFindEdge->second.begin();
			for(;MxPrimeItr!=MxFindEdge->second.end();MxPrimeItr++)
			{//label special every node in M(x')
				tokens[*MxPrimeItr]=true;
			}
			std::list<unsigned int> Mnew;
			for(std::list<unsigned int>::iterator MxItr=nodeitr->second.begin();MxItr!=nodeitr->second.end();MxItr++)
			{//for every yi in M(x)
				MyNode* yi=database->getNodeById(*MxItr);
				edgelist* yiEdges=yi->getOutEdges();
				edgelist::iterator r=yiEdges->begin();
				while(r!=yiEdges->end())
				{
					if(tokens.find(r->first)==tokens.end())
					{
						r++;
					}
					else
					{//there was a correct edge found.
						break;
					}
				}
				if(r!=yiEdges->end())
				{
					Mnew.push_back(*MxItr);
				}
			}
			//unlabel(automatic out of scope)
			nodeitr->second=Mnew;//M(x)=Mnew
		}
		//do in edges
		edges=node->getInEdges();
		for(edgelist::iterator edgeItr=edges->begin();
		edgeItr!=edges->end();edgeItr++)
		{//for every x' member E(x)
			std::map<unsigned int, bool> tokens;
			
			std::map<unsigned int, std::list<unsigned int> >::iterator MxFindEdge=Mx.find(edgeItr->first);
			if(MxFindEdge==Mx.end() || MxFindEdge->second.size()==0)
			{//this edge is not being considered in Mx
				continue;
			}
			std::list<unsigned int>::iterator MxPrimeItr=MxFindEdge->second.begin();
			for(;MxPrimeItr!=MxFindEdge->second.end();MxPrimeItr++)
			{//label special every node in M(x')
				tokens[*MxPrimeItr]=true;
			}
			
			std::list<unsigned int> Mnew;
			for(std::list<unsigned int>::iterator MxItr=nodeitr->second.begin();MxItr!=nodeitr->second.end();MxItr++)
			{//for every yi in M(x)
				MyNode* yi=database->getNodeById(*MxItr);
				edgelist* yiEdges=yi->getInEdges();
				edgelist::iterator r=yiEdges->begin();
				while(r!=yiEdges->end())
				{
					if(tokens.find(r->first)==tokens.end())
					{
						r++;
					}
					else
					{//there was a correct edge found.
						break;
					}
				}
				if(r!=yiEdges->end())
				{
					Mnew.push_back(*MxItr);
				}
			}
			//unlabel(automatic out of scope)
			nodeitr->second=Mnew;//M(x)=Mnew
		}
	}
}