#include "stats_keeper.h"

#include <cmath>

void stats_keeper::update(double d)
{
	std::map<double, unsigned int>::iterator it = this->countNumbers.find(d);
	
	if(it == this->countNumbers.end() )
	{
		this->countNumbers[d] = 1;
	}
	else
	{
		it->second++;
	}
}

double stats_keeper::getMode()const
{
	double result = 0; 
	unsigned int max = 0; 

	for(std::map<double, unsigned int>::const_iterator it = this->countNumbers.begin(); 
		it != this->countNumbers.end(); ++it)
	{
		if(it->second > max)
		{
			max = it->second; 
			result = it->first;
		}
	}

	return result;
} 

double stats_keeper::getAverage()const
{
	double result = 0; 
	unsigned int count = this->getSampleSize();
	
	for(std::map<double, unsigned int>::const_iterator it = this->countNumbers.begin(); 
		it != this->countNumbers.end(); ++it)
	{
		result += it->first * it->second;
	}

	if(count > 0)
	{
		result /= (float)count;
	}

	return result;
}  

double stats_keeper::getStdDev()const
{
	double result = 0;
	double sum = 0; 
	double sumSquares = 0; 
	unsigned int count=0;
	
	for(std::map<double, unsigned int>::const_iterator it = this->countNumbers.begin(); 
		it != this->countNumbers.end(); ++it)
	{
		sum += it->first*it->second;
		sumSquares += it->first*it->first*it->second;
		count += it->second;
	}
	
	if(count > 0)
	{
		result = (sumSquares - sum*sum / count) /count;
		if(result < 0)
		{ 
			result *= -1;
		}

		result = sqrt(result);
	}

	return result;
} 

unsigned int stats_keeper::getSampleSize()const
{
	unsigned int result = 0;
	for(std::map<double, unsigned int>::const_iterator it = this->countNumbers.begin(); 
		it != this->countNumbers.end(); ++it)
	{
		result += it->second;
	}
	return result;
} 

double stats_keeper::getMedian()const
{
	double median = 0;
	unsigned int count = this->getSampleSize(); 
	unsigned int currentCount = 0; 
	bool finished = false;

	for(std::map<double, unsigned int>::const_iterator it = this->countNumbers.begin(); 
		it != this->countNumbers.end()&&!finished; ++it)
	{
		currentCount += it->second;

		if(currentCount >= ((count+1)/2.0) )
		{
			median = it->first;

			if(count%2 == 0)//if there are an even numbers median is average if count = 2 [1 2]
			{
				if( (count+2)/2 > currentCount)//list is not [1 1]
				{
					it++;
				}
				median += it->first;
				median /= 2.0;
			}

			finished = true;
		}
	}

	return median;
} 

double stats_keeper::getMin()const
{
	double min = 0; 

	if(this->countNumbers.size() > 0)
	{
		min = this->countNumbers.begin()->first;
	}

	return min;
} 

double stats_keeper::getMax()const
{
	double max = 0;

	if(this->countNumbers.size() > 0)
	{
		max = this->countNumbers.rbegin()->first;
	}

	return max;
} 
