#include <algorithm>
#include <cstring>
#include <cstdlib>
#include <iostream>
#include <math.h>
//#include <stdargs.h>

#include "utilities.h"
#include <boost/random.hpp>
#include <boost/random/normal_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>

std::string truncateString(std::string a)
{
	if (a.size()>UCHAR_MAX) 
	{
		a=a.substr(0,UCHAR_MAX);
	}
	return a;
}

char* stringToCString(std::string a)
{
	char* b=0;
	if (a.size() > 0)
	{
		b=new char[a.size()+1];	//add 1 for the null terminator
		strcpy(b,a.c_str());
	}
	return b;
}

void printTrace(FILE* f, const char *fmt, ...)
{
#ifdef TRACE_find_by
    std::va_list args;
    va_start(args, fmt);
    fprintf(f, fmt, args);
	fflush(f);
#endif
}
std::string my_itoa(int value, int base) {
		std::string buf;

		enum { kMaxDigits = 35 };
		buf.reserve( kMaxDigits ); // Pre-allocate enough space.
	
		int quotient = value;
	
		// Translating number to string with base:
		do {
			buf += "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"[ std::abs( quotient % base ) ];
			quotient /= base;
		} while ( quotient );
	
		// Append the negative sign
		if ( value < 0) buf += '-';
	
		std::reverse( buf.begin(), buf.end() );
		return buf;
	}
	
void printDegree(const MyNode *n, FILE * f)
{
	fprintf(f,"%u, %u, %u %s\n", n->getID(), n->getType(), (unsigned)n->getNumOutNeighbors(),n->getLabel());
	for(unsigned int i = 0; i < 1; ++i)
	{
		const iDeg* nDeg=n->getIDegree(i+1,false);
		if(nDeg!=0)
		{
			iDegreeList::const_iterator itr=nDeg->getList()->begin();
			for(; itr !=nDeg->getList()->end();itr++)
			{
				fprintf(f, "<%u,%u>,", (unsigned int)itr->first,(unsigned int)itr->second);
			}
		}
		fprintf(f, "\n");
	}
	for(unsigned int i = 0; i < 1; ++i)
	{
		const iDeg* nDeg=n->getIDegree(i+1,true);
		if(nDeg!=0)
		{
			iDegreeList::const_iterator itr=nDeg->getList()->begin();
			for(; itr !=nDeg->getList()->end();itr++)
			{
				fprintf(f, "<%u,%u>,", (unsigned int)itr->first,(unsigned int)itr->second);
			}
		}
		fprintf(f, "\n");
	}
	fprintf(f,"\n");
}

//returns true if r has non-zeros in the spots (and possible others) as l
//r covers l i this is true.
bool covers(const std::vector<float>& l, const std::vector<float>& r)
{
	bool result = true;

	for(unsigned int i = 0; i < l.size() && i < r.size() && result; ++i)
	{
		if( (r[i]==0) && l[i]>0)
		{
			result = false;
		}
	}
	return result;
}

//Does r dominate l?
//r must have >= neighbors than l in all types.
bool dominates(const std::vector<float>&l, const std::vector<float>&r)
{
	bool result = true;

	for(unsigned int i = 0; i < l.size() && i < r.size() && result; ++i)
	{
		if( r[i] < l[i])
		{
			result = false;
		}
	}

	return result;
}
float metricDist(float a, float b, char Metric)
{
	switch(Metric){
	case '1':
		return a-b;
		break;
	case '2':
		return a*a-b*b;
		break;
	case 'S':
		if(a<b)
			{return -sqrt(b-a);}
		return sqrt(a-b);
		break;
	default:
		fprintf(stderr,"Unknown metric for Distance calulation...\n");
		return -1;
	}
	
	
}
float dominatingDist(const std::vector<float>&pm1, const std::vector<float>& pm2, struct arguments args)
{
	float difference = 0;
	float d;

	for(unsigned int i= 0; i<pm1.size() && i < pm2.size(); i++)
	{
		d = metricDist(pm1[i],pm2[i],args.distMethod);
		if(d<0)
		{
			difference = -1; 
			break;
		}
		difference += d;
	}
	return difference;
}
float dominatingDist(const iDegreeList* pm1, const iDegreeList* pm2, struct arguments args)
{
	float difference = 0;
	float d(0);
	iDegreeList bad;
	iDegreeList::const_iterator i;
	if(pm1==NULL)
	{
		//fprintf(stdout,"pm1bad\n");
		i=bad.begin();
	}
	else
	{	i=pm1->begin();
	}
	iDegreeList::const_iterator j;
	if(pm2==NULL)
	{
		//fprintf(stdout,"pm2bad\n");
		j=bad.begin();
	}
	else
	{	j=pm2->begin();
	}
	bool iBad=i==bad.begin()||i==pm1->end();
	bool jBad=j==bad.begin()||j==pm2->end();
	//fprintf(stdout,"ibad jbad= %d %d\n",iBad,jBad);
	for(;!iBad || !jBad;)
	{
		if(!iBad && !jBad && i->first ==j->first)
		{
			d = metricDist(i->second,j->second,args.distMethod);
			//fprintf(stdout,"a, |%d=%d| %f= %f %f\n",i->first,j->first,d,i->second, j->second);
			i++;
			j++;
		}
		else if(!iBad && (jBad || i->first < j->first))
		{
			d = metricDist(i->second,0.0,args.distMethod);
			
			unsigned int t=1337;
			if(!jBad)
				{t=j->first;}
			//fprintf(stdout,"b, |%d<%d| %f= %f %f\n",i->first,t,d,i->second, 0.0);
			i++;
		}
		else if(!jBad && (iBad || i->first > j->first))
		{
			d = metricDist(0.0,j->second,args.distMethod);
			unsigned int t=1337;
			if(!jBad)
				{t=j->first;}
			//fprintf(stdout,"c, |%d>%d| %f= %f %f\n",t,j->first,d,0.0, j->second);
			j++;
		}
		else
		{
			fprintf(stderr,"Not supposed to get here...\n");
		}
		if(d<0)
		{
			//fprintf(stdout,"d<0\n");
			difference = -1; 
			//exit(1);
			break;
		}
		difference += d;
		iBad=iBad||i==pm1->end();
		jBad=jBad||j==pm2->end();
	}
	return difference;
}

float dominatingDist(const MyNode* l, const MyNode* r, unsigned int level, struct arguments args)
{
	//printDegree(l,stdout);
	//printDegree(r,stdout);
	float diff = -1;
	if(l->getType() == r->getType())
	{
		diff = 0;
		for(unsigned int i = 0; i < level; ++i)
		{	
			//fprintf(stdout,"Domdist\n");
			float d = args.wO[i]*dominatingDist(l->getIDegree(i+1,false)->getList(),r->getIDegree(i+1,false)->getList(),args);
			//float d = args.wO[i]*dominatingDist(l->iDegree[i], r->iDegree[i],Metric,args);
			if( d < 0)
			{
				//fprintf(stdout,"2) d<0\n");
				diff = -1;
				break;
			}
			diff += d;
		}
	}
	else
	{
		//fprintf(stdout,"3) type unequal\n");
	}
	return diff;
}
float dominatingInDist(const MyNode* l, const MyNode* r, unsigned int level,char Metric,struct arguments args)
{
	float diff = -1;
	if(l->getType() == r->getType())
	{
		diff = 0;
		for(unsigned int i = 0; i < level; ++i)
		{
			float d = args.wI[i]*dominatingDist(l->getIDegree(i+1,true)->getList(),r->getIDegree(i+1,true)->getList(),args);
			//float d = args.wI[i]*dominatingDist(l->iniDegree[i], r->iniDegree[i],Metric,args);
			if( d < 0)
			{
				diff = -1;
				break;
			}
			diff += d;
		}
	}

	return diff;
}

float dist(const std::vector<float>& pm1, const std::vector<float>& pm2,char Metric,struct arguments args)
{
	float difference = 0;
	float d;

	for(unsigned int i= 0; i<pm1.size() && i < pm2.size(); i++)
	{
		d = metricDist(pm1[i],pm2[i],Metric);
		if(d<0)
			d*=-1;
		difference += d;
	}
	return difference;
}
float dist(const iDegreeList* pm1, const iDegreeList* pm2,char Metric,struct arguments args)
{
	float difference = 0;
	float d;
	iDegreeList::const_iterator i=pm1->begin();
	iDegreeList::const_iterator j=pm2->begin();
	for(;i!=pm1->end();)
	{
		while(j!=pm2->end() && i->first > j->first)
		{
			j++;
		}
		if(j==pm2->end() || i->first != j->first)
		{
			d = metricDist(i->second,0,Metric);
			if(d<0)
				d*=-1;
			difference += d;
		}
		else
		{
			d = metricDist(i->second,j->second,Metric);
			if(d<0)
				d*=-1;
			difference += d;
		}
		i++;
	}
	return difference;
}

float weightDist(const MyNode *l, const MyNode* r, unsigned int level,char Metric,struct arguments args)
{	
	float diff = -1;
	if(l->getType() == r->getType())
	{
		diff = 0;
		float w=1.0;
		for(unsigned i=1;i<=level;i++)
		{
			diff+=w*dist(l->getIDegree(i,false)->getList(),r->getIDegree(i,false)->getList(),Metric,args);
			w*=.5;
		}
		/*switch(level)
		{
		case 3:
			diff += 1*dist(l->iDegree[2], r->iDegree[2], Metric,args);
		case 2:
			diff += 2*dist(l->iDegree[1], r->iDegree[1], Metric,args);
		case 1:
			diff += 4*dist(l->iDegree[0], r->iDegree[0], Metric,args);
		}*/
	}

	return diff;
}

float dist(const MyNode* l, const MyNode* r, unsigned int level, char Metric,struct arguments args)
{
	float diff = -1;
	if(l->getType() == r->getType())
	{
		diff = 0;
		for(unsigned int i=1;i<=level;i++)
		{
			diff+=dist(l->getIDegree(i,false)->getList(),r->getIDegree(i,false)->getList(),Metric,args);
		}
		/*
		switch(level)
		{
		case 3:
			diff += dist(l->iDegree[2], r->iDegree[2], Metric,args);
		case 2:
			diff += dist(l->iDegree[1], r->iDegree[1], Metric,args);
		case 1:
			diff += dist(l->iDegree[0], r->iDegree[0], Metric,args);
		}*/
	}

	return diff;
}



float coverDist(const std::vector<float>& pm1, const std::vector<float>& pm2, char Metric,struct arguments args)
{
	float difference = 0;
	float d;

	for(unsigned int i = 0; i < pm1.size() && i < pm2.size(); ++i)
	{
		if(pm1[i] > pm2[i])
		{
			d = metricDist(pm1[i],pm2[i],Metric);
			if(d<0)
				d*=-1;
			difference += d;
		}
	}
	return difference;
}
float coverDist(const iDegreeList* pm1, const iDegreeList* pm2, char Metric,struct arguments args)
{
	float difference = 0;
	float d;
	iDegreeList::const_iterator i=pm1->begin();
	iDegreeList::const_iterator j=pm2->begin();
	for(; i !=pm1->end();)
	{
		while(j!= pm2->end() && i->first > j->first)
		{
			j++;
		}
		if(j==pm2->end()||i->first != j->first)
		{
			d = metricDist(i->second,0,Metric);
			if(d<0)
				d*=-1;
			difference += d;
		}
		else if(i->second > j->second)
		{
			d = metricDist(i->second,j->second,Metric);
			if(d<0)
				d*=-1;
			difference += d;
		}
		i++;
	}
	return difference;
}


float coverDist(const MyNode* l, const MyNode * r, unsigned int level,char Metric,struct arguments args)
{
	float diff = -1;
	if(l->getType() == r->getType())
	{
		diff = 0;
		for(unsigned int i=1;i<=level;i++)
		{
			diff+=coverDist(l->getIDegree(i,false)->getList(),r->getIDegree(i,false)->getList(),Metric,args);
		}
		/*
		switch(level)
		{
		case 3:
			diff += coverDist(l->iDegree[2], r->iDegree[2], Metric,args);
		case 2:
			diff += coverDist(l->iDegree[1], r->iDegree[1], Metric,args);
		case 1:
			diff += coverDist(l->iDegree[0], r->iDegree[0], Metric,args);
		}*/
	}

	return diff;
}

float nonZeroDist(const std::vector<float>& pm1, const std::vector<float>& pm2, char Metric,struct arguments args)
{
	float difference = 0;
	float d;

	for(unsigned int i = 0; i < pm1.size() && i < pm2.size(); ++i)
	{
		if(pm1[i] > 0)
		{
			d = metricDist(pm1[i],pm2[i],Metric);
			if(d<0)
				d*=-1;
			difference += d;
		}
	}
	return difference;
}
float nonZeroDist(const iDegreeList* pm1, const iDegreeList* pm2, char Metric,struct arguments args)
{
	float difference = 0;
	float d=0;
	iDegreeList::const_iterator i=pm1->begin();
	iDegreeList::const_iterator j=pm2->begin();
	for(; i !=pm1->end();)
	{
		while(j!=pm2->end() && i->first > j->first)
		{
			j++;
		}
		if(j==pm2->end() || i->first!=j->first)
		{		
			d = metricDist(i->second,0,Metric);
			if(d<0)
				d*=-1;
			difference += d;
		}
		else
		{
			d = metricDist(i->second,j->second,Metric);
			if(d<0)
				d*=-1;
			difference += d;
		}
		
		i++;
	}
	return difference;
}


float nonZeroDist(const MyNode* l, const MyNode * r, unsigned int level,char Metric,struct arguments args)
{
	float diff = -1;
	if(l->getType() == r->getType())
	{
		diff = 0;
		for(unsigned int i=1;i<=level;i++)
		{
			diff += nonZeroDist(l->getIDegree(i,false)->getList(),r->getIDegree(i,false)->getList(),Metric,args);
		}
		/*switch(level)
		{
		case 3:
			diff += nonZeroDist(l->iDegree[2], r->iDegree[2], Metric,args);
		case 2:
			diff += nonZeroDist(l->iDegree[1], r->iDegree[1], Metric,args);
		case 1:
			diff += nonZeroDist(l->iDegree[0], r->iDegree[0], Metric,args);
		}*/
	}

	return diff;
}

float similarity(const MyNode* l, const MyNode * r, char method, unsigned int i_degree, char Metric,struct arguments args)
{
	float d = -2;
	float temp =-1;
	switch(method)
	{
	case 'F':
		temp = dominatingDist(l, r, i_degree,args);
		d=temp;
		if(temp<0.0)
		{
			break;
		}
		temp=dominatingInDist(l,r,i_degree,Metric,args);
		if(temp<0.0)
		{
			d=temp;
			break;
		}
		d+=temp;
		break;
	case 'f':
		d = dominatingDist(l, r, i_degree,args);
		break;
	case 'd':
		d = dist(l, r, i_degree,Metric,args); 
		break;
	case 'c':
		d = coverDist(l, r, i_degree,Metric,args);
		break;
	case 'n':
		d = nonZeroDist(l, r, i_degree, Metric,args);
		break;
	case 'w':
		d = weightDist(l, r, i_degree, Metric,args);	
		break;
	case 'r':
		d = rand();
		break;
	default:
		printf("\nUnknown distance measure %c", method);
		break;
	}
	return d;
}

//r is confused with l if r is at most max different than l.
bool differsByAtMost(const std::vector<float>&l, const std::vector<float>&r, float max, char Metric,struct arguments args)
{
	bool result = (dist(l, r, Metric,args) <= max);
	return result;
}


//Used for sorting two nodes by their 0 deg and 1deg.
int compare(const MyNode*  l, const MyNode * r, unsigned int i)
{
	int result = -1;
	if(l->getType() > r->getType())
	{
		result = 1;
	}
	else if(l->getType() == r->getType())
	{
		result = 0;
		for(unsigned int j = 1; j <= i && result == 0; ++j)
		{
			const iDegreeList* lList=l->getIDegree(j,false)->getList();
			const iDegreeList* rList=r->getIDegree(j,false)->getList();
			result = compare(lList, rList);
		}
	}//else
	if(result ==0)
	{
		for(unsigned int j = 1; j <= i && result == 0; ++j)
		{
			result = compare(l->getIDegree(j,true)->getList(), r->getIDegree(j,true)->getList());
		}
	}
	return result;
}

// compares the two type vectors
//	returns: 
//			-1 if l < r
//			 0 if l == r
//			 1 if l > r
/*std::vector<float> getPosition(Node & n, Graph & g)
{
return g.partitions[n.type].partitions_1[n.partition1ptr].onedegree.second;
}*/
int compare(const std::vector<float>& l, const std::vector<float>& r)
{
	int result = 0;
	for(unsigned int i = 0; i < l.size() && i < r.size() && result == 0; ++i)
	{
		if(l[i] > r[i])
		{
			result = 1;

		}
		else if(l[i] < r[i])
		{
			result = -1;
		}
	}

	return result;
}//compare type vectors
int compare(const iDegreeList* l, const iDegreeList* r)
{
	if(l==0 && r == 0)
	{
		return 0;
	}
	else if(0==l)
	{
		return -1;
	}
	else if(0==r)
	{
		return 1;
	}
	int result = 0;
	iDegreeList::const_iterator i = l->begin();
	iDegreeList::const_iterator j = r->begin();
	for(; i != l->end() && j != r->end() && result == 0; )
	{
		if(i->first == j->first)
		{
			if(i->second > j->second)
			{
				return 1;
			}
			else if(i->second < j->second)
			{
				return -1;
			}
		}
		else if(i->first > j->first)
		{
			return -1;
		}
		else if(i->first < j->first)
		{
			return 1;
		}
		i++;
		j++;
	}
	if(i == l->end() && j == r->end())
	{
		return 0;
	}
	else if(i==l->end())
	{
		return -1;
	}
	else if(j==r->end())
	{
		return 1;
	}
	return result;
}//compare type vectors


//returns true if left comes before right
bool lexFirst(const MyNode* l, const MyNode* r)
{
	return compare(l, r, 3) < 0;
}
//generate a random float between 0 and 1
float randDouble()
{
	return ((float)rand())/(1.0+(float)RAND_MAX);
}

//returns the values based on a normal distribution centered at val
float normalDistribution(boost::mt19937 rng) 
{
	/*float x1,x2,w=1.0,y1,y2;
	while(w>=1.0)
	{
		x1=(2.0*randDouble()-1.0);
		x2=(2.0*randDouble()-1.0);
		w=x1*x1+x2*x2;
	}
	w=sqrt( (-2.0*log(w) ) / w);
	y1=x1*w;
	y2=x2*w;
	if(rand()%2==1)
		return (unsigned short)(y1);
	else
		return (unsigned short)(y2);*/
	//that was really slow

	
	boost::normal_distribution<> nd(0,0.33);
  	boost::variate_generator<boost::mt19937&, 
                           boost::normal_distribution<> > var_nor(rng, nd);
	return var_nor();
}

//generates a random label of length llen
char* randomLabel(unsigned char const llen)
{
	char * label= (char*)malloc(llen*sizeof(char));
	for(unsigned int i=0;i<llen;i++)
	{
		label[i]=rand()%26+65;
	}
	return label;
}
void updateBestCandidates(std::vector< std::pair<double, std::map<unsigned int, unsigned int> > > & candidates, std::pair<double, std::map<unsigned int, unsigned int> > potential, unsigned int maxSize, bool sorted)
{
	bool added = false;
	if(candidates.size()<maxSize || candidates[maxSize-1].first > potential.first)
	{
		candidates.push_back(potential);
		added=true;
	}

	if(added)
	{
		sort(candidates.begin(),candidates.end());
		if(candidates.size()>maxSize)
		{
			candidates.pop_back();
		}
	}
}

//checks if the potential should be added to the candidate list
void updateBestCandidates(std::vector< std::pair<double, unsigned int> > & candidates, std::pair<double, unsigned int> potential, unsigned int maxSize, bool sorted)
{
	bool added = false;
	if(candidates.size()<maxSize || candidates[maxSize-1].first > potential.first)
	{
		candidates.push_back(potential);
		added=true;
	}

	if(added)
	{
		sort(candidates.begin(),candidates.end());
		if(candidates.size()>maxSize)
		{
			candidates.pop_back();
		}
	}
}

bool mapRanks(const std::pair<std::map<unsigned int, unsigned int>, double>& p1,
	const std::pair<std::map<unsigned int, unsigned int>, double>& p2)
{
	return p1.second < p2.second;
}

bool sortTriple(const Triple& l, const Triple& r)
{
	return (l.diff < r.diff);
}

int iterativePartitionSortInt(const MyNode* l, const MyNode* r)
{
	return l->partitionString.compare(r->partitionString);
}
bool iterativePartitionSort(const MyNode* l, const MyNode* r)
{
	return (iterativePartitionSortInt(l,r)<0);
}
bool iterativePartitionNeighborSort(const MyEdge l, const MyEdge r)
{
	//when sorting the neighbors of a node, sort by that node's partition only.
	bool result=l.getNode()->lastPartition == r.getNode()->lastPartition;
	if(result!=true)
	{
		return (l.getNode()->lastPartition < r.getNode()->lastPartition);
	}
	else
	{
		return (l.getEdgeLabel() < r.getEdgeLabel());
	}
}
/*
int compareIPartitionIndexValue(const std::pair<iPartitionEntryType,unsigned int>& x,const std::pair<iPartitionEntryType,unsigned int>& y)
{
	int returnVal=0;
	if(x.first.direction!=y.first.direction)
	{
		if(x.first.direction>y.first.direction){return -1;}
		else {return 1;}
	}
	returnVal=x.first.label.compare(y.first.label);
	if(returnVal!=0)
	{
		return returnVal;
	}
	if(x.second!=y.second)
	{
		return x.second<y.second;
	}
	if(x.second==0)//both x.count and y.count are 0, they are both dummy
	{
		return 0;
	}
	//fprintf(stderr,"x= %d %d\n",x.nodePtr,y.nodePtr);
	if(x.first.nodePtr->getId()!=y.first.nodePtr->getId())
	{
		if(x.first.nodePtr->getId()>y.first.nodePtr->getId()){return -1;}
		else{return 1;}
	}
	return 0;
}
bool compareIPartitionIndexValueSort(const iPartitionEntryType& x,const iPartitionEntryType& y)
{
	return compareIPartitionIndexValue(std::pair<iPartitionEntryType,unsigned int>(x,1),std::pair<iPartitionEntryType,unsigned int>(y,1))>0;
}

bool indexCompare(const MyNode* l, const MyNode* r)
{
	return l->compareIndex(r);
}
*/
bool typeCompare(const MyNode* l,const MyNode* r)
{
	return l->getType()<r->getType();
}

bool lexFirstIdegree(const MyNode* l, const MyNode* r)
{
	if(typeCompare(l,r)){return true;}
	if(lexFirstIdegree2(&(l->currentDeg),&(r->currentDeg))){return true;}
	return false;
}
bool lexFirstIdegree2(const std::map<edgeDescriptor,float>* l, const std::map<edgeDescriptor,float>* r)
{
	std::map<edgeDescriptor,float>::const_iterator itr=l->begin();
	std::map<edgeDescriptor,float>::const_iterator jtr=r->begin();
	for(;itr!=l->end()&&jtr!=r->end();)
	{
		if(itr->first < jtr->first)
		{return true;}
		if(jtr->first < itr->first)
		{return false;}
		if(itr->second < jtr->second)
		{return true;
		}
		if(itr->second > jtr->second)
		{return false;
		}
		itr++;
		jtr++;
	}
	if(jtr==r->end())
	{
		return false;
	}
	return true;
}


bool mxSort(const std::pair<unsigned int, unsigned int >& l, const std::pair<unsigned int, unsigned int>& r)
{
	if( l.first < r.first)
	{return true;}
	if(l.first > r.first)
	{return false;}
	if(l.second < r.second)
	{return true;}
	return false;
}

bool idSort(const MyNode* l, const MyNode* r)
{
	return l->getID()<r->getID();
}


