#ifndef UTILITIES_H
#define UTILITIES_H

#include <vector>
#include <string>

#include "graph.h"
#include <boost/random.hpp>
#include <boost/random/normal_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>
class timing
{
	public:
	timing(){};
	void start(std::string s)
	{
		startTime[s]=time(NULL);
		//fprintf(stderr,"start %s\n",s.c_str());
	};
	void stop(std::string s)
	{
		if(times.find(s)==times.end()){times[s]=0;}
		times[s]+=(time(NULL)-startTime[s]);
		//fprintf(stderr,"stop %s\n",s.c_str());
	};
	void print(FILE* F){for(std::map<std::string,double>::iterator itr=times.begin();itr!=times.end();itr++)
	{
		fprintf(F,"%s\t->\t%.1f\n",itr->first.c_str(),itr->second);
	}
	};
	private:
	std::map<std::string,double> startTime;
	std::map<std::string,double> times;
};

struct arguments
{
	timing T;
	char searchMethod;		
	char similarityMethod;
	int i_degree;
	unsigned int closeness;
	int successMethod;
	char distMethod;//'1','2','s'
	char successMeth;//new//'a'nchor or 'o'verlap
	unsigned int maxMaps;
	float overlapPercent;
	char generateMethod;	
		//options: 
		//'o' - old method (numnodes, distortion), 
		//'s' - static probability expansion (numnodes), 
		//'e' - variable probaility, exact expansion (numnodes)
		//'p' - prandom walk
	char partitionMethod2;
	unsigned int fragSize, minFragSize;
	unsigned int nTests;
	double distortion;
	double wI[3];
	double wO[3];
	char partitionMethod;
	//'o' = out
	//'i' = in
	//'s' = out/in
	//'b' = outonly/bidirectional/inonly
	std::string fragFile;
	long int anchor;
	std::string outputPrefix;
	bool useEdgeLabels;
	int seedNum;
	std::string graphFile;
	FILE* mXoutput;
	FILE* mXoutput2;
	//std::map<std::string,File*> files;
};

struct Triple
{
	unsigned int id1;
	unsigned int id2;
	float diff;
};

std::string truncateString(std::string a);
char* stringToCString(std::string a);

void printDegree(const MyNode *n, FILE * f);

std::string my_itoa(int value,int base=62);

void printTrace(FILE* f, const char *fmt, ...);
float metricDist(float a, float b, char Metric,struct arguments args);

//returns true if r has non-zeros in the spots (and possible others) as l
//r covers l i this is true.
bool covers(const std::vector<float>& l, const std::vector<float>& r,char Metric,struct arguments args);
//Does r dominate l?
//r must have >= neighbors than l in all types.
bool dominates(const std::vector<float>&l, const std::vector<float>&r,char Metric,struct arguments args);
float dist(const std::vector<float>& pm1, const std::vector<float>& pm2,char Metric,struct arguments args);
float dist(const iDegreeList* pm1, const iDegreeList* pm2,char Metric,struct arguments args);
float weightDist(const MyNode *l, const MyNode* r, unsigned int level,char Metric,struct arguments args);
float dist(const MyNode* l, const MyNode* r, unsigned int level,char Metric,struct arguments args);
float coverDist(const std::vector<float>& pm1, const std::vector<float>& pm2,char Metric,struct arguments args);
float coverDist(const MyNode* l, const MyNode * r, unsigned int level,char Metric,struct arguments args);
float coverDist(const iDegreeList* pm1, const iDegreeList* pm2, char Metric,struct arguments args);
float nonZeroDist(const std::vector<float>& pm1, const std::vector<float>& pm2,char Metric,struct arguments args);
float nonZeroDist(const MyNode* l, const MyNode * r, unsigned int level,char Metric,struct arguments args);
float nonZeroDist(const iDegreeList* pm1, const iDegreeList* pm2, char Metric,struct arguments args);
float dominatingDist(const std::vector<float>&pm1, const std::vector<float>& pm2,struct arguments args);
float dominatingDist(const iDegreeList* pm1, const iDegreeList* pm2, char Metric,struct arguments args);
float dominatingDist(const MyNode* l, const MyNode* r, unsigned int level,char Metric,struct arguments args);
float dominatingInDist(const MyNode* l, const MyNode* r, unsigned int level,char Metric,struct arguments args);

float similarity(const MyNode* l, const MyNode * r, char method, unsigned int i_degree,char Metric,struct arguments args);

//r is confused with l if r is at most max different than l.
bool differsByAtMost(const std::vector<float>&l, const std::vector<float>&r, float max,struct arguments args);
//Used for sorting two nodes by their 0 deg and 1deg.
int compare(const MyNode*  l, const MyNode * r, unsigned int i =3);
// compares the two type vectors
//	returns: 
//			-1 if l < r
//			 0 if l == r
//			 1 if l > r
int compare(const std::vector<float>& l, const std::vector<float>& r);
int compare(const iDegreeList* l, const iDegreeList* r);

//sort lexiographically
bool lexFirst(const MyNode* l, const MyNode* r);
bool sortTriple(const Triple& l, const Triple& r);
int iterativePartitionSortInt(const MyNode* l, const MyNode* r);
bool iterativePartitionSort(const MyNode* l, const MyNode* r);
bool iterativePartitionNeighborSort(const MyEdge l, const MyEdge r);
//generate a random float between 0 and 1
float randDouble();
//generate a value on the normal distribution
float normalDistribution(boost::mt19937 rng);
//generates a random label of length llen
char* randomLabel(unsigned char const llen);	

void updateBestCandidates(std::vector< std::pair<double, unsigned int> > & candidates, std::pair<double, unsigned int> potential, unsigned int maxSize, bool sorted=true);
void updateBestCandidates(std::vector< std::pair<double, std::map<unsigned int, unsigned int> > > & candidates, std::pair<double, std::map<unsigned int, unsigned int> > potential, unsigned int maxSize, bool sorted=true);


bool mapRanks(const std::pair<std::map<unsigned int, unsigned int>, double>& p1,
	const std::pair<std::map<unsigned int, unsigned int>, double>& p2);
/*
int compareIPartitionIndexValue(const std::pair<iPartitionEntryType,unsigned int>& x,const std::pair<iPartitionEntryType,unsigned int>& y);
bool compareIPartitionIndexValueSort(const iPartitionEntryType& x,const iPartitionEntryType& y);
bool indexCompare(const MyNode* l, const MyNode* r);
*/
bool typeCompare(const MyNode* l,const MyNode* r);
bool lexFirstIdegree(const MyNode* l, const MyNode* r);
bool lexFirstIdegree2(const std::map<edgeDescriptor,float>* l, const std::map<edgeDescriptor,float>* r);

bool mxSort(const std::pair<unsigned int, unsigned int>& l, const std::pair<unsigned int, unsigned int>& r);

bool idSort(const MyNode* l, const MyNode* r);

#endif

