/*	Written May 6, 2011 
*	The purpose of this file is to time the various testing criteria components
*/

#include <cstdlib>
#include <cstring>
#include <iomanip>
#include <iostream>
#include <map>
#include <vector>
#include <time.h>
#include <string>
#include <fstream>	

//#include <windows.h>
//#define sleep(n) Sleep(1000 * n)

#include "graph.h"
#include "utilities.h"


#define CONSTANT_SEED
#define BACKTRACK

//#define TRACE


/*** Global Variables***/
unsigned int seed;
std::string lastGraphFile;
arguments args; //set in getArguments

#ifdef CONSTANT_SEED
std::vector<unsigned int> indicesToGenFrag;
#endif

float overlap(0);
unsigned int successfulOverlaps(0);

/*** Function Prototypes ***/
bool getArguments(std::vector<std::string> argP, MyGraph *&G);


//fill in the vector with potential candidates to anchor the center to
void generateCandidates(MyGraph * G, MyGraph * fragment, unsigned int center, std::vector<std::pair<double,std::map<unsigned int,unsigned int> > >& candidateIDs, unsigned int nBest);

void generateFrag(MyGraph& fragment, MyGraph* database,const MyNode* centerNode);

unsigned int generateFragCenterSingle(MyGraph* database)
{
	unsigned int edgeCount = 0;
	unsigned int centerIndex=0;
	do
	{
		centerIndex = rand() % database->getNumNodes();
		edgeCount = (database->getNodeByIndex(centerIndex))->getNumOutNeighbors(); 
	}while(edgeCount <= FEW_EDGES);
	return centerIndex;
}
//unsigned int seed;
void generateFragCenters(MyGraph* database)
{
	indicesToGenFrag.resize(args.nTests, 0);
	unsigned int num_nodes = database->getNumNodes();
	std::map<unsigned int, unsigned int> neighborhood;
	for(unsigned int i = 0; i < args.nTests; ++i)
	{
		indicesToGenFrag[i] = generateFragCenterSingle(database);
	}
}
/*** MAIN ***/
int main(int argc, char ** argv)
{
	time_t start = time(NULL);
	//unsigned int seed = (unsigned int)time(NULL);
	seed = /*1311746957;//*/2357;
	MyGraph * database;
	srand(seed);
	bool testing=true;
	args.T.start("reading args");
	//read in any changes to default arguments
	fprintf(stderr,"accepting arguments\n");
	std::vector<std::string> argP;
	for(unsigned int i=0;i<(unsigned)argc;i++)
	{
		argP.push_back(argv[i]);
	}
	bool success = getArguments(argP, database);
	std::cerr<<"done testing loading database\n";
	if(!success)
	{
		std::cout << "\nincorrect args, see documentation\n";
		return 1;	//exit failure
	}
	std::string timingFileName=args.outputPrefix;
	timingFileName+=".time";
	FILE* timingOutput=fopen(timingFileName.c_str(),"w");
	std::string mxFileName=args.outputPrefix;
	mxFileName+=".MX";
	args.mXoutput=fopen(mxFileName.c_str(),"w");
	std::string mx2FileName=args.outputPrefix;
	mx2FileName+=".MX2";
	args.mXoutput2=fopen(mx2FileName.c_str(),"w");
	positioningArgument p;
	p.method = args.i_degree;//default args.i_degree, assuming no index file is provided
	args.T.stop("reading args");
	args.T.print(stderr);
	
	
		std::cerr<<"*Database Loaded*\n";
		generateFragCenters(database);
		std::cerr<<"Generated Fragment Centers\n";
		
		for(unsigned int i=0;i<args.nTests;i++)
		{
			fprintf(stderr,"\nTest %d\n",i+1);
			MyGraph fragment(database->isDirected());//create frag
			fprintf(stderr,"Gen Fragment\n");
			args.T.start("gen fragment");
			generateFrag(fragment, database,database->getNodeByIndex(indicesToGenFrag[i]));
			//generate Fragment
			if(fragment.getNumNodes()<args.minFragSize)
			{
				fprintf(stderr,"--Fragment too small (%d<%d), retry\n",fragment.getNumNodes(),args.minFragSize);
				indicesToGenFrag[i]=generateFragCenterSingle(database);
				i--;
				continue;
			}
			args.T.stop("gen fragment");
			
			fprintf(stderr,"Number of nodes %d edges %lu\n",fragment.getNumNodes(),fragment.getNumEdges());
			//print fragment info to cerr
			std::cerr << "\nExperiment " << i+1 << " of " << args.nTests <<":" << std::flush;
			fprintf(stderr, "\nfragment originator in database: \n");
			const MyNode* tempNode=database->getNodeByIndex(indicesToGenFrag[i]);
			const MyNode* centerNode=fragment.getNodeById(tempNode->getID());
			printDegree(centerNode, stderr);
			
			unsigned temp=lastGraphFile.find_last_of("\\/");
			
			std::string tempStr=lastGraphFile.substr(temp+1);
			std::string fragFile="./fragments/"+tempStr+"fragment"+my_itoa(i,10)+".grf";
			fragment.writePlainGraph(fragFile.c_str());
		}
		args.T.print(stderr);
		args.T.print(timingOutput);
		//realTest('o', database, args.nTests);
		fclose(timingOutput);
		fclose(args.mXoutput);
	return 0;
}


/*** Function Definitions ***/
/*							 
std::cout << "\nusage: \"evaluate <graph spec> <frag gen spec> <search spec> <similarity spec> #samples\"";
std::cout << "\n\t<Graph spec> :\t1) fileName {1|2|3|<locality.ppg}";
std::cout << "\n\t              \t2) numNodes, numTypes, edgesPerNode";
std::cout << "\n\t<frag gen spec>:\t1) 'o' numNodes distortion";
std::cout << "\n\t                \t2) 's'|'e'|'n' numNodes";
std::cout << "\n\t<search spec>:\t1) 's' single center";
std::cout << "\n\t              \t2) 'p' adjacent pair";
std::cout << "\n\t              \t2) 'm' multiple searches based on an adjacent pair";
std::cout << "\n\t              \t2) 't' tree: central and two adjacent nodes";
std::cout << "\n\t              \t2) 'c' chain: 3 nodes ";
std::cout << "\n\t              \t2) 'q' quadruple: central and three adjacent nodes";
std::cout << "\n\t<similarity spec>:\t1) 'd' ";
std::cout << "\n\t                 :\t2) 'r' \n\n";
*/
bool getFragFile(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.fragFile = argv[nextArg];
	++nextArg;
	return true;
}
bool getFragSize(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.fragSize = atoi(argv[nextArg].c_str());
	++nextArg;
	args.minFragSize = atoi(argv[nextArg].c_str());
	++nextArg;
	return true;
}
bool getGenerateMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.generateMethod = argv[nextArg][0];
	++nextArg;
	switch(args.generateMethod){
		case 'o':
		case 'c':
		case 's':
		case 'n':
		case 'p':
		case 'e':
		case 'k':
			break;
		case 'f':
			if(args.generateMethod=='f')
			{
				if(!getFragFile(argv,nextArg))
					{return false;}
			}
			return true;
			break;
		default:
			std::cout << "\nError: Invalid fragment generation method (" << args.generateMethod << ")";
			return false;
	}
	if(!getFragSize(argv,nextArg))
		{return false;}
	return true;
}

bool getDistortion(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.distortion = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	if(args.distortion < 0 || args.distortion >= .1)
	{
		std::cout << "\nError: Invalid distortion (" << args.distortion << " not in (0.0, 0.1)";
		return false;
	}
	return true;
}
bool getSearchMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.searchMethod = argv[nextArg][0];
	++nextArg;
	switch(args.searchMethod)
	{
		case 's':
		case 'p':
		case 'm':
		case 't':
		case 'c':
		case 'I':
			break;//these are acceptable.
		default:
			if(!isdigit(args.searchMethod) )
			{
				std::cout << "\nError: Invalid search method (\'" << args.searchMethod << "\')";
				return false;
			}
			break;
	}
	return true;
}
bool getSimilarityMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.similarityMethod =argv[nextArg][0];
	++nextArg;
	switch(args.similarityMethod)
	{
		case 'd':
		case 'r':
		case 'f':
		case 'F':
		case 'e':
		case 'E':
			break;//these are acceptable.
		default:
			std::cout << "\nError: Invalid similarity method (" << args.similarityMethod << ")";
			return false;
	}
	return true;
}
bool getNTests(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.nTests = atoi(argv[nextArg].c_str());
	++nextArg;
	if(args.nTests <= 0)
	{
		std::cout << "\nError: Invalid number of tests(" << args.nTests << ")";
		return false;
	}
	return true;
}

bool getAnchorID(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.anchor = atoi(argv[nextArg].c_str());
	++nextArg;
	if(args.anchor <= 0)
	{
		std::cout << "\nError: Invalid anchor(" << args.anchor << ")";
		return false;
	}
	return true;
}

//gets the method to use for measuring distance between i-degrees
bool getDistMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.distMethod =argv[nextArg][0];
	++nextArg;
	switch(args.distMethod)
	{
		case '1'://x^1
		case '2'://x^2
		case 'S'://x^.5
			break;//these are acceptable.
		default:
			std::cout << "\nError: Invalid distMethod (" << args.distMethod << ")";
			return false;
	}
	return true;
}
bool getIDegree(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.i_degree =argv[nextArg][0]-'0';
	++nextArg;
	switch(args.i_degree)
	{
		case 0:
		case 1:
		case 2:
		case 3:
			break;//these are acceptable.
		default:
			std::cout << "\nError: Invalid iDegree(" << args.i_degree << ")";
			return false;
	}
	return true;
}
bool getWeights(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.wI[0] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wI[1] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wI[2] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wO[0] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wO[1] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wO[2] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	return true;
}
bool getArguments(std::vector<std::string> argv, MyGraph *&G)
{
	//set default arguments
	args.outputPrefix="defaultOutput";
	args.useEdgeLabels=false;
	args.searchMethod='9';		
	args.similarityMethod='F';//for use in i-degree algorithm//F=='bipartite matching, in/out edges
	args.i_degree=3;
	args.closeness=0;//not currently internal use
	args.successMethod=0;//currently internal use
	args.distMethod='1';
	args.generateMethod='p';//prandom walk to generate fragments
	args.partitionMethod='o';//'o','i','s','b'//which edges to use
	args.partitionMethod2='i';//describes the algorithm to index with
	args.fragSize=10;//size of the fragment
	args.nTests=1;//number of fragments to test
	args.distortion=0.0;//distortion is used in fragment generation, for some algorithms
	args.wI[0]=args.wI[1]=args.wI[2]=1.0;//default 1//in weights
	args.wO[0]=args.wO[1]=args.wO[2]=1.0;//default 1//out weights
	args.fragFile="";
	args.anchor=0;
	args.minFragSize=1;
	args.seedNum=2357;
	//take in arguments now
	unsigned int nextArg=1;
	positioningArgument p;
	p.method = args.i_degree;
	std::string graphFile=lastGraphFile;
	bool import = false; 
	unsigned int numNode=0, numTypes=0, edgesPerNode=0;
	while(nextArg<argv.size())
	{
		//std::cout<<argv[nextArg]<<std::endl;
		if(argv[nextArg]=="-edgeLabels")
		{++nextArg;
			args.useEdgeLabels=atoi(argv[nextArg].c_str())>0;
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-seed")
		{++nextArg;
			args.seedNum = atoi(argv[nextArg].c_str());
			srand(args.seedNum);
			seed=args.seedNum;
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-f")
		{++nextArg;
			import = true;
			p.method = atoi(argv[nextArg].c_str());
			args.i_degree=atoi(argv[nextArg].c_str());
			p.file = argv[nextArg];
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-g")
		{nextArg++;
			graphFile=argv[nextArg];
			nextArg++;
			continue;
		}
		if(argv[nextArg]=="-randGraph")
		{++nextArg;
			numNode = atoi(argv[nextArg].c_str());
			++nextArg;
			if(numNode == 0 || numNode >= UINT_MAX)
			{
				std::cout << "\nError: invalid size of random graph (" << numNode << ") ";
				return false;
			}
			numTypes = atoi(argv[nextArg].c_str());
			++nextArg;
			if(numTypes == 0 || numTypes >= UINT_MAX)
			{
				std::cout << "\nError: invalid type count of random graph (" << numTypes << ") ";
				return false;
			}
			edgesPerNode = atoi(argv[nextArg].c_str());
			++nextArg;
			if(edgesPerNode == 0 || edgesPerNode >= UINT_MAX)
			{
				std::cout << "\nError: invalid edge count of random graph (" << edgesPerNode << ") ";
				return false;
			}
			G = new MyGraph(true);
			fprintf(stderr,"gen-ing graph\n");
			G->generateGraph(numNode, numTypes, edgesPerNode, args,true);
			continue;
		}
		if(argv[nextArg]=="-gen")
		{++nextArg;
			if(!getGenerateMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-search")
		{++nextArg;
			if(!getSearchMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-sim")
		{++nextArg;
			if(!getSimilarityMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-n")
		{++nextArg;
			if(!getNTests(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-dist")
		{++nextArg;
			if(!getDistMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-distort")
		{++nextArg;
			if(!getDistortion(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-anchor")
		{++nextArg;
			if(!getAnchorID(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-i")
		{++nextArg;
			if(!getIDegree(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-w")
		{++nextArg;
			if(!getWeights(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-partMethod")
		{++nextArg;
			args.partitionMethod=argv[nextArg][0];
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-success")
		{++nextArg;
			args.successMeth=argv[nextArg][0];
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-overlapPercent")
		{++nextArg;
			args.overlapPercent=strtod(argv[nextArg].c_str(), NULL);
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-maxMaps")
		{++nextArg;
			args.maxMaps=std::atoi(argv[nextArg].c_str());
			++nextArg;
			continue;
		}
		fprintf(stderr,"unknown arg %s\n",argv[nextArg].c_str());
		exit(0);
	}
	fprintf(stderr,"graphfile = %s\n",graphFile.c_str());
	if(graphFile.compare(lastGraphFile)!=0)
	{
		lastGraphFile=graphFile;
		if(import)
		{
			bool plain=true;
			G = new MyGraph(graphFile, true, p, true, args.partitionMethod2, true, args);
		}
		args.graphFile=graphFile;
	}
	if(G->getNumNodes() <= args.fragSize || (args.fragSize == 0 && args.fragFile.size()==0))
	{
		std::cout << "\nError: Invalid fragment size (" << args.fragSize << ") not in range (0, " << G->getNumNodes() << ")";
		return false;
	}
	fprintf(stderr,"done with args\n");
	args.outputPrefix="./output/";
	for(unsigned int i=1;i<argv.size();i++)
	{
		if(argv[i-1].compare("-g")==0)
		{
			args.outputPrefix+=argv[i].substr(13,argv[i].size()-13)+"_";
		}
		else
		{
			args.outputPrefix+=argv[i]+"_";
		}
	}
	fprintf(stderr,"args.outputPrefix= %s\n",args.outputPrefix.c_str());
	return true;
}


//fill in the vector with potential candidates to anchor the center to

void generateFrag(MyGraph& fragment, MyGraph* database,const MyNode* centerNode)
{
#ifdef TRACE
		switch(generateMethod)
		{
			case 'o':
				std::cerr << "\nGenerating fragment using old method: size=" <<fragSize << ", distortion=" << distortion;
				break;
			case 's':
				std::cerr << "\nGenerating fragment static probability: size =" <<fragSize;
				break;
			case 'e':
				std::cerr << "\nGenerating fragment changing probability: size =" <<fragSize;
				break;
			case 'n':
				std::cerr << "\nGenerating fragment random 1N&2N: size =" <<fragSize;
				break;
			case 'k':
				std::cerr << "\nGenerating fragment k random walk: size =" <<fragSize;
				break;
			case 'p':
				std::cerr << "\nGenerating fragment p random walk: size =" <<fragSize;
				break;
				
		}
#endif

		switch(args.generateMethod)
		{
#ifdef CONSTANT_SEED
			case 'c'://complete
				fragment = database->getSubgraphToDegreeById(centerNode->getID(),2,false,args,false);
				break;
			case 'p':
				fragment.generateFragment_pRandomWalk(database,centerNode,args.fragSize,0.9, 0.7,args);
				break;
#else
			case 'c'://complete
				do
				{
					centerIndex = rand() % num_nodes;
					edgeCount = (database->getNodeByIndex(centerIndex))->outgoing_edges.size(); 
				}while(edgeCount <= FEW_EDGES || edgeCount >= fragSize);
				fragment = database->getSubgraphToDegreeById(database->getNodeById(centerIndex)->getID(), 2, false, false);
				break;
			case 'o':
				fragment = database->generateFragment(fragSize, args.distortion);
				break;
			case 's':
				fragment = database->generateFragment_staticprob(fragSize);
				break;
			case 'e':
				fragment = database->generateFragment_exactsize(fragSize);
				break;
			case 'n':
				fragment = database->generateFragment_rand1N(fragSize);
				break;
			case 'k':
				fragment = database->generateFragment_kRandomWalk(fragSize,2);
				break;
			case 'p':
				fragment = database->generateFragment_pRandomWalk(fragSize,0.9, 0.7);
				break;
#endif
		default:
			//THIS SHOULD NEVER BE REACHED
			std::cerr<< "\nError Invalid generation method (" << args.generateMethod << "),\nExiting program\n\n";
			exit(1);//exit failure  
		}
}
