/*	Written May 6, 2011 
*	The purpose of this file is to time the various testing criteria components
*/

#include <cstdlib>
#include <cstring>
#include <iomanip>
#include <iostream>
#include <map>
#include <vector>
#include <time.h>
#include <string>
#include <fstream>	

//#include <windows.h>
//#define sleep(n) Sleep(1000 * n)

#include "graph.h"
#include "utilities.h"

#define BACKTRACK

/*** Global Variables***/
unsigned int seed;
std::string lastGraphFile;
arguments args; //set in getArguments

std::vector<unsigned int> indicesToGenFrag;

float overlap(0);
unsigned int successfulOverlaps(0);

/*** Function Prototypes ***/
bool getArguments(std::vector<std::string> argP, MyGraph *&G);

bool idegreeCalc(MyGraph* database,std::string outputName,MyGraph* typeDatabase)
{
	std::string idegreeFileName=outputName;
	idegreeFileName+=".idegree";
	FILE* idegreeOutput=fopen(idegreeFileName.c_str(),"w");
	
	
	//calculate indexvalues and print to a file
	//i==0
	args.T.start("idegree-Calc");
	fprintf(stderr,"i=%d\n",0);
	unsigned int count=0;
	std::map<unsigned int, std::vector< std::map<unsigned int, unsigned int> > > idegrees;
	std::map<unsigned int, std::vector< std::map<unsigned int, unsigned int> > > idegreesIn;
	for(mapIDNodes::iterator itr=database->nodes.begin();itr!=database->nodes.end();itr++)
	{
		std::map<unsigned int, unsigned int> selfType;
		selfType[itr->second.getType()]=1;
		idegrees[itr->first].push_back(selfType);
		idegreesIn[itr->first].push_back(selfType);
	}
	//i>0
	for(unsigned int i=1;i<=args.i_degree;i++)
	{
		for(mapIDNodes::iterator itr=database->nodes.begin();itr!=database->nodes.end();itr++)
		{
			std::map<unsigned int, unsigned int> myidegree;
			idegrees[itr->first].push_back(myidegree);
			const edgelist* myOutEdges=itr->second.getOutEdgesConst();
			for(edgelist::const_iterator edgeItr=myOutEdges->begin();
				edgeItr!=myOutEdges->end();edgeItr++)
			{
				std::map<unsigned int, unsigned int>::iterator idegPtrEnd=idegrees[edgeItr->second.getNode()->getID()][i-1].end();
				for(std::map<unsigned int, unsigned int>::iterator idegPtr=idegrees[edgeItr->second.getNode()->getID()][i-1].begin();idegPtr!=idegPtrEnd;idegPtr++)
				{
					std::map<unsigned int, unsigned int>::iterator typePtr=idegrees[itr->first][i].find(idegPtr->first);
					if(typePtr==idegrees[itr->first][i].end())
					{
						idegrees[itr->first][i][idegPtr->first]=idegPtr->second;
					}
					else
					{
						typePtr->second+=idegPtr->second;
					}
				}
			}
		}
	}
	for(unsigned int i=1;i<=args.i_degree;i++)
	{
		for(mapIDNodes::iterator itr=database->nodes.begin();itr!=database->nodes.end();itr++)
		{
			std::map<unsigned int, unsigned int> myidegree;
			idegreesIn[itr->first].push_back(myidegree);
			const edgelist* myInEdges=itr->second.getInEdgesConst();
			for(edgelist::const_iterator edgeItr=myInEdges->begin();
				edgeItr!=myInEdges->end();edgeItr++)
			{
				std::map<unsigned int, unsigned int>::iterator idegPtrEnd=idegreesIn[edgeItr->second.getNode()->getID()][i-1].end();
				for(std::map<unsigned int, unsigned int>::iterator idegPtr=idegreesIn[edgeItr->second.getNode()->getID()][i-1].begin();idegPtr!=idegPtrEnd;idegPtr++)
				{
					std::map<unsigned int, unsigned int>::iterator typePtr=idegreesIn[itr->first][i].find(idegPtr->first);
					if(typePtr==idegreesIn[itr->first][i].end())
					{
						idegreesIn[itr->first][i][idegPtr->first]=idegPtr->second;
					}
					else
					{
						typePtr->second+=idegPtr->second;
					}
				}
			}
		}
	}
	args.T.stop("idegree-Calc");
	std::map<unsigned int, bool> myTypes;
	for(mapIDNodes::iterator itr=typeDatabase->nodes.begin();itr!=typeDatabase->nodes.end();itr++)
	{
		myTypes[itr->second.getType()]=true;
	}
	for(mapIDNodes::iterator itr=database->nodes.begin();itr!=database->nodes.end();itr++)
	{
		fprintf(idegreeOutput,"%d ",itr->first);
		for(unsigned int i=0;i<=args.i_degree;i++)
		{
			std::map<unsigned int, unsigned int>::iterator idegPtrEnd=idegrees[itr->first][i].end();
			for(std::map<unsigned int, bool>::iterator typeItr=myTypes.begin();typeItr!=myTypes.end();typeItr++)
			{
				unsigned int v=0;
				std::map<unsigned int,unsigned int>::iterator idegPtr=idegrees[itr->first][i].find((unsigned int) typeItr->first);
				if(idegPtr!=idegPtrEnd)
				{
					v=idegPtr->second;
				}
				fprintf(idegreeOutput,"%d ",v);
			}
		}
		for(unsigned int i=1;i<=args.i_degree;i++)
		{
			std::map<unsigned int, unsigned int>::iterator idegPtrEnd=idegreesIn[itr->first][i].end();

			for(std::map<unsigned int, bool>::iterator typeItr=myTypes.begin();typeItr!=myTypes.end();typeItr++)
			{
				unsigned int v=0;
				std::map<unsigned int,unsigned int>::iterator idegPtr=idegreesIn[itr->first][i].find((unsigned int) typeItr->first);
				if(idegPtr!=idegPtrEnd)
				{
					v=idegPtr->second;
				}
				fprintf(idegreeOutput,"%d ",v);
			}
		}
		fprintf(idegreeOutput,"\n");
	}
	fclose(idegreeOutput);
}

bool idegreeCalcSparse(MyGraph* database,std::string outputName)
{
	std::string idegreeFileName=outputName;
	idegreeFileName+=".idegree";
	FILE* idegreeOutput=fopen(idegreeFileName.c_str(),"w");
	
	
	//calculate indexvalues and print to a file
	//i==0
	args.T.start("idegree-Calc");
	fprintf(stderr,"i=%d\n",0);
	unsigned int count=0;
	std::map<unsigned int, std::vector< std::map<unsigned int, unsigned int> > > idegrees;
	std::map<unsigned int, std::vector< std::map<unsigned int, unsigned int> > > idegreesIn;
	for(mapIDNodes::iterator itr=database->nodes.begin();itr!=database->nodes.end();itr++)
	{
		std::map<unsigned int, unsigned int> selfType;
		selfType[itr->second.getType()]=1;
		idegrees[itr->first].push_back(selfType);
		idegreesIn[itr->first].push_back(selfType);
	}
	//i>0
	for(unsigned int i=1;i<=args.i_degree;i++)
	{
		for(mapIDNodes::iterator itr=database->nodes.begin();itr!=database->nodes.end();itr++)
		{
			std::map<unsigned int, unsigned int> myidegree;
			idegrees[itr->first].push_back(myidegree);
			const edgelist* myOutEdges=itr->second.getOutEdgesConst();
			for(edgelist::const_iterator edgeItr=myOutEdges->begin();
				edgeItr!=myOutEdges->end();edgeItr++)
			{
				std::map<unsigned int, unsigned int>::iterator idegPtrEnd=idegrees[edgeItr->second.getNode()->getID()][i-1].end();
				for(std::map<unsigned int, unsigned int>::iterator idegPtr=idegrees[edgeItr->second.getNode()->getID()][i-1].begin();idegPtr!=idegPtrEnd;idegPtr++)
				{
					std::map<unsigned int, unsigned int>::iterator typePtr=idegrees[itr->first][i].find(idegPtr->first);
					if(typePtr==idegrees[itr->first][i].end())
					{
						idegrees[itr->first][i][idegPtr->first]=idegPtr->second;
					}
					else
					{
						typePtr->second+=idegPtr->second;
					}
				}
			}
		}
	}
	for(unsigned int i=1;i<=args.i_degree;i++)
	{
		for(mapIDNodes::iterator itr=database->nodes.begin();itr!=database->nodes.end();itr++)
		{
			std::map<unsigned int, unsigned int> myidegree;
			idegreesIn[itr->first].push_back(myidegree);
			const edgelist* myInEdges=itr->second.getInEdgesConst();
			for(edgelist::const_iterator edgeItr=myInEdges->begin();
				edgeItr!=myInEdges->end();edgeItr++)
			{
				std::map<unsigned int, unsigned int>::iterator idegPtrEnd=idegreesIn[edgeItr->second.getNode()->getID()][i-1].end();
				for(std::map<unsigned int, unsigned int>::iterator idegPtr=idegreesIn[edgeItr->second.getNode()->getID()][i-1].begin();idegPtr!=idegPtrEnd;idegPtr++)
				{
					std::map<unsigned int, unsigned int>::iterator typePtr=idegreesIn[itr->first][i].find(idegPtr->first);
					if(typePtr==idegreesIn[itr->first][i].end())
					{
						idegreesIn[itr->first][i][idegPtr->first]=idegPtr->second;
					}
					else
					{
						typePtr->second+=idegPtr->second;
					}
				}
			}
		}
	}
	args.T.stop("idegree-Calc");
	for(mapIDNodes::iterator itr=database->nodes.begin();itr!=database->nodes.end();itr++)
	{
		fprintf(idegreeOutput,"%d ",itr->first);
		for(unsigned int i=0;i<=args.i_degree;i++)
		{
			std::map<unsigned int, unsigned int>::iterator idegPtrEnd=idegrees[itr->first][i].end();
			for(std::map<unsigned int, unsigned int>::iterator idegPtr=idegrees[itr->first][i].begin();idegPtr!=idegPtrEnd;idegPtr++)
			{
				if(i==0)
				{
				fprintf(idegreeOutput,"A %d ",idegPtr->first);
				}
				else
				{
				fprintf(idegreeOutput,"O%d%d %d ",i,idegPtr->first,idegPtr->second);
				}
			}
		}
		for(unsigned int i=1;i<=args.i_degree;i++)
		{
			std::map<unsigned int, unsigned int>::iterator idegPtrEnd=idegreesIn[itr->first][i].end();
			for(std::map<unsigned int, unsigned int>::iterator idegPtr=idegreesIn[itr->first][i].begin();idegPtr!=idegPtrEnd;idegPtr++)
			{
				fprintf(idegreeOutput,"I%d%d %d ",i,idegPtr->first,idegPtr->second);
			}
		}
		fprintf(idegreeOutput,"\n");
	}
	fclose(idegreeOutput);
}

/*** MAIN ***/
int main(int argc, char ** argv)
{
	time_t start = time(NULL);
	//unsigned int seed = (unsigned int)time(NULL);
	seed = /*1311746957;//*/2357;
	MyGraph * database;
	srand(seed);
	bool testing=true;
	args.T.start("reading args");
	//read in any changes to default arguments
	fprintf(stderr,"accepting arguments\n");
	std::vector<std::string> argP;
	for(unsigned int i=0;i<(unsigned)argc;i++)
	{
		argP.push_back(argv[i]);
	}
	bool success = getArguments(argP, database);
	std::cerr<<"done testing loading database\n";
	if(!success)
	{
		std::cout << "\nincorrect args, see documentation\n";
		return 1;	//exit failure
	}
	std::string timingFileName=args.outputPrefix;
	timingFileName+=".time";
	FILE* timingOutput=fopen(timingFileName.c_str(),"w");
	std::string mxFileName=args.outputPrefix;
	mxFileName+=".MX";
	args.mXoutput=fopen(mxFileName.c_str(),"w");
	std::string mx2FileName=args.outputPrefix;
	mx2FileName+=".MX2";
	args.mXoutput2=fopen(mx2FileName.c_str(),"w");
	positioningArgument p;
	p.method = args.i_degree;//default args.i_degree, assuming no index file is provided
	args.T.stop("reading args");
	args.T.print(stderr);
	
	
	//idegreeCalc(database,args.outputPrefix,database);
	for(unsigned int i=0;i<args.nTests;i++)
	{
		fprintf(stdout,"%d\n",i);
		unsigned temp=lastGraphFile.find_last_of("\\/");
			
		std::string tempStr=lastGraphFile.substr(temp+1);
		std::string fragPrefix="./fragments/"+tempStr+"fragment"+my_itoa(i,10);
		std::string fragFile=fragPrefix+".grf";
		MyGraph fragment(fragFile,true,p,true,args.partitionMethod,true,args);
		idegreeCalc(&fragment,fragPrefix,database);
	}
	
	args.T.print(stderr);
	args.T.print(timingOutput);
	
	fclose(timingOutput);
	fclose(args.mXoutput);

	return 0;
}


/*** Function Definitions ***/
/*							 
std::cout << "\nusage: \"evaluate <graph spec> <frag gen spec> <search spec> <similarity spec> #samples\"";
std::cout << "\n\t<Graph spec> :\t1) fileName {1|2|3|<locality.ppg}";
std::cout << "\n\t              \t2) numNodes, numTypes, edgesPerNode";
std::cout << "\n\t<frag gen spec>:\t1) 'o' numNodes distortion";
std::cout << "\n\t                \t2) 's'|'e'|'n' numNodes";
std::cout << "\n\t<search spec>:\t1) 's' single center";
std::cout << "\n\t              \t2) 'p' adjacent pair";
std::cout << "\n\t              \t2) 'm' multiple searches based on an adjacent pair";
std::cout << "\n\t              \t2) 't' tree: central and two adjacent nodes";
std::cout << "\n\t              \t2) 'c' chain: 3 nodes ";
std::cout << "\n\t              \t2) 'q' quadruple: central and three adjacent nodes";
std::cout << "\n\t<similarity spec>:\t1) 'd' ";
std::cout << "\n\t                 :\t2) 'r' \n\n";
*/
bool getFragFile(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.fragFile = argv[nextArg];
	++nextArg;
	return true;
}
bool getFragSize(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.fragSize = atoi(argv[nextArg].c_str());
	++nextArg;
	args.minFragSize = atoi(argv[nextArg].c_str());
	++nextArg;
	return true;
}
bool getGenerateMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.generateMethod = argv[nextArg][0];
	++nextArg;
	switch(args.generateMethod){
		case 'o':
		case 'c':
		case 's':
		case 'n':
		case 'p':
		case 'e':
		case 'k':
			break;
		case 'f':
			if(args.generateMethod=='f')
			{
				if(!getFragFile(argv,nextArg))
					{return false;}
			}
			return true;
			break;
		default:
			std::cout << "\nError: Invalid fragment generation method (" << args.generateMethod << ")";
			return false;
	}
	if(!getFragSize(argv,nextArg))
		{return false;}
	return true;
}

bool getDistortion(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.distortion = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	if(args.distortion < 0 || args.distortion >= .1)
	{
		std::cout << "\nError: Invalid distortion (" << args.distortion << " not in (0.0, 0.1)";
		return false;
	}
	return true;
}
bool getSearchMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.searchMethod = argv[nextArg][0];
	++nextArg;
	switch(args.searchMethod)
	{
		case 's':
		case 'p':
		case 'm':
		case 't':
		case 'c':
		case 'I':
			break;//these are acceptable.
		default:
			if(!isdigit(args.searchMethod) )
			{
				std::cout << "\nError: Invalid search method (\'" << args.searchMethod << "\')";
				return false;
			}
			break;
	}
	return true;
}
bool getSimilarityMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.similarityMethod =argv[nextArg][0];
	++nextArg;
	switch(args.similarityMethod)
	{
		case 'd':
		case 'r':
		case 'f':
		case 'F':
		case 'e':
		case 'E':
			break;//these are acceptable.
		default:
			std::cout << "\nError: Invalid similarity method (" << args.similarityMethod << ")";
			return false;
	}
	return true;
}
bool getNTests(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.nTests = atoi(argv[nextArg].c_str());
	++nextArg;
	if(args.nTests <= 0)
	{
		std::cout << "\nError: Invalid number of tests(" << args.nTests << ")";
		return false;
	}
	return true;
}

bool getAnchorID(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.anchor = atoi(argv[nextArg].c_str());
	++nextArg;
	if(args.anchor <= 0)
	{
		std::cout << "\nError: Invalid anchor(" << args.anchor << ")";
		return false;
	}
	return true;
}

//gets the method to use for measuring distance between i-degrees
bool getDistMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.distMethod =argv[nextArg][0];
	++nextArg;
	switch(args.distMethod)
	{
		case '1'://x^1
		case '2'://x^2
		case 'S'://x^.5
			break;//these are acceptable.
		default:
			std::cout << "\nError: Invalid distMethod (" << args.distMethod << ")";
			return false;
	}
	return true;
}
bool getIDegree(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.i_degree =argv[nextArg][0]-'0';
	++nextArg;
	switch(args.i_degree)
	{
		case 0:
		case 1:
		case 2:
		case 3:
			break;//these are acceptable.
		default:
			std::cout << "\nError: Invalid iDegree(" << args.i_degree << ")";
			return false;
	}
	return true;
}
bool getWeights(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.wI[0] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wI[1] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wI[2] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wO[0] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wO[1] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wO[2] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	return true;
}
bool getArguments(std::vector<std::string> argv, MyGraph *&G)
{
	//set default arguments
	args.outputPrefix="defaultOutput";
	args.useEdgeLabels=false;
	args.searchMethod='9';		
	args.similarityMethod='F';//for use in i-degree algorithm//F=='bipartite matching, in/out edges
	args.i_degree=3;
	args.closeness=0;//not currently internal use
	args.successMethod=0;//currently internal use
	args.distMethod='1';
	args.generateMethod='p';//prandom walk to generate fragments
	args.partitionMethod='o';//'o','i','s','b'//which edges to use
	args.partitionMethod2='i';//describes the algorithm to index with
	args.fragSize=10;//size of the fragment
	args.nTests=1;//number of fragments to test
	args.distortion=0.0;//distortion is used in fragment generation, for some algorithms
	args.wI[0]=args.wI[1]=args.wI[2]=1.0;//default 1//in weights
	args.wO[0]=args.wO[1]=args.wO[2]=1.0;//default 1//out weights
	args.fragFile="";
	args.anchor=0;
	args.minFragSize=1;
	args.seedNum=2357;
	//take in arguments now
	unsigned int nextArg=1;
	positioningArgument p;
	p.method = args.i_degree;
	std::string graphFile=lastGraphFile;
	bool import = false; 
	unsigned int numNode=0, numTypes=0, edgesPerNode=0;
	while(nextArg<argv.size())
	{
		//std::cout<<argv[nextArg]<<std::endl;
		if(argv[nextArg]=="-edgeLabels")
		{++nextArg;
			args.useEdgeLabels=atoi(argv[nextArg].c_str())>0;
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-seed")
		{++nextArg;
			args.seedNum = atoi(argv[nextArg].c_str());
			srand(args.seedNum);
			seed=args.seedNum;
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-f")
		{++nextArg;
			import = true;
			p.method = atoi(argv[nextArg].c_str());
			args.i_degree=atoi(argv[nextArg].c_str());
			p.file = argv[nextArg];
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-g")
		{nextArg++;
			graphFile=argv[nextArg];
			nextArg++;
			continue;
		}
		if(argv[nextArg]=="-randGraph")
		{++nextArg;
			numNode = atoi(argv[nextArg].c_str());
			++nextArg;
			if(numNode == 0 || numNode >= UINT_MAX)
			{
				std::cout << "\nError: invalid size of random graph (" << numNode << ") ";
				return false;
			}
			numTypes = atoi(argv[nextArg].c_str());
			++nextArg;
			if(numTypes == 0 || numTypes >= UINT_MAX)
			{
				std::cout << "\nError: invalid type count of random graph (" << numTypes << ") ";
				return false;
			}
			edgesPerNode = atoi(argv[nextArg].c_str());
			++nextArg;
			if(edgesPerNode == 0 || edgesPerNode >= UINT_MAX)
			{
				std::cout << "\nError: invalid edge count of random graph (" << edgesPerNode << ") ";
				return false;
			}
			G = new MyGraph(true);
			fprintf(stderr,"gen-ing graph\n");
			G->generateGraph(numNode, numTypes, edgesPerNode, args,true);
			continue;
		}
		if(argv[nextArg]=="-gen")
		{++nextArg;
			if(!getGenerateMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-search")
		{++nextArg;
			if(!getSearchMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-sim")
		{++nextArg;
			if(!getSimilarityMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-n")
		{++nextArg;
			if(!getNTests(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-dist")
		{++nextArg;
			if(!getDistMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-distort")
		{++nextArg;
			if(!getDistortion(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-anchor")
		{++nextArg;
			if(!getAnchorID(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-i")
		{++nextArg;
			if(!getIDegree(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-w")
		{++nextArg;
			if(!getWeights(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-partMethod")
		{++nextArg;
			args.partitionMethod=argv[nextArg][0];
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-success")
		{++nextArg;
			args.successMeth=argv[nextArg][0];
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-overlapPercent")
		{++nextArg;
			args.overlapPercent=strtod(argv[nextArg].c_str(), NULL);
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-maxMaps")
		{++nextArg;
			args.maxMaps=std::atoi(argv[nextArg].c_str());
			++nextArg;
			continue;
		}
		fprintf(stderr,"unknown arg %s\n",argv[nextArg].c_str());
		exit(0);
	}
	fprintf(stderr,"graphfile = %s\n",graphFile.c_str());
	if(graphFile.compare(lastGraphFile)!=0)
	{
		lastGraphFile=graphFile;
		if(import)
		{
			bool plain=true;
			G = new MyGraph(graphFile, true, p, true, args.partitionMethod2, true, args);
		}
		args.graphFile=graphFile;
	}
	if(G->getNumNodes() <= args.fragSize || (args.fragSize == 0 && args.fragFile.size()==0))
	{
		std::cout << "\nError: Invalid fragment size (" << args.fragSize << ") not in range (0, " << G->getNumNodes() << ")";
		return false;
	}
	fprintf(stderr,"done with args\n");
	args.outputPrefix="./output/";
	for(unsigned int i=1;i<argv.size();i++)
	{
		if(argv[i-1].compare("-g")==0)
		{
			args.outputPrefix+=argv[i].substr(13,argv[i].size()-13)+"_";
		}
		else
		{
			args.outputPrefix+=argv[i]+"_";
		}
	}
	fprintf(stderr,"args.outputPrefix= %s\n",args.outputPrefix.c_str());
	return true;
}
