/*	Written May 6, 2011 
*	The purpose of this file is to time the various testing criteria components
*/

#include <cstdlib>
#include <cstring>
#include <iomanip>
#include <iostream>
#include <map>
#include <vector>
#include <time.h>
#include <string>
#include <fstream>	

//#include <windows.h>
//#define sleep(n) Sleep(1000 * n)

#include "Container.h"
#include "graph.h"
#include "utilities.h"
#include "searcher.h"
#include "scorer.h"


#define CONSTANT_SEED
#define BACKTRACK

//#define TRACE


/*** Global Variables***/
unsigned int seed;
std::string lastGraphFile;
arguments args; //set in getArguments

#ifdef CONSTANT_SEED
std::vector<unsigned int> indicesToGenFrag;
#endif

float overlap(0);
unsigned int successfulOverlaps(0);

/*** Function Prototypes ***/
bool getArguments(std::vector<std::string> argP, MyGraph *&G);


//fill in the vector with potential candidates to anchor the center to
void generateCandidates(MyGraph * G, MyGraph * fragment, unsigned int center, std::vector<std::pair<double,std::map<unsigned int,unsigned int> > >& candidateIDs, unsigned int nBest);

void generateFrag(MyGraph& fragment, MyGraph* database,const MyNode* centerNode);

void runTest(char centerMethod, MyGraph * database);

void realTest(char centerMethod, MyGraph * database, unsigned int nBins);
	
float successFunc2(std::map<unsigned int, unsigned int> candidate, MyGraph* fragment,unsigned int center)
{
	unsigned int totalSize=fragment->getNumNodes();
	unsigned int correctValues=0;
	if(center==(unsigned int)-1){
	for(std::map<unsigned int, unsigned int>::iterator itr=candidate.begin();
		itr!=candidate.end();itr++)
	{
		correctValues+=(itr->first==itr->second);
	}}
	else
	{
		correctValues=candidate[center]==center;
		return correctValues*100;
	}
	//std::cout<<correctValues<<" out of "<< candidate.size() <<"\t "<<(((float)totalSize)/(float)totalSize)<<"\n";
	if((((float)correctValues)/(float)totalSize) >= 0.1)
	{return (((float)correctValues)/(float)totalSize );
	}
	return -1;
}
float successFunction(std::vector< std::pair<double, std::map<unsigned int, unsigned int> > > candidates,MyGraph* fragment, int &rank, int &order,unsigned int center=(unsigned int)-1)
{
	rank=0;
	for(order=0;order<candidates.size();order++)
	{
		if(order>0 && (candidates[order-1].first !=candidates[order].first))
		{
			rank++;
		}
		float result=successFunc2(candidates[order].second,fragment,center);
		if(result>=0)
		{
			return result;
		}
	}
	return -1;
}
unsigned int generateFragCenterSingle(MyGraph* database)
{
	unsigned int edgeCount = 0;
	unsigned int centerIndex=0;
	do
	{
		centerIndex = rand() % database->getNumNodes();
		edgeCount = (database->getNodeByIndex(centerIndex))->getNumOutNeighbors(); 
	}while(edgeCount <= FEW_EDGES);
	return centerIndex;
}
//unsigned int seed;
void generateFragCenters(MyGraph* database)
{
	indicesToGenFrag.resize(args.nTests, 0);
	unsigned int num_nodes = database->getNumNodes();
	std::map<unsigned int, unsigned int> neighborhood;
	for(unsigned int i = 0; i < args.nTests; ++i)
	{
		indicesToGenFrag[i] = generateFragCenterSingle(database);
	}
}
/*** MAIN ***/
int main(int argc, char ** argv)
{
	time_t start = time(NULL);
	//unsigned int seed = (unsigned int)time(NULL);
	seed = /*1311746957;//*/2357;
	MyGraph * database;
	srand(seed);
	bool testing=true;
	args.T.start("reading args");
	//read in any changes to default arguments
	fprintf(stderr,"accepting arguments\n");
	std::vector<std::string> argP;
	for(unsigned int i=0;i<(unsigned)argc;i++)
	{
		argP.push_back(argv[i]);
	}
	bool success = getArguments(argP, database);
	std::cerr<<"done testing loading database\n";
	if(!success)
	{
		std::cout << "\nincorrect args, see documentation\n";
		return 1;	//exit failure
	}
	std::string timingFileName=args.outputPrefix;
	timingFileName+=".time";
	FILE* timingOutput=fopen(timingFileName.c_str(),"w");
	std::string mxFileName=args.outputPrefix;
	mxFileName+=".MX";
	args.mXoutput=fopen(mxFileName.c_str(),"w");
	std::string mx2FileName=args.outputPrefix;
	mx2FileName+=".MX2";
	args.mXoutput2=fopen(mx2FileName.c_str(),"w");
	positioningArgument p;
	p.method = args.i_degree;//default args.i_degree, assuming no index file is provided
	args.T.stop("reading args");
	args.T.print(stderr);
	if(testing)
	{
		std::cerr<<"*Database Loaded*\n";
		std::string pauseString;
		std::cin>>pauseString;
		generateFragCenters(database);
		std::cerr<<"Generated Fragment Centers\n";
		
		for(unsigned int i=0;i<args.nTests;i++)
		{
			fprintf(stderr,"\nTest %d\n",i+1);
			MyGraph* fragment;//create frag
			fprintf(stderr,"Gen Fragment\n");
			args.T.start("gen fragment");
			MyNode* tempNode1,*centerNode1;
			if(args.fragFile!="")
			{
				p.method = 3;
				fragment=new MyGraph(args.fragFile, true, p, true, 'N', true, args);
				fprintf(stderr,"Fragsize == %d\n",fragment->getNumNodes());
			}
			else{
				fragment=new MyGraph(database->isDirected());
				generateFrag(*fragment, database,database->getNodeByIndex(indicesToGenFrag[i]));
				//generate Fragment
				if(fragment->getNumNodes()<args.minFragSize)
				{
					fprintf(stderr,"--Fragment too small (%d<%d), retry\n",fragment->getNumNodes(),args.minFragSize);
					indicesToGenFrag[i]=generateFragCenterSingle(database);
					i--;
					continue;
				}
				fprintf(stderr, "\nfragment originator in database: \n");
				const MyNode* tempNode=database->getNodeByIndex(indicesToGenFrag[i]);
				const MyNode* centerNode=fragment->getNodeById(tempNode->getID());
				printDegree(centerNode, stderr);
			}
			
			args.T.stop("gen fragment");
			
			fprintf(stderr,"Number of nodes %d edges %lu\n",fragment->getNumNodes(),fragment->getNumEdges());
			//print fragment info to cerr
			std::cerr << "\nExperiment " << i+1 << " of " << args.nTests <<":\n" << std::flush;
			
		
			args.T.start("Testing");
			args.T.start("search step");
			Searcher searcher(database->indexing,fragment,args,database);//create searcher
			args.T.stop("search step");
			/*
			//create mappings
			args.T.start("map step");
			CandidateList L=searcher.mapFragment(database,args);
			args.T.stop("map step");
			//score mappings
			args.T.start("score step");
			Scorer scorer;
			if(args.successMeth=='o')
			{
				scorer.overlapMethod(L,args.overlapPercent);
			}
			else if(args.successMeth=='a')
			{
				scorer.anchorMethod(L,centerNode1->getID());
			}
			args.T.stop("score step");
			args.T.stop("Testing");
			delete fragment;*/
		}
		args.T.print(stderr);
		args.T.print(timingOutput);
		//realTest('o', database, args.nTests);
		fclose(timingOutput);
		fclose(args.mXoutput);
		fprintf(stderr,"Done\n");
		exit(0);
	}
	
	//get command line arguemnts (if there is a problem, getArguments will print the detailed error message
	if(!testing && (argc==2 || argc==1))
	{
		//run without arguments
		//load graph
		if(argc==2)
		{
			std::cout<<"loading database\n";
			positioningArgument p;
			p.method = 3;
			lastGraphFile=argv[1];
			args.partitionMethod='b';//use standard out/in partitioning....//TODO?!?! make this take as input
			//doesn't take in arguments by default, so set some values
			args.searchMethod='9';
			database = new MyGraph(lastGraphFile, true, p, true, true, true, args);//modification 6/6/2013 partitions the default database using argument method, instead of always i-degree method
			std::cout<<"Database Loaded.\n";
		}
		else
		{
			std::cout<<"No default database\n";
		}
		while(true)
		{//perform jobs
			{
				std::cout<<"waiting for ready signal\n";
				std::ifstream ready("ready.txt");
				while( !ready )
				{
					std::cout<<".\n";
					sleep(5);
					ready.open("ready.txt");
				} 
			}
			std::cout<<"Ready...\n";
			std::ifstream argFile("args.txt");
			std::vector<std::string> argP;
			std::string newArg;
			while(getline(argFile,newArg))
			{
				argP.push_back(newArg);
			}
			bool success = getArguments(argP, database);
			if(!success)
			{
				std::cout << "\nincorrect args, see documentation\n";
				return 1;	//exit failure
			}
			std::cout<<"Searching...\n";
			realTest('o',database, args.nTests);
			remove("ready.txt");
		}
		return 0;
	}
	
	std::cerr << "\nBegin testing" << std::flush;

	srand(seed);
	std::cerr << "\nSeed = " << seed << "\n";
	time_t end = time(NULL);
	std::cerr<<"index time= "<<difftime(end, start)<<std::flush<<std::endl;
	
	generateFragCenters(database);


if(args.generateMethod!='f')
{
	//	runTest('d',generateMethod, database, overlap, fragSize, distortion, nTests);
	for(args.closeness = 0; args.closeness <= 0; ++args.closeness)
	{
		for(args.i_degree = 3; args.i_degree >=3; --args.i_degree)
		{
		//fprintf(stdout,"Performing a test on idegree %d\n",args.i_degree);
			for(args.successMethod = 0; args.successMethod >=0; --args.successMethod)
			{
				std::cerr << "\nRunning test: \n";
				std::cerr << "fragment(generation; size; distortion),search, similarity, closeness, i_degree, success, samples\n";
				std::cerr << "(\'" << args.generateMethod << "\'; " << args.fragSize << "; " << args.distortion << "), \'"<< args.searchMethod<< "\', \'"<< args.similarityMethod<<"\', " << args.closeness << ", " << args.i_degree << ", " << args.successMethod << ", " << args.nTests << "\n";
				runTest('o', database);
			}
			if(args.similarityMethod == 'r')
			{
				args.i_degree = -1;//terminate this loop after 1 iteration
			}
		}
	}
}
else
{
	realTest('o', database, args.nTests);
}
	std::cout << std::endl << std::endl;
	return 0;
}


/*** Function Definitions ***/
/*							 
std::cout << "\nusage: \"evaluate <graph spec> <frag gen spec> <search spec> <similarity spec> #samples\"";
std::cout << "\n\t<Graph spec> :\t1) fileName {1|2|3|<locality.ppg}";
std::cout << "\n\t              \t2) numNodes, numTypes, edgesPerNode";
std::cout << "\n\t<frag gen spec>:\t1) 'o' numNodes distortion";
std::cout << "\n\t                \t2) 's'|'e'|'n' numNodes";
std::cout << "\n\t<search spec>:\t1) 's' single center";
std::cout << "\n\t              \t2) 'p' adjacent pair";
std::cout << "\n\t              \t2) 'm' multiple searches based on an adjacent pair";
std::cout << "\n\t              \t2) 't' tree: central and two adjacent nodes";
std::cout << "\n\t              \t2) 'c' chain: 3 nodes ";
std::cout << "\n\t              \t2) 'q' quadruple: central and three adjacent nodes";
std::cout << "\n\t<similarity spec>:\t1) 'd' ";
std::cout << "\n\t                 :\t2) 'r' \n\n";
*/
bool getFragFile(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.fragFile = argv[nextArg];
	++nextArg;
	return true;
}
bool getFragSize(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.fragSize = atoi(argv[nextArg].c_str());
	++nextArg;
	args.minFragSize = atoi(argv[nextArg].c_str());
	++nextArg;
	return true;
}
bool getGenerateMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.generateMethod = argv[nextArg][0];
	++nextArg;
	switch(args.generateMethod){
		case 'o':
		case 'c':
		case 's':
		case 'n':
		case 'p':
		case 'e':
		case 'k':
			break;
		case 'f':
			if(args.generateMethod=='f')
			{
				if(!getFragFile(argv,nextArg))
					{return false;}
			}
			return true;
			break;
		default:
			std::cout << "\nError: Invalid fragment generation method (" << args.generateMethod << ")";
			return false;
	}
	if(!getFragSize(argv,nextArg))
		{return false;}
	return true;
}

bool getDistortion(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.distortion = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	if(args.distortion < 0 || args.distortion >= .1)
	{
		std::cout << "\nError: Invalid distortion (" << args.distortion << " not in (0.0, 0.1)";
		return false;
	}
	return true;
}
bool getSearchMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.searchMethod = argv[nextArg][0];
	++nextArg;
	switch(args.searchMethod)
	{
		case 's':
		case 'p':
		case 'm':
		case 't':
		case 'c':
		case 'I':
			break;//these are acceptable.
		default:
			if(!isdigit(args.searchMethod) )
			{
				std::cout << "\nError: Invalid search method (\'" << args.searchMethod << "\')";
				return false;
			}
			break;
	}
	return true;
}
bool getSimilarityMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.similarityMethod =argv[nextArg][0];
	++nextArg;
	switch(args.similarityMethod)
	{
		case 'd':
		case 'r':
		case 'f':
		case 'F':
		case 'e':
		case 'E':
			break;//these are acceptable.
		default:
			std::cout << "\nError: Invalid similarity method (" << args.similarityMethod << ")";
			return false;
	}
	return true;
}
bool getNTests(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.nTests = atoi(argv[nextArg].c_str());
	++nextArg;
	if(args.nTests <= 0)
	{
		std::cout << "\nError: Invalid number of tests(" << args.nTests << ")";
		return false;
	}
	return true;
}

bool getAnchorID(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.anchor = atoi(argv[nextArg].c_str());
	++nextArg;
	if(args.anchor <= 0)
	{
		std::cout << "\nError: Invalid anchor(" << args.anchor << ")";
		return false;
	}
	return true;
}

//gets the method to use for measuring distance between i-degrees
bool getDistMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.distMethod =argv[nextArg][0];
	++nextArg;
	switch(args.distMethod)
	{
		case '1'://x^1
		case '2'://x^2
		case 'S'://x^.5
			break;//these are acceptable.
		default:
			std::cout << "\nError: Invalid distMethod (" << args.distMethod << ")";
			return false;
	}
	return true;
}
bool getIDegree(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.i_degree =argv[nextArg][0]-'0';
	++nextArg;
	switch(args.i_degree)
	{
		case 0:
		case 1:
		case 2:
		case 3:
			break;//these are acceptable.
		default:
			std::cout << "\nError: Invalid iDegree(" << args.i_degree << ")";
			return false;
	}
	return true;
}
bool getWeights(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.wI[0] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wI[1] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wI[2] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wO[0] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wO[1] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wO[2] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	return true;
}
bool getArguments(std::vector<std::string> argv, MyGraph *&G)
{
	//set default arguments
	args.outputPrefix="defaultOutput";
	args.useEdgeLabels=false;
	args.searchMethod='9';		
	args.similarityMethod='F';//for use in i-degree algorithm//F=='bipartite matching, in/out edges
	args.i_degree=3;
	args.closeness=0;//not currently internal use
	args.successMethod=0;//currently internal use
	args.distMethod='1';
	args.generateMethod='p';//prandom walk to generate fragments
	args.partitionMethod='o';//'o','i','s','b'//which edges to use
	args.partitionMethod2='i';//describes the algorithm to index with
	args.fragSize=10;//size of the fragment
	args.nTests=1;//number of fragments to test
	args.distortion=0.0;//distortion is used in fragment generation, for some algorithms
	args.wI[0]=args.wI[1]=args.wI[2]=1.0;//default 1//in weights
	args.wO[0]=args.wO[1]=args.wO[2]=1.0;//default 1//out weights
	args.fragFile="";
	args.anchor=0;
	args.minFragSize=1;
	args.seedNum=2357;
	//take in arguments now
	unsigned int nextArg=1;
	positioningArgument p;
	p.method = args.i_degree;
	std::string graphFile=lastGraphFile;
	bool import = false; 
	unsigned int numNode=0, numTypes=0, edgesPerNode=0;
	while(nextArg<argv.size())
	{
		//std::cout<<argv[nextArg]<<std::endl;
		if(argv[nextArg]=="-edgeLabels")
		{++nextArg;
			args.useEdgeLabels=atoi(argv[nextArg].c_str())>0;
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-seed")
		{++nextArg;
			args.seedNum = atoi(argv[nextArg].c_str());
			srand(args.seedNum);
			seed=args.seedNum;
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-f")
		{++nextArg;
			import = true;
			p.method = atoi(argv[nextArg].c_str());
			args.i_degree=atoi(argv[nextArg].c_str());
			p.file = argv[nextArg];
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-g")
		{nextArg++;
			graphFile=argv[nextArg];
			nextArg++;
			continue;
		}
		if(argv[nextArg]=="-randGraph")
		{++nextArg;
			numNode = atoi(argv[nextArg].c_str());
			++nextArg;
			if(numNode == 0 || numNode >= UINT_MAX)
			{
				std::cout << "\nError: invalid size of random graph (" << numNode << ") ";
				return false;
			}
			numTypes = atoi(argv[nextArg].c_str());
			++nextArg;
			if(numTypes == 0 || numTypes >= UINT_MAX)
			{
				std::cout << "\nError: invalid type count of random graph (" << numTypes << ") ";
				return false;
			}
			edgesPerNode = atoi(argv[nextArg].c_str());
			++nextArg;
			if(edgesPerNode == 0 || edgesPerNode >= UINT_MAX)
			{
				std::cout << "\nError: invalid edge count of random graph (" << edgesPerNode << ") ";
				return false;
			}
			G = new MyGraph(true);
			fprintf(stderr,"gen-ing graph\n");
			G->generateGraph(numNode, numTypes, edgesPerNode, args,true);
			continue;
		}
		if(argv[nextArg]=="-gen")
		{++nextArg;
			if(!getGenerateMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-search")
		{++nextArg;
			if(!getSearchMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-sim")
		{++nextArg;
			if(!getSimilarityMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-n")
		{++nextArg;
			if(!getNTests(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-dist")
		{++nextArg;
			if(!getDistMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-distort")
		{++nextArg;
			if(!getDistortion(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-anchor")
		{++nextArg;
			if(!getAnchorID(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-i")
		{++nextArg;
			if(!getIDegree(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-w")
		{++nextArg;
			if(!getWeights(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-partMethod")
		{++nextArg;
			args.partitionMethod=argv[nextArg][0];
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-success")
		{++nextArg;
			args.successMeth=argv[nextArg][0];
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-overlapPercent")
		{++nextArg;
			args.overlapPercent=strtod(argv[nextArg].c_str(), NULL);
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-maxMaps")
		{++nextArg;
			args.maxMaps=std::atoi(argv[nextArg].c_str());
			++nextArg;
			continue;
		}
		fprintf(stderr,"unknown arg %s\n",argv[nextArg].c_str());
		exit(0);
	}
	fprintf(stderr,"graphfile = %s\n",graphFile.c_str());
	if(graphFile.compare(lastGraphFile)!=0)
	{
		lastGraphFile=graphFile;
		if(import)
		{
			bool plain=true;
			G = new MyGraph(graphFile, true, p, true, args.partitionMethod2, true, args);
		}
		args.graphFile=graphFile;
	}
	if(G->getNumNodes() <= args.fragSize || (args.fragSize == 0 && args.fragFile.size()==0))
	{
		std::cout << "\nError: Invalid fragment size (" << args.fragSize << ") not in range (0, " << G->getNumNodes() << ")";
		return false;
	}
	fprintf(stderr,"done with args\n");
	args.outputPrefix="./output/";
	for(unsigned int i=1;i<argv.size();i++)
	{
		if(argv[i-1].compare("-g")==0)
		{
			std::string tempStr=argv[i].substr(13,argv[i].size()-13);
			std::replace(tempStr.begin(),tempStr.end(),'/','_');
			args.outputPrefix+=tempStr+"_";
		}
		else
		{
			std::string tempStr=argv[i];
			std::replace(tempStr.begin(),tempStr.end(),'/','_');
			args.outputPrefix+=tempStr+"_";
		}
	}
	fprintf(stderr,"args.outputPrefix= %s\n",args.outputPrefix.c_str());
	return true;
}


//fill in the vector with potential candidates to anchor the center to
void generateCandidates(MyGraph * G, MyGraph * fragment, unsigned int center, std::vector<std::pair<double,std::map<unsigned int,unsigned int> > >& candidateIDs, unsigned int nBest)
{
#ifdef TRACE
	std::cerr << "\nIn generateCandidates" << std::flush;
	std::cerr << "\n\tG->find_by_i(" << center << ", " << similarityMethod << ", " << i_degree << ",15)" << std::flush;
#endif

	candidateIDs.clear();
	//Temporary. TODO: Perminant solution
	/*std::list<std::pair<double,struct Node*> > candidate_list = G->find(fragment);
	for(std::list<std::pair<double,struct Node*> >::iterator itr=candidate_list.begin();itr!=candidate_list.end();itr++)
	{
	candidateIDs.insert(candidateIDs.end(),itr->second->getID());
	}*/

	time_t start = time(NULL);
	switch(args.searchMethod)
	{
		case 'I':
			candidateIDs = G->find_by_itPart(fragment->getNodeById(center),fragment,args);
			break;
		case 's':
			fprintf(stderr,"Depricated find_by_i\n");
			//candidateIDs = G->find_by_i(fragment->getNodeById(center), args.similarityMethod, args.i_degree, nBest, args.distMethod,args);
			break;
		case 'p':
			fprintf(stderr,"Depricated find_by_pair\n");
			//candidateIDs = G->find_by_pair(fragment->getNodeById(center), fragment->getMostDiverseNeighbor(center),args.similarityMethod, args.i_degree,nBest, args.distMethod,args);
			break;
		case 'm':
			fprintf(stderr,"Depricated multi_find_by_i\n");
			//candidateIDs = G->multi_find_by_i(fragment->getNodeById(center), args.similarityMethod, args.i_degree, nBest,args.distMethod,args);
			break;
		case 't':
			fprintf(stderr,"Depricated find_by_tree");
			//candidateIDs = G->find_by_tree(fragment->getNodeById(center), fragment, args.similarityMethod, args.i_degree, nBest,2,args.distMethod,args);
			break;
		default:
			if(isdigit(args.searchMethod))
			{
				if(args.searchMethod=='9')
				{
					candidateIDs = G->find_by_NtreeNeighborhood(fragment->getNodeById(center), fragment, args.similarityMethod, args.i_degree, nBest,9999999 ,args.distMethod,args);
				}
				else
				{
					candidateIDs = G->find_by_NtreeNeighborhood(fragment->getNodeById(center), fragment, args.similarityMethod, args.i_degree, nBest,(args.searchMethod - '0') ,args.distMethod,args);
				}
			}
			else
			{
				std::cerr << "\n\tERROR: what search method is this (" << args.searchMethod << ")?";
				return;
			}
	}
	/*else if(searchMethod == 'c')
	{
		std::vector<const Node *> chainLinks;
		chainLinks.push_back(fragment->getNodeById(center));
		chainLinks.push_back(fragment->getMostDiverseNeighbor(center));
		unsigned int limits[2] = {chainLinks[0]->getID(), chainLinks[1]->getID()};
		chainLinks.push_back(fragment->getMostDiverseNeighbor(chainLinks[1]->getID(), limits, 2) );

		if(chainLinks[2] == NULL)
		{
			std::cerr << "\nCannot achieve a fragment with a chain of 3";
			return; 
		}

		candidateIDs = G->find_by_chain(chainLinks, similarityMethod, i_degree, nBest);
	}*/
	
	time_t finish = time(NULL);
	std::cerr << "\n\tCandidate Generation: " << difftime(finish, start) << std::endl<<std::flush;

#ifdef TRACE
	std::cerr << "\n #candidates: " << candidateIDs.size();
#endif
}

void generateFrag(MyGraph& fragment, MyGraph* database, const MyNode* centerNode)
{
	positioningArgument p;
		switch(args.generateMethod)
		{
#ifdef CONSTANT_SEED
			case 'c'://complete
				fragment = database->getSubgraphToDegreeById(centerNode->getID(),2,false,args,false);
				break;
			case 'p':
				fragment.generateFragment_pRandomWalk(database,centerNode,args.fragSize,0.9, 0.7,args);
				break;
#else
			case 'c'://complete
				do
				{
					centerIndex = rand() % num_nodes;
					edgeCount = (database->getNodeByIndex(centerIndex))->outgoing_edges.size(); 
				}while(edgeCount <= FEW_EDGES || edgeCount >= fragSize);
				fragment = database->getSubgraphToDegreeById(database->getNodeById(centerIndex)->getID(), 2, false, false);
				break;
			case 'o':
				fragment = database->generateFragment(fragSize, args.distortion);
				break;
			case 's':
				fragment = database->generateFragment_staticprob(fragSize);
				break;
			case 'e':
				fragment = database->generateFragment_exactsize(fragSize);
				break;
			case 'n':
				fragment = database->generateFragment_rand1N(fragSize);
				break;
			case 'k':
				fragment = database->generateFragment_kRandomWalk(fragSize,2);
				break;
			case 'p':
				fragment = database->generateFragment_pRandomWalk(fragSize,0.9, 0.7);
				break;
#endif

		default:
			//THIS SHOULD NEVER BE REACHED
			std::cerr<< "\nError Invalid generation method (" << args.generateMethod << "),\nExiting program\n\n";
			exit(1);//exit failure  
		}
}

//this will run one or more tests, and then it will
//exit the program
void runTest(char centerMethod,MyGraph * database)
{

#ifndef CONSTANT_SEED
	unsigned int edgeCount = 0;
	unsigned int centerIndex;
	unsigned int num_nodes = database->getNumNodes();
	unsigned int randIn;
#endif

	//timing info
	time_t start, finish, tempStart, tempFinish;
	double fragGen=0, canGen=0, containerGen=0;//, successTime=0;

	//run the specified tests
	std::vector<std::pair<double,std::map<unsigned int, unsigned int> > > candidates;
	candidates.reserve(120);//average fuzziness*10
	std::vector<std::map<unsigned int, unsigned int> > bestMaps;
	bestMaps.reserve(15);//10 *candidates

	//int successes = 0;
	//int successfulTests = 0;
	bool testSuccess;
	unsigned int nCandidates = 0;

	const unsigned int nBins = 200;
	const unsigned int atLeast = 6;
	std::vector<unsigned int> binCount[10];

	std::map<unsigned int, unsigned int> histo_idist[2];//[0] is order [1] is strictly rank;


	for(unsigned int i = 0; i < atLeast; ++i)
	{
		binCount[i].resize(nBins, 0);
	}

	stats_keeper fragSizeStats;

	start = time(NULL);
	for(unsigned int test = 1; test <= args.nTests; ++test)
	{
		MyGraph fragment(database->isDirected());
		std::cerr << "\nExperiment " << test << " of " << args.nTests <<":" << std::flush;
		fprintf(stderr, "\nfragment originator in database: \n");
		const MyNode* centerNode=database->getNodeByIndex(indicesToGenFrag[test-1]);
		//centerNode=database->getNodeById(3647);
		printDegree(centerNode, stderr);

		testSuccess = false;
		//generate the fragment
		//be sure that all fragment generation methods call initPartitions***
		tempStart = time(NULL);
		generateFrag(fragment, database,centerNode);
		std::cerr << "fragment.size() = " << fragment.getNumNodes()<<"\n";


		tempFinish = time(NULL);
		fragGen += difftime(tempFinish, tempStart);
		std::cerr << "\n\tFragment Generation: " << difftime(tempFinish, tempStart) << std::flush;
		//std::string fname="./fragments/frag"+my_itoa(seed,10)+".grf";
		//fragment->writePlainGraph(fname.c_str());

		if(fragment.getNumNodes() == 1)
		{
			const MyNode * node = database->getNodeById(fragment.getOriginator()->getID());
			std::cerr<< "\nError: fragment is single node (" << node->getLabel() << ", " << node->getID() << ")";
			const edgelist* outedges=node->getOutEdgesConst();
			for(edgelist::const_iterator m=outedges->begin(); m !=outedges->end(); ++m)
			{
				std::cerr << "\n\t" << node->getID() << " " << m->first;
			}
		}

		fragSizeStats.update(fragment.getNumNodes());

		//get the center
		tempStart = time(NULL);
		unsigned int center;
		switch(centerMethod)
		{ 
		case 'd':
			std::cerr << "\n\tfinding center by diverse";
			center= (fragment.centerDiverse())->getID();
			break;
		case 'o':
			center = fragment.getOriginator()->getID();
			break;
		default:
			fprintf(stderr,"Error, unknown centerMethod\n");
			exit(0);
		}
		tempFinish = time(NULL); 
		std::cerr << "\n\tFinding Center: " << difftime(tempFinish, tempStart) << std::flush;
		//printDegree(fragment->getNodeById(center), stderr);

		if(fragment.getNodeById(center) == NULL)
		{
			std::cout << "\nError: Null center" << std::flush;
			exit(1);
		}


		tempStart = time(NULL);
		//generate the candidates
		generateCandidates(database, &fragment, center, candidates, UINT_MAX);
		fprintf(stderr,"ok\n");
		tempFinish = time(NULL);
		canGen += difftime(tempFinish,tempStart);

		if(candidates.size()==0)
		{
			fprintf(stdout,"No Results for search\n");
		}
		else{
			int order = -1, rank = candidates.size(), curRank = 1;
			bool found = false;
			int correctindex=0;
			float overlapVal;
			overlapVal = successFunction(candidates,&fragment,rank,order,center);
			if(overlapVal>0)
			{
				overlap+=overlapVal;
				successfulOverlaps+=1;
			}
			fprintf(stdout," o, %d\n",order);
			//std::cout<<"found = "<<found<<"\n"<<std::flush;
			//print degrees
			/*if(order<200)
			{
				FILE * a;
				a=fopen("inoutGood7.txt","a");
				//fprintf(a,"FNode: %d\n",center);
				//printDegree(fragment->getNodeById(center),a);
				//fprintf(a,"GNode: %d\n",center);
				//printDegree(database->getNodeById(center),a);
				unsigned int i=0;
				for(;i<200 &&i<=order && i<candidates.size();i++)
				{
					fprintf(a,"%f, ",candidates[order].first-candidates[i].first);
					//printDegree(database->getNodeById(candidates[i].second),a);
				}
				for(;i<200;i++)
				{
					fprintf(a,"-1.0, ");
					//printDegree(database->getNodeById(candidates[i].second),a);
				}
				fprintf(a,"\n");
				fclose(a);
			}*/
			if(order!=-1)
			{
				std::map<unsigned int, unsigned int>::iterator it; 
				it = histo_idist[0].find(order);
				if(it == histo_idist[0].end() ) 
				{
					histo_idist[0][order] = 1;
				}
				else
				{
					it->second++;
				}

				it = histo_idist[1].find(rank);
				if(it == histo_idist[1].end() ) 
				{
					histo_idist[1][rank] = 1;
				}
				else
				{
					it->second++;
				}
			}
		}
			nCandidates += candidates.size();
	}
	fprintf(stdout,"%f, %f, %f\n",fragSizeStats.getMin(),fragSizeStats.getMax(),fragSizeStats.getAverage());

	fprintf(stdout,"average overlap% = %f\n",((float)overlap)/((float)successfulOverlaps));
	printf("\n\nOrder, Percent");
	float tot = 0;
	for(std::map<unsigned int, unsigned int>::iterator i = histo_idist[0].begin(); 
		i != histo_idist[0].end(); ++i)
	{
		tot += (float)i->second / (float)args.nTests;
		printf("\n%u, %f", i->first, tot);
	}

	printf("\n\nRank, Percent");
	tot = 0.0;
	for(std::map<unsigned int, unsigned int>::iterator i = histo_idist[1].begin(); 
		i != histo_idist[1].end(); ++i)
	{
		tot += (float)i->second / (float)args.nTests;
		printf("\n%u, %f", i->first, tot);
	}

	finish = time(NULL);
//timing output
	double totalTime = difftime(finish, start);
	double fragTime  = (fragGen) / args.nTests;
	double canTime   = canGen / args.nTests;
	double contTime  = containerGen/nCandidates;

	std::cerr << "\n\nTotal Time, Avg. Frag Gen Time, Avg. Candidate Search, Avg. Container Generation\n";
	std::cerr << std::setprecision(6) << std::fixed << totalTime << ", " << fragTime << ", " << canTime << ", " << contTime << "\n";

	fflush(stdout);
	fflush(stderr);
	return;
}
