/*	Written May 6, 2011 
*	The purpose of this file is to time the various testing criteria components
*/

#include <cstdlib>
#include <cstring>
#include <iomanip>
#include <iostream>
#include <map>
#include <vector>
#include <time.h>
#include <string>
#include <fstream>	
#include <sstream>

//#include <windows.h>
//#define sleep(n) Sleep(1000 * n)

#include "graph.h"
#include "utilities.h"


#define CONSTANT_SEED
#define BACKTRACK

//#define TRACE


/*** Global Variables***/
unsigned int seed;
std::string lastGraphFile;
arguments args; //set in getArguments

#ifdef CONSTANT_SEED
std::vector<unsigned int> indicesToGenFrag;
#endif

float overlap(0);
unsigned int successfulOverlaps(0);

/*** Function Prototypes ***/
bool getArguments(std::vector<std::string> argP, MyGraph *&G);


bool loadStrings(std::map<unsigned int, std::map<std::string, unsigned int> > &strings, std::string filename,
	std::map<std::string,bool>& uniqueStrings)
{
	std::ifstream input(filename);
	std::string line;
	while(!input.eof())
	{
		std::getline(input,line);
		std::istringstream ss( line );
		// new container of words
		std::vector<std::string> words;
		// parse out words using string stream and store words in container
		std::string word;
		while( ss >> word )
		{
			words.push_back( word );
		}
		if(words.size()==0)
		{
			break;
		}
		std::map<std::string,unsigned int> newString;
		for(unsigned int i=1;i<words.size();i+=2)
		{
			newString[words[i]]=std::atoi(words[i+1].c_str());
			uniqueStrings[words[i]]=true;
		}
		strings[std::atoi(words[0].c_str())]=newString;
	}
	return true;
}


struct treeNode
{
	public:
	std::string coordinate;
	std::map<unsigned int, treeNode*> subpartitions;
	std::list<unsigned int> leafNodes;
};
struct treeIndex
{
	public:
	std::list<treeNode> allTreeNodes;
	treeNode* root;
};
bool craftTree(std::map<unsigned int, std::map<std::string, unsigned int> > databaseStrings, std::map<std::string,bool>& uniqueStrings, treeIndex& index)
{unsigned int count=0;
	index.allTreeNodes.push_back(treeNode());
	index.root=&(index.allTreeNodes.front());
	//add each node one at a time
	for(std::map<unsigned int, std::map<std::string,unsigned int> >::iterator itr=databaseStrings.begin();itr!=databaseStrings.end();itr++)
	{
		//default current pos to the root
		treeNode* currentPos=index.root;
		std::map<std::string,unsigned int>::iterator descriptor=itr->second.begin();
		for(std::map<std::string,bool>::iterator strItr=uniqueStrings.begin();
			strItr!=uniqueStrings.end();strItr++)
		{//for each possible index string, in order
			if(currentPos->coordinate=="")
			{
				currentPos->coordinate=strItr->first;
			}
			unsigned int v=0;
			if(descriptor!=itr->second.end() && descriptor->first ==strItr->first)
			{//descriptor is correct, index with the value at this coordinate
				v=descriptor->second;
				descriptor++;
			}//else it was toofar ahead, and we should index this string with value 0
			
			//check if this indexvalue yet exists at the current position
			std::map<unsigned int, treeNode*>::iterator findVal=currentPos->subpartitions.find(v);
			if(findVal==currentPos->subpartitions.end())
			{//the indexvalue v doesn't exist at this position in the tree
				index.allTreeNodes.push_back(treeNode());
				currentPos->subpartitions[v]=&(index.allTreeNodes.back());
				currentPos=currentPos->subpartitions[v];
				if(v==0){count++;}
			}
			else
			{
				currentPos=findVal->second;
			}
		}
		//after each indexvalue from unique strings, add graph nodes to the current position in the index
		currentPos->leafNodes.push_back(itr->first);
	}
	fprintf(stderr,"0's   Size =%d\n",count);
	fprintf(stderr,"index Size =%d\n",index.allTreeNodes.size());
	return true;
}

bool mxCalc(treeIndex& index, std::map<unsigned int, std::map<std::string, unsigned int> > fragmentStrings, std::map<std::string,bool>& uniqueStrings,FILE* output,
	std::map<unsigned int, std::list<unsigned int> >& Mx)
{
	for(std::map<unsigned int, std::map<std::string, unsigned int> >::iterator nodeItr=fragmentStrings.begin();nodeItr!=fragmentStrings.end();nodeItr++)
	{
		std::list<treeNode*> stack;
		stack.push_back(index.root);
		while(stack.size()>0)
		{
			if(stack.back()==0)
			{
				stack.pop_back();
				continue;
			}
			if(stack.back()->coordinate=="")
			{//this is a leaf node
				Mx[nodeItr->first].insert(Mx[nodeItr->first].end(),stack.back()->leafNodes.begin(),stack.back()->leafNodes.end());
				stack.pop_back();
				continue;
			}
			bool remove=false;
			unsigned v =0;
			treeNode* stackBack=stack.back();
			stack.pop_back();
			std::map<std::string,unsigned int>::iterator valuePtr=nodeItr->second.find(stackBack->coordinate);
			
			if(valuePtr!=nodeItr->second.end())
			{//this coordinate is represented in this string
				v=valuePtr->second;
				//take all possibilities that =v or higher
			}
			std::map<unsigned,treeNode*>::iterator valueFind=stackBack->subpartitions.find(v);
			if(valueFind==stackBack->subpartitions.end())
			{
				remove=true;
				stackBack->subpartitions[v]=0;//temporarily add
				valueFind=stackBack->subpartitions.find(v);
			}
			while(valueFind!=stackBack->subpartitions.end())
			{
				stack.push_back(valueFind->second);
				if(stackBack->coordinate.compare("A")==0)
				{//this is a special signal, for the node type, in i-degree search.
				//yes this is less generic.
				//I didn't think of a good way to change this to make it generic
					break;
				}
				valueFind++;
			}
			if(remove==true)
			{
				stackBack->subpartitions.erase(v);
			}
		}
		fprintf(output,"%d ",Mx[nodeItr->first].size());
		fprintf(stderr,"Mx=%d\n",Mx[nodeItr->first].size());
	}
	fprintf(output,"\n");
	fflush(output);
}

bool mxReduce(std::map<unsigned int, std::list<unsigned int> >& Mx,MyGraph* fragment, MyGraph* database,FILE* output)
{
	for(std::map<unsigned int, std::list<unsigned int> >::iterator nodeitr=Mx.begin();nodeitr!=Mx.end();nodeitr++)
	{//for each node in the graph
		//let M(x)==y1...yk
		MyNode* node=fragment->getNodeById(nodeitr->first);
		edgelist* edges=node->getOutEdges();
		for(edgelist::iterator edgeItr=edges->begin();
		edgeItr!=edges->end();edgeItr++)
		{//for every x' member E(x)
			std::map<unsigned int, bool> tokens;
			std::list<unsigned int>::iterator MxPrimeItr=Mx[edgeItr->first].begin();
			for(;MxPrimeItr!=Mx[edgeItr->first].end();MxPrimeItr++)
			{//label special every node in M(x')
				tokens[*MxPrimeItr]=true;
			}
			
			std::list<unsigned int> Mnew;
			for(std::list<unsigned int>::iterator MxItr=nodeitr->second.begin();MxItr!=nodeitr->second.end();MxItr++)
			{//for every yi in M(x)
				MyNode* yi=database->getNodeById(*MxItr);
				edgelist* yiEdges=yi->getOutEdges();
				edgelist::iterator r=yiEdges->begin();
				while(r!=yiEdges->end())
				{
					if(tokens.find(r->first)==tokens.end())
					{
						r++;
					}
					else
					{//there was a correct edge found.
						break;
					}
				}
				if(r!=yiEdges->end())
				{
					Mnew.push_back(*MxItr);
				}
			}
			//unlabel(automatic out of scope)
			nodeitr->second=Mnew;//M(x)=Mnew
		}
		//do in edges
		edges=node->getInEdges();
		for(edgelist::iterator edgeItr=edges->begin();
		edgeItr!=edges->end();edgeItr++)
		{//for every x' member E(x)
			std::map<unsigned int, bool> tokens;
			std::list<unsigned int>::iterator MxPrimeItr=Mx[edgeItr->first].begin();
			for(;MxPrimeItr!=Mx[edgeItr->first].end();MxPrimeItr++)
			{//label special every node in M(x')
				tokens[*MxPrimeItr]=true;
			}
			
			std::list<unsigned int> Mnew;
			for(std::list<unsigned int>::iterator MxItr=nodeitr->second.begin();MxItr!=nodeitr->second.end();MxItr++)
			{//for every yi in M(x)
				MyNode* yi=database->getNodeById(*MxItr);
				edgelist* yiEdges=yi->getInEdges();
				edgelist::iterator r=yiEdges->begin();
				while(r!=yiEdges->end())
				{
					if(tokens.find(r->first)==tokens.end())
					{
						r++;
					}
					else
					{//there was a correct edge found.
						break;
					}
				}
				if(r!=yiEdges->end())
				{
					Mnew.push_back(*MxItr);
				}
			}
			//unlabel(automatic out of scope)
			nodeitr->second=Mnew;//M(x)=Mnew
		}
		
	}
}

/*** MAIN ***/
int main(int argc, char ** argv)
{
	time_t start = time(NULL);
	//unsigned int seed = (unsigned int)time(NULL);
	seed = /*1311746957;//*/2357;
	MyGraph * database;
	srand(seed);
	bool testing=true;
	args.T.start("reading args");
	//read in any changes to default arguments
	fprintf(stderr,"accepting arguments\n");
	std::vector<std::string> argP;
	for(unsigned int i=0;i<(unsigned)argc;i++)
	{
		argP.push_back(argv[i]);
	}
	bool success = getArguments(argP, database);
	std::cerr<<"done testing loading database\n";
	if(!success)
	{
		std::cout << "\nincorrect args, see documentation\n";
		return 1;	//exit failure
	}
	std::string timingFileName=args.outputPrefix;
	timingFileName+=".time";
	FILE* timingOutput=fopen(timingFileName.c_str(),"w");
	std::string mxFileName=args.outputPrefix;
	mxFileName+=".MX";
	args.mXoutput=fopen(mxFileName.c_str(),"w");
	std::string mx2FileName=args.outputPrefix;
	mx2FileName+=".MX2";
	args.mXoutput2=fopen(mx2FileName.c_str(),"w");
	positioningArgument p;
	p.method = args.i_degree;//default args.i_degree, assuming no index file is provided
	args.T.stop("reading args");
	args.T.print(stderr);
	std::map<unsigned int, std::map<std::string, unsigned int> > databaseStrings;
	args.T.start("load strings");
	std::map<std::string,bool> uniqueStrings;
	loadStrings(databaseStrings,args.outputPrefix+".idegree",uniqueStrings);
	args.T.stop("load strings");
	args.T.start("buildtree");
	treeIndex index;
	craftTree(databaseStrings,uniqueStrings,index);
	args.T.stop("buildtree");
	
	std::cerr<<"*Database Loaded*\n";
	
	for(unsigned int i=0;i<args.nTests;i++)
	{
		fprintf(stderr,"\nTest %d\n",i+1);
		unsigned temp=lastGraphFile.find_last_of("\\/");
			
		std::string tempStr=lastGraphFile.substr(temp+1);
		std::string fragPrefix="./fragments/"+tempStr+"fragment"+my_itoa(i,10);
		
		std::string fragFile=fragPrefix+".grf";
		MyGraph fragment(fragFile,true,p,true,args.partitionMethod,true,args);
		std::string fragIdegreeFile=fragPrefix+".idegree";
		std::map<unsigned int, std::map<std::string, unsigned int> > fragmentStrings;
		args.T.start("load fragment strings");
		loadStrings(fragmentStrings,fragIdegreeFile,uniqueStrings);//this program currently assumes there are no coordinates that are *not* present within the database. 
		args.T.stop("load fragment strings");
		
		unsigned int mxtotal=0;
		
		args.T.start("Mx Calc");
		std::map<unsigned int, std::list<unsigned int> > Mx;
		mxCalc(index,fragmentStrings,uniqueStrings,args.mXoutput,Mx);
		args.T.stop("Mx Calc");
		for(std::map<unsigned int, std::list<unsigned int> >::iterator mxitr=Mx.begin();mxitr!=Mx.end();mxitr++)
		{
			mxtotal+=mxitr->second.size();
		}
		
		args.T.start("Mx Shrink");
		unsigned int mxNew=mxtotal;
		do{
			mxtotal=mxNew;
			mxNew=0;
			mxReduce(Mx,&fragment,database,args.mXoutput2);
			
			for(std::map<unsigned int, std::list<unsigned int> >::iterator mxitr=Mx.begin();mxitr!=Mx.end();mxitr++)
			{
				mxNew+=mxitr->second.size();
			}
		}while(mxNew<mxtotal);
		
		
		for(std::map<unsigned int, std::list<unsigned int> >::iterator nodeitr=Mx.begin();nodeitr!=Mx.end();nodeitr++)
		{
			fprintf(args.mXoutput2,"%d ",nodeitr->second.size());
			fprintf(stderr,"Mx2=%d\n",nodeitr->second.size());
		}
		fprintf(args.mXoutput2,"\n");
		fflush(args.mXoutput2);
		
		args.T.stop("Mx Shrink");
		
	}
	args.T.print(stderr);
	args.T.print(timingOutput);
	//realTest('o', database, args.nTests);
	fclose(timingOutput);
	fclose(args.mXoutput);
	fclose(args.mXoutput2);
	
	return 0;
}


/*** Function Definitions ***/
/*							 
std::cout << "\nusage: \"evaluate <graph spec> <frag gen spec> <search spec> <similarity spec> #samples\"";
std::cout << "\n\t<Graph spec> :\t1) fileName {1|2|3|<locality.ppg}";
std::cout << "\n\t              \t2) numNodes, numTypes, edgesPerNode";
std::cout << "\n\t<frag gen spec>:\t1) 'o' numNodes distortion";
std::cout << "\n\t                \t2) 's'|'e'|'n' numNodes";
std::cout << "\n\t<search spec>:\t1) 's' single center";
std::cout << "\n\t              \t2) 'p' adjacent pair";
std::cout << "\n\t              \t2) 'm' multiple searches based on an adjacent pair";
std::cout << "\n\t              \t2) 't' tree: central and two adjacent nodes";
std::cout << "\n\t              \t2) 'c' chain: 3 nodes ";
std::cout << "\n\t              \t2) 'q' quadruple: central and three adjacent nodes";
std::cout << "\n\t<similarity spec>:\t1) 'd' ";
std::cout << "\n\t                 :\t2) 'r' \n\n";
*/
bool getFragFile(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.fragFile = argv[nextArg];
	++nextArg;
	return true;
}
bool getFragSize(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.fragSize = atoi(argv[nextArg].c_str());
	++nextArg;
	args.minFragSize = atoi(argv[nextArg].c_str());
	++nextArg;
	return true;
}
bool getGenerateMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.generateMethod = argv[nextArg][0];
	++nextArg;
	switch(args.generateMethod){
		case 'o':
		case 'c':
		case 's':
		case 'n':
		case 'p':
		case 'e':
		case 'k':
			break;
		case 'f':
			if(args.generateMethod=='f')
			{
				if(!getFragFile(argv,nextArg))
					{return false;}
			}
			return true;
			break;
		default:
			std::cout << "\nError: Invalid fragment generation method (" << args.generateMethod << ")";
			return false;
	}
	if(!getFragSize(argv,nextArg))
		{return false;}
	return true;
}

bool getDistortion(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.distortion = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	if(args.distortion < 0 || args.distortion >= .1)
	{
		std::cout << "\nError: Invalid distortion (" << args.distortion << " not in (0.0, 0.1)";
		return false;
	}
	return true;
}
bool getSearchMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.searchMethod = argv[nextArg][0];
	++nextArg;
	switch(args.searchMethod)
	{
		case 's':
		case 'p':
		case 'm':
		case 't':
		case 'c':
		case 'I':
			break;//these are acceptable.
		default:
			if(!isdigit(args.searchMethod) )
			{
				std::cout << "\nError: Invalid search method (\'" << args.searchMethod << "\')";
				return false;
			}
			break;
	}
	return true;
}
bool getSimilarityMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.similarityMethod =argv[nextArg][0];
	++nextArg;
	switch(args.similarityMethod)
	{
		case 'd':
		case 'r':
		case 'f':
		case 'F':
		case 'e':
		case 'E':
			break;//these are acceptable.
		default:
			std::cout << "\nError: Invalid similarity method (" << args.similarityMethod << ")";
			return false;
	}
	return true;
}
bool getNTests(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.nTests = atoi(argv[nextArg].c_str());
	++nextArg;
	if(args.nTests <= 0)
	{
		std::cout << "\nError: Invalid number of tests(" << args.nTests << ")";
		return false;
	}
	return true;
}

bool getAnchorID(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.anchor = atoi(argv[nextArg].c_str());
	++nextArg;
	if(args.anchor <= 0)
	{
		std::cout << "\nError: Invalid anchor(" << args.anchor << ")";
		return false;
	}
	return true;
}

//gets the method to use for measuring distance between i-degrees
bool getDistMethod(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.distMethod =argv[nextArg][0];
	++nextArg;
	switch(args.distMethod)
	{
		case '1'://x^1
		case '2'://x^2
		case 'S'://x^.5
			break;//these are acceptable.
		default:
			std::cout << "\nError: Invalid distMethod (" << args.distMethod << ")";
			return false;
	}
	return true;
}
bool getIDegree(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.i_degree =argv[nextArg][0]-'0';
	++nextArg;
	switch(args.i_degree)
	{
		case 0:
		case 1:
		case 2:
		case 3:
			break;//these are acceptable.
		default:
			std::cout << "\nError: Invalid iDegree(" << args.i_degree << ")";
			return false;
	}
	return true;
}
bool getWeights(std::vector<std::string> argv, unsigned int &nextArg)
{
	args.wI[0] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wI[1] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wI[2] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wO[0] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wO[1] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	args.wO[2] = strtod(argv[nextArg].c_str(), NULL);
	++nextArg;
	return true;
}
bool getArguments(std::vector<std::string> argv, MyGraph *&G)
{
	//set default arguments
	args.outputPrefix="defaultOutput";
	args.useEdgeLabels=false;
	args.searchMethod='9';		
	args.similarityMethod='F';//for use in i-degree algorithm//F=='bipartite matching, in/out edges
	args.i_degree=3;
	args.closeness=0;//not currently internal use
	args.successMethod=0;//currently internal use
	args.distMethod='1';
	args.generateMethod='p';//prandom walk to generate fragments
	args.partitionMethod='o';//'o','i','s','b'//which edges to use
	args.partitionMethod2='i';//describes the algorithm to index with
	args.fragSize=10;//size of the fragment
	args.nTests=1;//number of fragments to test
	args.distortion=0.0;//distortion is used in fragment generation, for some algorithms
	args.wI[0]=args.wI[1]=args.wI[2]=1.0;//default 1//in weights
	args.wO[0]=args.wO[1]=args.wO[2]=1.0;//default 1//out weights
	args.fragFile="";
	args.anchor=0;
	args.minFragSize=1;
	args.seedNum=2357;
	//take in arguments now
	unsigned int nextArg=1;
	positioningArgument p;
	p.method = args.i_degree;
	std::string graphFile=lastGraphFile;
	bool import = false; 
	unsigned int numNode=0, numTypes=0, edgesPerNode=0;
	while(nextArg<argv.size())
	{
		//std::cout<<argv[nextArg]<<std::endl;
		if(argv[nextArg]=="-edgeLabels")
		{++nextArg;
			args.useEdgeLabels=atoi(argv[nextArg].c_str())>0;
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-seed")
		{++nextArg;
			args.seedNum = atoi(argv[nextArg].c_str());
			srand(args.seedNum);
			seed=args.seedNum;
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-f")
		{++nextArg;
			import = true;
			p.method = atoi(argv[nextArg].c_str());
			args.i_degree=atoi(argv[nextArg].c_str());
			p.file = argv[nextArg];
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-g")
		{nextArg++;
			graphFile=argv[nextArg];
			nextArg++;
			continue;
		}
		if(argv[nextArg]=="-randGraph")
		{++nextArg;
			numNode = atoi(argv[nextArg].c_str());
			++nextArg;
			if(numNode == 0 || numNode >= UINT_MAX)
			{
				std::cout << "\nError: invalid size of random graph (" << numNode << ") ";
				return false;
			}
			numTypes = atoi(argv[nextArg].c_str());
			++nextArg;
			if(numTypes == 0 || numTypes >= UINT_MAX)
			{
				std::cout << "\nError: invalid type count of random graph (" << numTypes << ") ";
				return false;
			}
			edgesPerNode = atoi(argv[nextArg].c_str());
			++nextArg;
			if(edgesPerNode == 0 || edgesPerNode >= UINT_MAX)
			{
				std::cout << "\nError: invalid edge count of random graph (" << edgesPerNode << ") ";
				return false;
			}
			G = new MyGraph(true);
			fprintf(stderr,"gen-ing graph\n");
			G->generateGraph(numNode, numTypes, edgesPerNode, args,true);
			continue;
		}
		if(argv[nextArg]=="-gen")
		{++nextArg;
			if(!getGenerateMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-search")
		{++nextArg;
			if(!getSearchMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-sim")
		{++nextArg;
			if(!getSimilarityMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-n")
		{++nextArg;
			if(!getNTests(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-dist")
		{++nextArg;
			if(!getDistMethod(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-distort")
		{++nextArg;
			if(!getDistortion(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-anchor")
		{++nextArg;
			if(!getAnchorID(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-i")
		{++nextArg;
			if(!getIDegree(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-w")
		{++nextArg;
			if(!getWeights(argv,nextArg))
				{return false;}
			continue;
		}
		if(argv[nextArg]=="-partMethod")
		{++nextArg;
			args.partitionMethod=argv[nextArg][0];
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-success")
		{++nextArg;
			args.successMeth=argv[nextArg][0];
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-overlapPercent")
		{++nextArg;
			args.overlapPercent=strtod(argv[nextArg].c_str(), NULL);
			++nextArg;
			continue;
		}
		if(argv[nextArg]=="-maxMaps")
		{++nextArg;
			args.maxMaps=std::atoi(argv[nextArg].c_str());
			++nextArg;
			continue;
		}
		fprintf(stderr,"unknown arg %s\n",argv[nextArg].c_str());
		exit(0);
	}
	fprintf(stderr,"graphfile = %s\n",graphFile.c_str());
	if(graphFile.compare(lastGraphFile)!=0)
	{
		lastGraphFile=graphFile;
		if(import)
		{
			bool plain=true;
			G = new MyGraph(graphFile, true, p, true, args.partitionMethod2, true, args);
		}
		args.graphFile=graphFile;
	}
	if(G->getNumNodes() <= args.fragSize || (args.fragSize == 0 && args.fragFile.size()==0))
	{
		std::cout << "\nError: Invalid fragment size (" << args.fragSize << ") not in range (0, " << G->getNumNodes() << ")";
		return false;
	}
	fprintf(stderr,"done with args\n");
	args.outputPrefix="./output/";
	for(unsigned int i=1;i<argv.size();i++)
	{
		if(argv[i-1].compare("-g")==0)
		{
			args.outputPrefix+=argv[i].substr(13,argv[i].size()-13)+"_";
		}
		else
		{
			args.outputPrefix+=argv[i]+"_";
		}
	}
	fprintf(stderr,"args.outputPrefix= %s\n",args.outputPrefix.c_str());
	return true;
}